%module Sadppac
%{
#define SWIG_FILE_WITH_INIT
#include<linlog++.h>
#include<IrfPac/SaraPac/saradecomp.h>
#include<IrfPac/irfpac.h>
#include<IrfPac/SaraPac/sarapac.h>
#include<IrfPac/SaraPac/AscFmt/ascfmt.h>
#include<IrfPac/Encode/encode.h>

%}

%include "std_string.i"

%include exception.i

%exception {
	try{
		$action
	}
	catch(char const* ex){
		SWIG_exception(SWIG_RuntimeError, ex);
	}
}

#ifdef SWIGPYTHON
%include "SadppacDoc.i"   // python python/doxy2swig.py xml/index.xml SadppacDoc.i
%include "SadppacPy.i"
#endif


/* NOTE THAT YOU MAY NEED TO MODIFY THESE TYPEDEF IF NEEDED */
typedef unsigned char uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int uint32_t;
#ifdef LL64
typedef unsigned long long uint64_t;
#else
typedef unsigned long uint64_t;
#endif

%include "carrays.i"
%array_functions(size_t, sizeT);
%array_functions(uint8_t, uint8arr);

unsigned char linear12tolog8(int);
unsigned char linear16tolog8(int);
unsigned char linear32tolog8(int);
int log8tolinear12(unsigned char);
int log8tolinear16(unsigned char);
int log8tolinear32(unsigned char);

%include "std_vector.i"
%template(vectord) std::vector<double>;

namespace IrfPac {

/*
# class Decompression {
# public:
# 	Decompression();
# 	~Decompression();
# 	uint8_t* decompressionAll(char*, size_t, size_t* len);
# 	double getVersion();
# };
*/

class PacketU8Arr {
public:
	PacketU8Arr(uint8_t* , int);
	PacketU8Arr(const PacketU8Arr &);
	virtual uint8_t* getStream();
	virtual size_t getSize();
	virtual uint8_t* getValues();
	virtual uint8_t getValue(size_t);
	virtual void dumpAll();
        virtual void dumpHexStream();
        virtual std::string hash();
};


namespace SaraPac {

class TimeConversion {
public:
	virtual double obt2utc(double)=0;
};

class TimeConversionOffset : public TimeConversion {
public:
	TimeConversionOffset(double);
	double obt2utc(double);
	static double OFFSET(int);
};

class CenaFastHkPacket : public PacketU8Arr {
public:
	CenaFastHkPacket(uint8_t*, size_t);
	CenaFastHkPacket(uint8_t* stream) : PacketU8Arr(stream, 119);
	bool getFlagHvMain(){return getBitFlag(0);};
	bool getFlagHvSafety(){return getBitFlag(1);};
	bool getFlagHvOnEna(){return getBitFlag(4);};
	bool getFlagHvLvlEna(){return getBitFlag(5);};
	bool getFlagError(){return getBitFlag(7);};
	uint16_t getHvMain();
	uint16_t getHvStartMcp();
	uint16_t getHvStopMcp();
	uint16_t getHvTof();
	uint16_t getHvDef();
	uint16_t getTempIFE();
	uint16_t getTempHVPS();
	uint16_t getPsyncCntr();
	uint16_t getBaseCntr();
	uint16_t getTotalStartCounts();
	uint16_t getTotalStopCounts();
       	uint16_t getTotalCoinCounts();
	uint16_t getSvMonitor(int SvName, int level);
       	static const int SV_WAVE1=0;
       	static const int SV_WAVE2A=1;
       	static const int SV_WAVE2B=2;
       	static const int SV_LENS=3;

        uint8_t getSv1Table();
        uint8_t getSv2Table();
        uint8_t getSv3Table();
        uint8_t getSv4Table();


};


class CenaAccumSclMatx : public PacketU8Arr {
public:
	CenaAccumSclMatx(uint8_t *stream, int n_e, int n_p) : PacketU8Arr(stream, n_e*n_p*4);
	uint8_t getRaw(int, int, int);
	uint32_t get(int, int, int);

	static const int START_COUNTER=0;
	static const int STOP_COUNTER=1;
	static const int COIN_COUNTER=2;
	static const int PADDING=3;

	static const char* getCounterName(int);
	uint32_t collapseEnergyRaw(int, int);
	uint32_t collapsePhaseRaw(int, int);
	uint32_t collapseAllRaw(int);

};


class CenaScienceDataPacket : public PacketU8Arr {
public:
	CenaScienceDataPacket(uint8_t*, size_t);
};

class CenaSciCntrAcc : public CenaScienceDataPacket {
public:
	CenaSciCntrAcc(uint8_t*, int, int, int);
	uint8_t getRaw(int, int, int);
	uint16_t get(int, int, int);
	int getNrEne();
	int getNrPhase();
	int getNrCounter();
	static int START_RING_SECTOR(int,int);
	static int START_RING(int);
	static int START_SECTOR(int);
	static int STOP_PLATE(int);
	static int STOP_MESH();
	static int COIN_SECTOR(int);
};

class CenaSciMassAcc : public CenaScienceDataPacket {
public:
	CenaSciMassAcc(uint8_t*, int, int, int, int);
	uint8_t getRaw(int, int, int, int);
	uint16_t get(int, int, int, int);
	std::vector<double> getCountAsArray();
	int indexOf(int, int, int, int);
	int getNrEne();
	int getNrPhase();
	int getNrCh();
	int getNrChIntr();
	int getNrMass();
};

class CenaSciMassAccE16 : public CenaSciMassAcc {
        int getSvTable();
};

class CenaSciTofAcc : public CenaScienceDataPacket {
public:
	CenaSciTofAcc(uint8_t*, int, int);
	uint8_t getRaw(int, int);
	uint16_t get(int, int);
	int getNrEne();
	int getNrTof();
};

%exception CenaMatrix::decompress(){
	try{
		$action
	}
	catch(int i){
		char msg[5];
		sprintf(msg, "%3d", i);
		SWIG_exception(SWIG_IndexError, msg);
	}
}

%newobject CenaMatrix::decompress();
class CenaMatrix : public PacketU8Arr {
public:
	CenaMatrix(uint8_t *, int);
	CenaMatrix* decompress();
//	CenaScienceDataPacket* interpret(uint8_t, uint8_t, uint8_t, uint8_t, uint8_t);   // Implement

	static const int NOT_COMPRESSED = 120;
	static const int DECOMPRESSION_FAILED =255;
	static const int ILLEGAL_PACKET = 75;
	static const uint32_t COMPRESSED_HEADER = 0xAA55AA55;
	static const uint32_t NON_COMPRESSED_HEADER = 0xAADDAADD;
	
};

%newobject CenaPacket::getFastHkPac();
%newobject CenaPacket::getAccumSclMatx();
%newobject CenaPacket::getCenaMatrix();
%newobject CenaPacket::decompress();
%newobject CenaPacket::getCenaSciCntrAcc();
%newobject CenaPacket::getCenaSciTofAcc();
%newobject CenaPacket::getCenaSciMassAcc();
%newobject CenaPacket::getCenaSciMassAccE16();
%exception CenaPacket::getCenaSciMassAccE16(){
	try{
		$action
	}
	catch(std::runtime_error &e){
		std::cerr << "Exception in Sadppac.  Convert by SWIG." << std::endl;
		SWIG_exception(SWIG_RuntimeError, const_cast<char*>(e.what()));
	}
}
%newobject CenaPacket::newInstance();
class CenaPacket : public PacketU8Arr {
public:
	CenaPacket(uint8_t*, size_t);
	~CenaPacket();
	void dumpAll();

	static const uint32_t CENA_SCI_ID=0xAAAA5555;

	uint32_t getHeadMagicNumber();
	uint64_t getObtTime();
	uint16_t getDpuTime();
	uint16_t getTcountDpu();
	uint16_t getEndIntegCycleTime();
	uint16_t getTcountAcq();
	double getMarkerObt();
	double getMarkerDpu();
	double getObsTimeObt();
	double getObsTimeDpu();
	double getUniversalTime(TimeConversion*);
	double getUtc();
	uint16_t get4sCycleCntr();
	uint16_t getPacketCounter();
	uint16_t getBinningPrm();
	uint8_t getNrCh();
	uint8_t getNrEne();
	uint8_t getNrPhase();
	uint8_t getNrMass();
	uint16_t getNrCycles();
	uint16_t getModes();
	bool isSwimOn();
	bool isCenaOn();
	bool isCenaCompOn();
	uint8_t getTMmode();
	std::string getTMmodeStr();
	uint8_t getSensorMode();
	std::string getSensorModeStr();
	CenaFastHkPacket* getFastHkPac();
	CenaAccumSclMatx* getAccumSclMatx();
	CenaMatrix* getCenaMatrix();
	bool isMatrixCompressed();
	CenaPacket* decompress();

	CenaSciCntrAcc* getCenaSciCntrAcc();
	CenaSciTofAcc* getCenaSciTofAcc();
	CenaSciMassAcc* getCenaSciMassAcc();
	CenaSciMassAccE16* getCenaSciMassAccE16();

	static const uint8_t TMMODE_MASS_ACC=0x02;
	static const uint8_t TMMODE_TOF_ACC=0x04;
	static const uint8_t TMMODE_COUNT_ACC=0x03;
	static const uint8_t TMMODE_NON_PROCESS=0x01;
	static const uint8_t SENSOR_COINCIDENCE=0x00;
	static const uint8_t SENSOR_COUNTER=0x01;

	CenaPacket* newInstance();

	bool isValid();
};

%newobject CenaPacketArray::getPacket(int);
class CenaPacketArray {
public:
	void append(CenaPacket* pac);
	CenaPacket* getPacket(int);
	size_t size();
};

class SwimScienceDataPacket : public PacketU8Arr {
public:
	SwimScienceDataPacket(uint8_t *stream, size_t size);
	virtual void dumpAll(){dumpAll(stdout);};
	virtual void dumpAll(FILE *fp);
};

%newobject SwimSciAcc::reform16x16();
class SwimSciAcc : public SwimScienceDataPacket {
public:
	SwimSciAcc(uint8_t *pac, int nEnergy, int nDefl, int nMass);
	uint8_t getRaw(int iE, int iD, int iC);
	uint16_t get(int iE, int iD, int iC);
	int getNrEne();
	int getNrDefl();
	int getNrMass();
	SwimSciAcc* reform16x16();
};

%newobject SwimMatrix::decompress();
class SwimMatrix : public PacketU8Arr {
public:
	SwimMatrix(uint8_t* packet, int size);
	SwimMatrix* decompress();
//	SwimScienceDataPacket* interpret(uint8_t ne, uint8_t nd, uint8_t nm);
	static const int NOT_COMPRESSED=120;
	static const int DECOMPRESSION_FAILED=255;
	static const int ILLEGAL_PACKET=75;
	static const uint32_t COMPRESSED_HEADER=0xAA55AA55;
	static const uint32_t NON_COMPRESSED_HEADER=0xAADDAADD;
};

class SwimSubcHkPacket : public PacketU8Arr {
public:
	SwimSubcHkPacket(uint8_t* stream) : PacketU8Arr (stream, 14);
	void dumpAll();
	uint16_t getHV ();
	uint16_t getVCeem ();
	uint16_t getVUpDef ();
	uint16_t getVLoDef ();
	uint16_t getVAnaCel ();
	uint16_t getVTofCel ();
	uint16_t getTempTofBrd ();
};

class SwimFastHkPacket : public PacketU8Arr {
public:
	SwimFastHkPacket(uint8_t* stream) : PacketU8Arr (stream, 14);
	void dumpAll();
	uint16_t getHV ();
	uint16_t getVCeem ();
	uint16_t getVUpDef ();
	uint16_t getVLoDef ();
	uint16_t getVAnaCel ();
	uint16_t getVTofCel ();
	uint16_t getTempTofBrd ();
};

%newobject SwimAccumTotMatx::reform16x16();
%exception SwimAccumTotMatx::reform16x16(){
	try{
		$action
	}
	catch(std::runtime_error &e){
		std::cerr << "Exception in Sadppac.  Convert by SWIG." << std::endl;
		SWIG_exception(SWIG_RuntimeError, const_cast<char*>(e.what()));
	}
}
class SwimAccumTotMatx : public PacketU8Arr {
public:
	SwimAccumTotMatx(uint8_t *stream, int n_e, int n_d) : PacketU8Arr (stream, n_e*n_d*5);
	uint8_t getRaw(int, int, int);
	uint32_t get(int, int, int);
	uint32_t collapseEnergyRaw(int, int);
	uint32_t collapseDeflbinRaw(int, int);
	uint32_t collapseAllRaw(int);
	double collapseEnergy(int, int);
	double collapseDeflbin(int, int);
	double collapseAll(int);
	static const char* getCounterName(int);
	static const int START_COUNTER=0;
	static const int STOP_COUNTER=1;
	static const int OVER_COUNTER=2;
	static const int UNDER_COUNTER=3;
	static const int TOF_COUNTER=4;
	SwimAccumTotMatx* reform16x16();
};

%newobject SwimPacket::getFastHkPac();
%newobject SwimPacket::getAccumTotMatx();
%newobject SwimPacket::getSwimMatrix();
%newobject SwimPacket::getSwimSciAcc();
%newobject SwimPacket::decompress();
%newobject SwimPacket::newInstance();
class SwimPacket : public PacketU8Arr{
public:
	SwimPacket(uint8_t*, size_t size);
	void dumpAll();
	uint32_t getHeadMagicNumber();
	uint64_t getObtTime();
	uint16_t getDpuTime();
	uint16_t getTcountDpu();
	uint16_t getEndIntegCycleTime();
	uint16_t getTcountAcq();
//	double getDataAcquisTimeInMsec();
//	double getUniversalTimeInMsec();
	double getMarkerObt();
	double getMarkerDpu();
	double getObsTimeObt();
	double getObsTimeDpu();
	double getUniversalTime(TimeConversion*);
	double getUtc();
	uint16_t get8sCycleCntr();
	uint16_t getPacketCounter();
	uint8_t getPostAccIdx();
	uint8_t getMassBins();
	uint8_t getDeflectionBins();
	uint8_t getEnergyBins();
	uint16_t getNrCycles();
	bool isTdcPowerOn ();
	bool isLatchUpProtectOn ();
	bool isLatchUp ();
	bool isHvpsOscOn ();
	bool isSelectedMemoryBank ();
	bool isModeSampling ();
	bool isSwimOn();
	bool isCenaOn();
	bool isSwimCompOn();
	bool isHealthCheckEvent ();
	SwimFastHkPacket* getFastHkPac();
	SwimAccumTotMatx* getAccumTotMatx();
	SwimMatrix* getSwimMatrix();
	SwimSciAcc* getSwimSciAcc();
	bool isMatrixCompressed();
	bool isValid();
	SwimPacket* decompress();
	SwimPacket* newInstance();

	static const uint32_t SWIM_SCI_ID=0xDDDDEEEE;
};

%newobject SwimPacketArray::getPacket(int);
class SwimPacketArray {
public:
	void append(SwimPacket* pac);
	SwimPacket* getPacket(int);
	size_t size();
};

class SaraPacket : public PacketU8Arr{
public:
	SaraPacket(uint8_t* stream) : PacketU8Arr(stream, 896);
	~SaraPacket();
	uint16_t getCounter();
};

%newobject SaraBufferedDemultiplexor::getCena();
%newobject SaraBufferedDemultiplexor::getSwim();
%newobject SaraBufferedDemultiplexor::getLatestCena();
%newobject SaraBufferedDemultiplexor::getLatestSwim();
class SaraBufferedDemultiplexor {
public:
	SaraBufferedDemultiplexor();
	~SaraBufferedDemultiplexor();
	void appendSaraPacket(SaraPacket*);
	int getNrCena();
	int getNrSwim();
	CenaPacket* getCena();
	SwimPacket* getSwim();
	CenaPacket* getLatestCena();
	SwimPacket* getLatestSwim();
	void flushCena();
	void flushSwim();
	void showBufferStatus();
};

class SaraHkConversion{
protected:
  SaraHkConversion(){}
public:
  static const int TABLE_GROUND_CALIBRATION=1;
  static SaraHkConversion* getConversionFunctions(int id);
  virtual ~SaraHkConversion(){}
  virtual std::string getName();
  virtual double swimHV(uint32_t monitor);  ///< SWIM Main in Volts
  virtual double swimVCeem(uint32_t monitor);   ///< SWIM CEM in Volts
  virtual double swimVUpDef(uint32_t monitor); ///< SWIM Defl. upper in Volts
  virtual double swimVLoDef(uint32_t monitor); ///< SWIM Defl. lower in Volts
  virtual double swimVAnaCel(uint32_t monitor);  ///< SWIM Anyz in Volts
  virtual double swimVTofCel(uint32_t monitor);  ///< SWIM Cell in Volts
  virtual double swimTempTofBrd(uint32_t monitor);  ///< SWIM Temp in degC

  virtual double cenaHvMain(uint32_t monitor);  ///< CENA HV_Ref in Volts
  virtual double cenaHvStartMcp(uint32_t monitor);  ///< CENA HV_StartMCP in Volts
  virtual double cenaHvStopMcp(uint32_t monitor);  ///< CENA HV_StopMCP in Volts
  virtual double cenaHvDef(uint32_t monitor);  ///< CENA HV_Def in Volts
  virtual double cenaHvTof(uint32_t monitor);  ///< CENA HV_Tef in Volts
  virtual double cenaSvWave1(uint32_t monitor);  ///< CENA SV_WAVE1 in Volts
  virtual double cenaSvWave2a(uint32_t monitor);  ///< CENA SV_WAVE2A in Volts
  virtual double cenaSvWave2b(uint32_t monitor);  ///< CENA SV_WAVE2B in Volts
  virtual double cenaSvLens(uint32_t monitor);  ///< CENA HV_LENS in Volts
  virtual double cenaTempHVPS(uint32_t monitor);  ///< CENA Temp_HVPS in degC
  virtual double cenaTempIFE(uint32_t monitor);  ///< CENA Temp_IFE in degC
};

class SaraHkConversionGrndCal : public SaraHkConversion{
public:
  SaraHkConversionGrndCal();
  virtual std::string getName();
  virtual double swimHV(uint32_t monitor);  ///< SWIM Main in Volts
  virtual double swimVCeem(uint32_t monitor);   ///< SWIM CEM in Volts
  virtual double swimVUpDef(uint32_t monitor); ///< SWIM Defl. upper in Volts
  virtual double swimVLoDef(uint32_t monitor); ///< SWIM Defl. lower in Volts
  virtual double swimVAnaCel(uint32_t monitor);  ///< SWIM Anyz in Volts
  virtual double swimVTofCel(uint32_t monitor);  ///< SWIM Cell in Volts
  virtual double swimTempTofBrd(uint32_t monitor);  ///< SWIM Temp in degC
  virtual double cenaHvMain(uint32_t monitor);  ///< CENA HV_Ref in Volts
  virtual double cenaHvStartMcp(uint32_t monitor);  ///< CENA HV_StartMCP in Volts
  virtual double cenaHvStopMcp(uint32_t monitor);  ///< CENA HV_StopMCP in Volts
  virtual double cenaHvDef(uint32_t monitor);  ///< CENA HV_Def in Volts
  virtual double cenaHvTof(uint32_t monitor);  ///< CENA HV_Tef in Volts
  virtual double cenaSvWave1(uint32_t monitor);  ///< CENA SV_WAVE1 in Volts
  virtual double cenaSvWave2a(uint32_t monitor);  ///< CENA SV_WAVE2A in Volts
  virtual double cenaSvWave2b(uint32_t monitor);  ///< CENA SV_WAVE2B in Volts
  virtual double cenaSvLens(uint32_t monitor);  ///< CENA HV_LENS in Volts
  virtual double cenaTempHVPS(uint32_t monitor);  ///< CENA Temp_HVPS in degC
  virtual double cenaTempIFE(uint32_t monitor);  ///< CENA Temp_IFE in degC
};

namespace AscFmt {

%newobject CenaBmuReader::getArray();
class CenaBmuReader{
public:
      void readFile(char* fname);
      CenaPacketArray* getArray();
};

%newobject SwimBmuReader::getArray();
class SwimBmuReader{
public:
      void readFile(char* fname);
      SwimPacketArray* getArray();
};

%newobject SaraAscFile::getSaraPacket();
class SaraAscFile{
public:

	SaraAscFile(char*);
	~SaraAscFile();
	bool openStream();
	bool closeStream();
	bool eof();
	void clearerror();
	SaraPacket* getSaraPacket();
	void setTimeout(int);
};

class TimeFile {
public:
	void readFile(FILE* fp);
};

class TimeConversionTimeFile : public TimeConversion {
public:
	TimeConversionTimeFile(TimeFile*);
	double obt2utc(double);
};
class TimeConversionTimeFileExtpol : public TimeConversion {
public:
	TimeConversionTimeFileExtpol(TimeFile*);
	double obt2utc(double);
};

}  // End of namespace AscFmt

}  // End of namespace SaraPac

}  // End of namespace IrfPac

%include<IrfPac/irfpac.h>
%include<IrfPac/SaraPac/sarapac.h>
%include<IrfPac/SaraPac/AscFmt/ascfmt.h>
%include<IrfPac/Encode/encode.h>
