package jsadppac.ascfmt;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;

/**
 * Simulator of SARA data streaming.
 * As we do not know how the data stream is input, I currently develop
 * the data stream from a single file.  However, the way is not good
 * because the data is transfered with interval of 250 msec.
 * 
 * @author futaana
 *
 */

public class SaraAscFileSimulator implements Runnable{

	private int waitmsec=250;    // Wait between packets.
	private int waitlinemsec=0;  // Wait between lines... for debug purpose.
	private BufferedReader reader=null;
	private PrintStream writer=null;
	
	static private BufferedReader stdin=null;
	
	public void setWaitPacket(int delayMsec){
		waitmsec=delayMsec;
		setWaitLine(waitlinemsec);
	}
	public void setWaitLine(int delayMsec){
		if(delayMsec *30 > waitmsec){
			System.err.println("Too long delay between lines.  Set to "+(waitmsec/50));
			delayMsec=waitmsec/50;
		}
		
		waitlinemsec=delayMsec;
	}

	public SaraAscFileSimulator(File inStream, File outStream) throws FileNotFoundException {
		reader = new BufferedReader(new InputStreamReader(new FileInputStream(inStream)));
		writer = new PrintStream(new FileOutputStream(outStream));
	}
	
	public SaraAscFileSimulator(File inStream) throws FileNotFoundException {
		reader = new BufferedReader(new InputStreamReader(new FileInputStream(inStream)));
		writer = System.out;
	}
	
	public SaraAscFileSimulator() {
		if(stdin==null){
			stdin=new BufferedReader(new InputStreamReader(System.in));
		}
		reader = stdin;
		writer = System.out;
	}
	
	private int counter=0;
	private void processing() throws IOException{
		String buffer=null;
		
		try {
			buffer=reader.readLine();

			if(buffer==null){
				Thread.sleep(waitmsec);
			}
			else{
				counter++;
//				System.err.println("TEST "+counter+":"+System.currentTimeMillis());
//				System.err.println(buffer);
				if(buffer.startsWith("********")){
					Thread.sleep(waitmsec-waitlinemsec*30);
				}
				writer.println(buffer);
				writer.flush();
				Thread.sleep(waitlinemsec);
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

	}
	
	public void close() throws IOException{
		if(reader!=null && reader != stdin)reader.close();
		if(writer!=null && writer!=System.out)writer.close();
	}
	
	// Below is the 'template' of the thread control

	private Thread simulator=null;

	/**
	 * Start the simulator.
	 */
	public void start() {
		simulator=new Thread(this);
		simulator.start();
	}
	
	/**
	 * Stop the simulator.
	 */
	public void stop() {
		simulator=null;
	}
	
	public void run() throws RuntimeException {
		Thread thisThread=Thread.currentThread();
		while(thisThread==simulator){
			try {
				processing();
			} catch (IOException e) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}
		}
		
	}
	
	
}
