import sys
import time
import calendar

from pylab import *
import matplotlib.pyplot as plt

import Sadppac
import pyDemultiplex

def enestep32():
	return (106, 106, 132, 132, 165, 165, 206, 206, 258, 258, 322, 322, 403, 403, 503, 503, 629, 629, 786, 786, 983, 983, 1229, 1229, 1536, 1536, 1920, 1920, 2400, 2400, 3000, 3000) 


def mplSwimEnergySpectrum(filename, index=0, vdf=False):
	"""
	Sample function of plotting energy spectrum (count rate or velocity distribution function) for SWIM at specified time.

	The matplotlib is used for the plotting.
	If vdf=True is specified, the velocity distribution function is plotted.
	"""

	print "vdf=", vdf

	cena,swim=pyDemultiplex.pyDemultiplex(filename)

	ncena=len(cena)
	nswim=len(swim)
	print "CENA data nr: ", ncena
	print "SWIM data nr: ", nswim

	idx=index   ### The data number
	s = swim[idx]

	epoch = calendar.timegm( (2008,12,23,23,57,3) )

#	print 'Ellpased time from the first data', s.getUniversalTimeInMsec()-swim[0].getUniversalTimeInMsec()
	print 'Time of observations', s.getUniversalTimeInMsec()
	print 'Time of observations (From EPOCH)', time.gmtime(s.getUniversalTimeInMsec()/1000.)
	print 'Time of observations (UT)', time.gmtime(s.getUniversalTimeInMsec()/1000.+epoch)

	cntr=zeros(32)  ### 32 energy step is assumed.

	# Obtain the accumulation total matrix
	st = s.getAccumTotMatx()

	# Collapse over all deflection bin.
	for i in range(s.getEnergyBins()):
		cntr[i] += st.collapseDeflbin(i, 0)    # 0 is the START

	fig=figure()
	ax=fig.add_subplot(111)

	ene=enestep32()   # Energy step in eV
	vel=[sqrt(iene)*13.8 for iene in ene]   # Corrensponding velocity in km/s

	if vdf:
		print "VDF"
		cntr=cntr/(array(ene)**2)
		ax.plot(vel, cntr, 'ro')
		ax.set_xscale('linear')
		ax.set_yscale('log')
		ax.set_ylabel('Velocity Distrubution [Arbitrary Unit]')
		ax.plot(vel, ones(32)/(array(ene)**2), '-')
		ax.set_xlim(100,800)
	else:
		print "Cntr-Ene"
		ax.plot(ene, cntr, 'ro')
		ax.set_xscale('log')
		ax.set_yscale('linear')
		ax.set_xlabel('Energy [eV]')
		ax.set_ylabel('Count rate [cnt/samp]')
		ax.set_xlim(100,3100)

	fig.show()

	fig.canvas.draw()

if __name__ == '__main__':


	### First of all, obtain the datafile.

	if len(sys.argv) < 3 :
		print >> sys.stderr, "USAGE:  python    ", sys.argv[0], "filename_sci.dat   index_number"
		sys.exit(-1)
	else:
		filename=sys.argv[1]
		index=int(sys.argv[2])

	mplSwimEnergySpectrum(filename, index=index)

