#!/usr/bin/env python

'''
setup.py for python-binding of Sadppac with SWIG
'''

import os
import shutil
import sys
import glob
import subprocess
from fnmatch import fnmatch

workdir='src'

def copy_resources():
    ''' Copy cpp files and header file into the working directory.
    '''
    os.makedirs(workdir)
    swigfiles=['Sadppac.i', 'SadppacPy.i', 'SadppacDoc.i']
    for f in swigfiles:
        shutil.copy(os.path.join('..', f), workdir)

    # Specify CPP source directory.
    cppdir=['IrfPac',
	os.path.join('IrfPac', 'SaraPac'),
	os.path.join('IrfPac', 'SaraPac', 'AscFmt'),
	os.path.join('IrfPac', 'Encode')]

    for d in cppdir:
        src = os.path.join('..', 'src', d)

        dst = os.path.join(workdir, d)
        if os.path.exists(dst):
            shutil.rmtree(dst)
        os.makedirs(dst)

        # Header file
        h_files = glob.glob( os.path.join(src, '*.h') )
        for h_file in h_files:
            if fnmatch(h_file, '*_test.h') or fnmatch(h_file, '*_sample.h') or fnmatch(h_file, '*Sample.h'):
                print('Skipped: ', h_file, dst)
                continue
            print('Copying: ', h_file, dst)
            shutil.copy(h_file, dst)

        # Cpp file
        cpp_files = glob.glob( os.path.join(src, '*.cpp') )
        for cpp_file in cpp_files:
            if fnmatch(cpp_file, '*_test.cpp') or fnmatch(cpp_file, '*_sample*.cpp') or fnmatch(cpp_file, '*App.cpp') or fnmatch(cpp_file, '*/AllTest*.cpp'):
                print('Skipped: ', cpp_file, dst)
                continue
            print('Copying: ', cpp_file, workdir)
            shutil.copy(cpp_file, workdir)

def copy_liblinlog():
    '''
    copy_liblinlog:
            for file in linlog.h linlog++.h linlog++.i linlog.c; do \
                    cp $(LIBLINLOGSRC)/$${file} src; \
            done
    '''
    srcs=['linlog.h', 'linlog++.h', 'linlog++.i', 'linlog.c']
    for src in srcs:
        file = os.path.join('../vendor/liblinlog/current', src)
        print('Copying: ', file, workdir)
        shutil.copy(file, workdir)

def copy_libricesrc():
    '''
    copy_libricesrc:
            for file in saradecomp.h saradecomp.i; do \
                    cp $(LIBRICESRC)/$${file} src/IrfPac/SaraPac; \
            done
    '''
    srcs=['saradecomp.h', 'saradecomp.i']
    dst=os.path.join(workdir, 'IrfPac', 'SaraPac')
    for src in srcs:
        file = os.path.join('../vendor/librice/current', src)
        print('Copying: ', file, dst)
        shutil.copy(file, dst)

if os.path.exists(workdir):
    shutil.rmtree(workdir)


def make_swig():
    cwd=os.getcwd()
    os.chdir(workdir)

    swig_opts=''
    if os.uname()[0] == 'Darwin':
        print('In Darwin one should set -DLL64 for SWIG')
        swig_opts=swig_opts+"-DLL64"

    cmd='swig %s -c++ -python Sadppac.i'%(swig_opts,)
    print('Running: %s'%cmd)
    try:
        val = subprocess.check_output(cmd, shell=True)
    except subprocess.CalledProcessError as e:
        print('Failed running SWIG.')
        print('  Return code = {}'.format(e.returncode))
        raise

    os.chdir(cwd)

def get_source_file(workdir):
    cwd=os.getcwd()
    os.chdir(workdir)

    cfiles=glob.glob('*.c')
    cppfiles=glob.glob('*.cpp')
    cxxfiles=glob.glob('*.cxx')

    os.chdir(cwd)

    return cfiles+cppfiles+cxxfiles



#### Below is the main script to be given to distutils

from setuptools import setup,Extension
#from distutils.core import setup,Extension

copy_resources()
copy_liblinlog()
copy_libricesrc()
make_swig()

print(get_source_file(workdir))

sadppac_module = Extension('_Sadppac',
	sources=get_source_file(workdir),
        include_dirs=['.'],
        libraries=['crypto'],
	)

cwd=os.getcwd()
os.chdir(workdir)

setup(name='Sadppac',
	version='0.4',
	author='Futaana',
	description='Sadppac, a python binding for libsadppac',
	ext_modules=[sadppac_module],
	py_modules=['Sadppac'],
	)

os.chdir(cwd)
