import sys
import time
import calendar

from pylab import *
import matplotlib.pyplot as plt

import Sadppac
import pyDemultiplex

def enestep32():
	return (106, 106, 132, 132, 165, 165, 206, 206, 258, 258, 322, 322, 403, 403, 503, 503, 629, 629, 786, 786, 983, 983, 1229, 1229, 1536, 1536, 1920, 1920, 2400, 2400, 3000, 3000) 

def enestep16():
	ene=enestep32()
	ene16=[ene[i*2]  for i in range(16)]
	return ene16

def mplSwimEnergySpectrum090108(filename, vdf=False, swim=[], offset=0, accum=8, rebin=False):
	"""
	Sample function of plotting energy spectrum (count rate or velocity distribution function) for SWIM.

	The matplotlib is used for the plotting.
	If vdf=True is specified, the velocity distribution function is plotted.
	"""

	print "vdf=", vdf

	if len(swim) == 0:
		cena,swim=pyDemultiplex.pyDemultiplex(filename)

#	ncena=len(cena)
	nswim=len(swim)
#	print "CENA data nr: ", ncena
	print "SWIM data nr: ", nswim

#	idx=index   ### The data number
#	s = swim[idx]

	epoch = calendar.timegm( (2008,12,23,23,57,3) )


	fig=figure()
	id0=offset
	id1=id0+accum-1

	if rebin:
		ene=enestep16()
	else:
		ene=enestep32()   # Energy step in eV
	vel=[sqrt(iene)*13.8 for iene in ene]   # Corrensponding velocity in km/s

	while id1<len(swim):
		print "INDEX RANGE ", id0, id1

		# Time
		t0=time.gmtime(swim[id0].getUniversalTimeInMsec()/1000.+epoch)
		t1=time.gmtime(swim[id1].getUniversalTimeInMsec()/1000.+epoch)
		print "Time of obs:", t0, t1

		filename=time.strftime("espec-%y%m%d-%H%M%S", t0)
		print "FILE NAME",  filename
		fig.clear()
		ax=fig.add_subplot(111)

		cntr0=zeros(32)  ### 32 energy step is assumed.
		cntr1=zeros(32)  ### 32 energy step is assumed.
		cntr2=zeros(32)  ### 32 energy step is assumed.

		# adding the data for time axis
		for id in range(id0, id1+1):
			print id;
			st = swim[id].getAccumTotMatx()
			for i in range(swim[id].getEnergyBins()):
				cntr0[i] += st.collapseDeflbin(i, 0)    # 0 is the START
		cntr0=cntr0/accum

		if rebin:
			cntr=cntr0
			cntr0=zeros(16)
			for i in range(16):
				cntr0[i]=cntr[i*2]+cntr[i*2+1]

		# adding the data for time axis
		for id in range(id0, id1+1):
			print id;
			st = swim[id].getAccumTotMatx()
			for i in range(swim[id].getEnergyBins()):
				cntr1[i] += st.collapseDeflbin(i, 1)    # 1 is the STOP
		cntr1=cntr1/accum

		if rebin:
			cntr=cntr1
			cntr1=zeros(16)
			for i in range(16):
				cntr1[i]=cntr[i*2]+cntr[i*2+1]

		# adding the data for time axis
		for id in range(id0, id1+1):
			print id;
			st = swim[id].getAccumTotMatx()
			for i in range(swim[id].getEnergyBins()):
				cntr2[i] += st.collapseDeflbin(i, 4)    # 4 is the COIN
		cntr2=cntr2/accum

		if rebin:
			cntr=cntr2
			cntr2=zeros(16)
			for i in range(16):
				cntr2[i]=cntr[i*2]+cntr[i*2+1]


		if vdf:
			print "VDF"
			cntr0=cntr0/(array(ene)**2)
			cntr1=cntr1/(array(ene)**2)
			cntr2=cntr2/(array(ene)**2)
			ax.plot(vel, cntr0, 'ro')
#			ax.plot(vel, cntr1, 'bo')
#			ax.plot(vel, cntr2, 'go')
			ax.set_xscale('linear')
			ax.set_yscale('log')
			ax.set_ylabel('Velocity Distrubution [Arbitrary Unit]')
			ax.plot(vel, ones(len(ene))/(array(ene)**2), '-')
			ax.set_xlim(100,800)
		else:
			print "Cntr-Ene"
			ax.plot(ene, cntr0, 'ro')
#			ax.plot(ene, cntr1, 'bo')
#			ax.plot(ene, cntr2, 'go')
			ax.set_xscale('log')
			ax.set_yscale('linear')
			ax.set_xlabel('Energy [eV]')
			ax.set_ylabel('Count rate [cnt/samp]')
			ax.set_xlim(100,3100)
			ax.set_ybound(lower=0)


		title=time.strftime("TOTAL MATRIX %y-%m-%dT%H:%M:%S", t0)+" til "+time.strftime("%H:%M:%S", t1)
		ax.set_title(title)

		# Legend
#		ax.legend(("START", "STOP", "COIN"), "upper right")

		# Saving the file.
		savefig(filename)

		id0=id0+accum
		id1=id1+accum

#		fig.show()
#		fig.canvas.draw()
	return swim

if __name__ == '__main__':


	### First of all, obtain the datafile.

	if len(sys.argv) < 2 :
		print >> sys.stderr, "USAGE:  python    ", sys.argv[0], "filename_sci.dat"
		sys.exit(-1)
	else:
		filename=sys.argv[1]

	swim=mplSwimEnergySpectrum090108(filename)

