/**@file CenaCompressionErrorList.cpp

This is a sample to dump decompression error list
by a searching over the CENA orbit separated data.

USAGE:
	Change the pathprefix according to the system.
	cmake, and make.

	CenaCompressionErrorList.exe 2> /dev/null > errorlog.txt

*/
#include<cstdlib>
#include<iostream>
#include<IrfPac/irfpac.h>
#include<IrfPac/SaraPac/sarapac.h>
#include<IrfPac/SaraPac/AscFmt/ascfmt.h>

using namespace std;
using namespace IrfPac;
using namespace IrfPac::SaraPac;
using namespace IrfPac::SaraPac::AscFmt;

int main(int argc, char* argv[]){

	char* pathprefix = "/irf/sara/saradds/archive/bmu_orbit/";

	for(int orbnr=1000; orbnr<3500; orbnr++){

		char fnamn[500];
		sprintf(fnamn, "%sorb_%04d/orb_%04d.cena", pathprefix, orbnr, orbnr);
		fprintf(stderr, "Opening file %s\n", fnamn);

		try{
			CenaBmuReader sr;        // Instance the CENA BMU reader class
			sr.readFile(fnamn);    // Specify the filename to BMU reader via readFile method.

			CenaPacketArray*  cenaPac = sr.getArray();	// getArray method will return the pointer to the CenaPacketArry, which is the storage of CenaArray.

			for(int idx=0; idx<cenaPac->size(); idx++){

				CenaPacket* packet = cenaPac->getPacket(idx);  // Get the CENA packet.
				double utc = packet->getUtc();
				int cntr = (int) packet->get4sCycleCntr();

				CenaPacket* packet2 = packet->decompress();

				if(packet2 == NULL){
					struct tm *strtm;
					time_t tt = (time_t) utc;
					strtm = gmtime(&tt);
					char str[300];
					strftime(str, 299, "%FT%T", strtm);
					fprintf(stdout, "Decompression error at orbit %d counter 0x%04x (%s)\n", orbnr, cntr, str);
				}
				else{
					delete packet2;
				}
	
				delete packet;
			}

			delete cenaPac;

		}
		catch(runtime_error e){
			fprintf(stderr, "No data on orbit %d\n", orbnr);
		}

	}

	return 0;
}

