/**@file CenaSciencePackage.cpp
   @brief The program converts CENA BMU-data to a CENA package.

   CENA BMU-data file is converted to a CENA package.
   <pre>
   CenaSciencePackage [options] input_file.cena
       options:  -h  Show help and exit
                 -o pkgname   Specify output package name.
   </pre>

   This is the same output as the program SaraDemultiplex, so
   one can use idlscilib IDL program.
*/

#include<iostream>
#include<cstdlib>
#include<libgen.h>
#include<sys/stat.h>
#include<getopt.h>

#include<stdexcept>

#include<IrfPac/irfpac.h>
#include<IrfPac/SaraPac/sarapac.h>
//#include<IrfPac/SaraPac/AscFmt/ascfmt.h>

using namespace std;
using namespace IrfPac;
using namespace IrfPac::SaraPac;

#include"IrfPac/SaraPac/AscFmt/ascfmt.h"
using namespace IrfPac::SaraPac::AscFmt;

void show_help(char* prog){
	fprintf(stderr, "USAGE:   %s  [opts]  SWIM_DATA.swim\n", prog);
        fprintf(stderr, "options:  -h  Show help and exit\n");
        fprintf(stderr, "          -o pkgname   Specify output package name.\n");
}


//
static std::string outputpkg("");
static std::string inputbmu;

void setopt(int argc, char** argv){
	int result;
	while( (result = getopt(argc, argv, "ho:")) != -1 ){
		switch (result){
		case 'h':
			show_help(argv[0]);
			exit(0);
		case 'o':
			char pkgname[256];
			sprintf(pkgname, "%s", optarg);
			fprintf(stderr, "Output package name set to %s\n", pkgname);
			outputpkg=std::string(pkgname);
			break;
		}
	}

	if(argc==optind){
		show_help(argv[0]);
		exit(-1);
	}
	inputbmu=std::string(argv[optind]);
}

int main(int argc, char** argv){
	// Set option.
	setopt(argc, argv);

	// Output package name selection
	if(outputpkg.empty()){
		char dirout[256];
		sprintf(dirout, "%s.sarapkg", basename((char*)inputbmu.c_str()));
		outputpkg=std::string(dirout);
	}
	fprintf(stderr, "OUTPUT PKG= %s \n", outputpkg.c_str());

	CenaBmuReader fp=CenaBmuReader();
	fp.readFile((char*)inputbmu.c_str());
	CenaPacketArray *arr=fp.getArray();


	CenaSciencePackage cpac;
	cpac.appendCenaPacketArray(arr);
	cpac.dump((char*)outputpkg.c_str());

	delete arr;

	return 0;
}



