#include <cstdlib>
#include<iostream>
#include<BitBuffer.h>


using namespace IrfPac::Encode;

int main(void){

	// This is a tutrial how to make BitBuffer class.
	
	
	// First, you must instance the class.
	BitBuffer buf;
	
	// You can also instance with new parameter.
	// BitBuffer* p_buf=new BitBuffer();
	
	// Let's look at the contents.
	std::cout << "--- No buffering ---" << std::endl;
	buf.showAllBuffer(stdout);
	std::cout << "--------------------" << std::endl;

	// Try appending a bit stream
	// First, appennd '101'.
	std::cout << "--- Append 101 ---" << std::endl;
	uint8_t value=0x05;   // 101 is 0x05 in HEX
	buf.appendLBits(value, 3);   // 3 least significant bit for 0x05 (i.e. value & '00000111') is appended.
	std::cout << "--- Buffer ---" << std::endl;
	buf.showAllBuffer(stdout);
	std::cout << "--------------" << std::endl;
	
	// Second, append 2 most significant bit of 0xe3.  i.e. (value=0x11100011) & '11000000' is appended.
	std::cout << "--- Append 2 MSB of 0xe3 ---" << std::endl;
	value=0xe3;
	buf.appendMBits(value, 2);
	std::cout << "--- Buffer ---" << std::endl;
	buf.showAllBuffer(stdout);
	std::cout << "--------------" << std::endl;
	// Now we know how to append the bit stream.
	
	// You next append 5 bit value of '00110'
	// Buffer should be 101 11 00110, ie 1011 1001 10 = 0xb9 '10'
	std::cout << "--- Append 00110 ---" << std::endl;
	value=0x06;
	buf.appendLBits(value, 5);
	std::cout << "--- Buffer should be 0xb9 '10' ---" << std::endl;	
	std::cout << "--- Buffer ---" << std::endl;
	buf.showAllBuffer(stdout);
	std::cout << "--------------" << std::endl;
	
	// Before finishing the buffering, you have to fill in the 'padding' value.
	// 
	std::cout << "--- Check the buffer is at the boundary. ---" << std::endl;
	std::cout << "Results should be false? " << buf.atBoundary() << std::endl;
	std::cout << "The buffer is " << buf.getBitSize() << " bits" << std::endl;
	std::cout << "--- Here you make padding.---" << std::endl;
	buf.paddingToBoundary(1);   // Padding with 1.
	std::cout << "The buffer is " << buf.getBitSize() << " bits" << std::endl;
	// Then, the buffer stream should be 0xb9 '10111111' = 0xb9 0xbf
	std::cout << "--- Buffer should be 0xb9 0xbf '' ---" << std::endl;	
	std::cout << "--- Buffer ---" << std::endl;
	buf.showAllBuffer(stdout);
	std::cout << "--------------" << std::endl;
	std::cout << "--- Check the buffer is at the boundary. ---" << std::endl;
	std::cout << "Results should be true? " << buf.atBoundary() << std::endl;

	// Obtain the bit stream
	uint8_t* stream=buf.getBuffer();
	if(stream[0] != 0xb9)abort();
	std::cout << "[0] should be 0xb9=185? " << (int)stream[0] << std::endl;
	if(stream[1] != 0xbf)abort();
	std::cout << "[1] should be 0xbf=191? " << (int)stream[1] << std::endl;

	// Write to a file
	FILE* fp=fopen("tmp-delete-me-please.out", "wb");
	if(fp==NULL){
		std::cerr << "Failed to write file" << std::endl;
		return -5;
	}
	buf.write(fp);
	fclose(fp);
	
	
	
	// At the end, please delete the contents.
	delete[] stream;
	
	return 0;
}

