#include"IrfPac/packetbytearr.h"

using namespace std;
using namespace IrfPac;

#include"cppunit/extensions/HelperMacros.h"

class PacketByteArrTest : public CPPUNIT_NS::TestFixture {

			CPPUNIT_TEST_SUITE( PacketByteArrTest);
			CPPUNIT_TEST( testGetSize);
			CPPUNIT_TEST( testStreamCopy);
			CPPUNIT_TEST( testStreamIdentity);
			CPPUNIT_TEST( testWith8Arr);
			CPPUNIT_TEST_SUITE_END();

protected:

		void testGetSize(){
			cout << "Test for getSize" << endl;

			// Prepare the original stream
			uint8_t stream[35];
			for(int i=0; i<35; i++){
				stream[i] = i*15;
			}
	
			// Create packet
		 	PacketByteArr pac(stream, 35);

			// Get the size of packet, and compare to the original size.
			CPPUNIT_ASSERT_EQUAL(35, (int) pac.getSize());
		};
		
		void testStreamCopy(){
			cout << "Test for contents of the stream" << endl;
			
			// Prepare the original stream
			uint8_t stream[35];
			for(int i=0; i<35; i++){
				stream[i] = i*12+6;
			}

			// Create packet
		 	PacketByteArr pac(stream, 35);
			
			// Obtain the byte stream
			uint8_t* pacStream=pac.getStream();

			// Assert the consistency to the original stream
			for(int i=0; i<35; i++){
				CPPUNIT_ASSERT_EQUAL((int)stream[i], (int)pacStream[i]);
			}

			// Explicitly delete the obtained stream
			delete[] pacStream;
		}

		void testStreamIdentity(){
			cout << "Test for stream identity" << endl;
			// Prepare the original stream
			uint8_t stream[35];
			for(int i=0; i<35; i++){
				stream[i] = i*14+6;
			}
			// Create packet
		 	PacketByteArr pac(stream, 35);
			// Obtain the byte stream
			uint8_t* pacStream=pac.getStream();

			// Modify the obtained stream
			for(int i=0; i<35; i++){
				pacStream[i] = i*14 + 16;
			}

			// Obtain another byte stream
			uint8_t* pacStream2=pac.getStream();

			// Assert if the changes in the obtained stream
			// will affect the original packet.
			for(int i=0; i<35; i++){
				CPPUNIT_ASSERT(pacStream2[i]!=pacStream[i]);
			}

			// Release resources
			delete[] pacStream;
			delete[] pacStream2;
		}

		void testWith8Arr(){
			cout << "packetbytearr - PacketU8Arr consistency check" << endl;

			// Prepare the original stream
			uint8_t stream[35];
			for(int i=0; i<35; i++){
				stream[i] = i*14+6;
			}

			// Instance packetbytearr
			PacketByteArr pbarr=PacketByteArr(stream, 35);

			// Instance PacketU8Arr
			PacketU8Arr p8arr=PacketU8Arr(stream, 35);

			CPPUNIT_ASSERT_EQUAL((int)pbarr.getSize(), (int)p8arr.getSize());

			pbarr.dumpHexStream(stdout);
			p8arr.dumpHexStream(stdout);

			uint8_t* st_b=pbarr.getStream();
			uint8_t* st_8=p8arr.getStream();

			for(int i=0; i<35; i++){
				CPPUNIT_ASSERT_EQUAL((int)st_b[i], (int)st_8[i]);
			}

		}
		

};

CPPUNIT_TEST_SUITE_REGISTRATION (PacketByteArrTest);

