#include<iostream>
using namespace std;

#include<cppunit/extensions/HelperMacros.h>

#include"IrfPac/irfpac.h"
#include"IrfPac/packetint32arr.h"

using namespace IrfPac;

class PacketInt32Arr_test : public CPPUNIT_NS::TestFixture {

	CPPUNIT_TEST_SUITE(PacketInt32Arr_test);

            CPPUNIT_TEST(testGetSize);
            CPPUNIT_TEST(testContents);
            CPPUNIT_TEST(testStream);
            CPPUNIT_TEST(testPrint);
            CPPUNIT_TEST(testConsistentWithPacket32Arr);

	    CPPUNIT_TEST_SUITE_END();

protected:

		void testGetSize(){
			cout << "Test for getSize" << endl;

			// Prepare the original stream
			uint32_t stream[35];
			for(int i=0; i<35; i++){
				stream[i] = i*15;
			}
	
			// Create packet
		 	PacketInt32Arr pac(stream, 35);

			// Get the size of packet, and compare to the original size.
			CPPUNIT_ASSERT_EQUAL(35*4, (int) pac.getSize());
			CPPUNIT_ASSERT_EQUAL(35, (int) pac.getIndexSize());
		};

        void testContents(){
		cout << "Test for Contents" << endl;

		// Prepare the original stream
		uint32_t stream[94];
		for(int i=0; i<94; i++){
			stream[i] = i+(1<<(i%32));
		}
	
		// Create packet
	 	PacketInt32Arr pac(stream, 94);

		// Obtain the stream
		uint32_t* pacStream=pac.getValues();

		// Assert the consistency to the original stream
		for(int i=0; i<94; i++){
			CPPUNIT_ASSERT_EQUAL((int)stream[i], (int)pacStream[i]);
			CPPUNIT_ASSERT_EQUAL((int)stream[i], (int)pac.getValue(i));
		}

		// Explicitly delete the obtained stream
		delete[] pacStream;
        }

	void testStream(){
		cout << "Test for Stream" << endl;

		// Prepare the original stream
		uint32_t stream[94];
		for(int i=0; i<94; i++){
			stream[i] = i+(1<<(i%32));
		}
	
		// Create packet
	 	PacketInt32Arr pac(stream, 94);

		uint8_t* byteStream=pac.getStream();

		delete[] byteStream;
	}

	void testPrint(){
		cout << "Test for Print" << endl;

		// Prepare the original stream
		uint32_t stream[32];
		for(int i=0; i<32; i++){
			stream[i] = i+(1<<(i%32));
		}
	
		// Create packet
	 	PacketInt32Arr pac(stream, 32);

		// Test print
		uint32_t size=pac.dumpHexStream(stdout);

		// Original one
		for(int i=0; i<32; i++){
			printf("%08x ", stream[i]);
			if(i%4 == 3)printf("\n");
		}

		CPPUNIT_ASSERT_EQUAL(32*4, (int)size);


		FILE* fp=fopen("tmp.out", "wb");
		pac.dumpByteStream(fp);

	}

	void testConsistentWithPacket32Arr(){
		cout << "packetbytearr - PacketU32Arr consistency check" << endl;

		// Prepare the original stream
		uint32_t stream[94];
		for(int i=0; i<94; i++){
			stream[i] = i+(1<<(i%32));
		}
	
		// Instance PacketInt32Arr
		PacketInt32Arr pbarr=PacketInt32Arr(stream, 94);

		// Instance PacketU32Arr
		PacketU32Arr p32arr=PacketU32Arr(stream, 94);

		CPPUNIT_ASSERT_EQUAL((int)pbarr.getSize(), (int)p32arr.getSize());

		pbarr.dumpHexStream(stdout);
		p32arr.dumpHexStream(stdout);

		uint8_t* st_b=pbarr.getStream();
		uint8_t* st_8=p32arr.getStream();

		for(int i=0; i<94*4; i++){
			CPPUNIT_ASSERT_EQUAL((int)st_b[i], (int)st_8[i]);
		}
}

};

CPPUNIT_TEST_SUITE_REGISTRATION ( PacketInt32Arr_test);


