#include <openssl/md5.h>
#include "IrfPac/PacketIntArr.h"


namespace IrfPac {

template<class T>
PacketIntArr<T>::PacketIntArr(T *stream, int nIndex)
{
	contents=new T[nIndex];
	bytes=sizeof(T);
	size=nIndex*bytes;
	nind=nIndex;
	for(size_t i=0; i<nind; i++){
		contents[i] = stream[i];
	}

}

template<class T>
PacketIntArr<T>::PacketIntArr(const PacketIntArr<T>& obj){
#ifdef DEBUG
	std::cout << "COPY CONSTRUCTOR FOR PacketIntArr IS CALLED" << std::endl;
#endif
	size=obj.size;
	nind=obj.nind;
	bytes=obj.bytes;
	
	contents=new T[nind];
	for(size_t i=0; i<nind; i++){
		contents[i] = obj.contents[i];
	}

}

template<class T>
PacketIntArr<T>::~PacketIntArr()
{
	delete[] contents;
}

/** @brief value is split into byte array.

	Explicit delete[] is needed to avoid memory leak.
*/
template<class T>
uint8_t* PacketIntArr<T>::split2byte(T val){
	size_t retsize=bytes;
	uint8_t* retval=new uint8_t[retsize];
	uint8_t mask=0xff;

	// BIG ENDIAN ASSUMED HERE
	for(size_t i=0; i<retsize; i++){
		retval[retsize-i-1] = val & mask;
		val = val >> 8;
	}
	return retval;
}

template<class T>
uint8_t* PacketIntArr<T>::getStream(void){
	uint8_t* stream=new uint8_t[size];
	int ist=0;

	for(size_t i=0; i<nind; i++){
		uint8_t* segm=split2byte(getValue(i));
		for(size_t j=0; j<bytes; j++){
			stream[ist]=segm[j];
			ist++;
		}
		delete[] segm;
	}

	return stream;
}

template<class T>
size_t PacketIntArr<T>::dumpByteStream( FILE* fp){
	uint8_t* stream=getStream();
	return fwrite(stream, sizeof(uint8_t), getSize(), fp);
}

template<class T>
size_t PacketIntArr<T>::dumpHexStream( FILE* fp){
	uint8_t* stream=getStream();
	for(size_t i=0; i<size; i++){
		if(i%16==0) fprintf(fp, "0x%08x: ", (int)i);
		fprintf(fp, "%02x", stream[i]);
		if(i%2==1)fprintf(fp, " ");
		if(i%16==15)fprintf(fp, "\n");
	}
	fprintf(fp, "\n");

	delete[] stream;
	return size;
}

template<class T>
size_t PacketIntArr<T>::getSize(){
	return size;
}

template<class T>
size_t PacketIntArr<T>::getIndexSize(){
	return nind;
}

template<class T>
T* PacketIntArr<T>::getValues(void){
	T* retCon=new T[nind];
	for(size_t i=0; i<nind; i++)retCon[i]=contents[i];
	return retCon;
}

template<class T>
T PacketIntArr<T>::getValue(size_t index){
	return contents[index];
}

       	template<class T>
	std::string PacketIntArr<T>::hash(){
		uint8_t* stream=getStream();
		int size=getSize();
		unsigned char md5[16];
		unsigned char* st=new unsigned char[size];
		for(int i=0; i<size; i++)st[i]=stream[i];

		MD5(st, size, md5);

#if 0
		for(int i=0; i<16; i++) fprintf(stderr, "%02x ", md5[i]);
#endif

		char output[33];
		sprintf(output, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", md5[0], md5[1], md5[2], md5[3], md5[4], md5[5], md5[6], md5[7], md5[8], md5[9], md5[10], md5[11], md5[12], md5[13], md5[14], md5[15]);

		delete[] stream;
		delete[] st;
		return std::string((char*)output);
	}

template class PacketIntArr<uint64_t>;
template class PacketIntArr<uint32_t>;
template class PacketIntArr<uint16_t>;
template class PacketIntArr<uint8_t>;

}
