#include<iostream>
using namespace std;

#include <cppunit/extensions/HelperMacros.h>

#include"IrfPac/PacketIntArr.h"

using namespace IrfPac;

class PacketIntArr_test : public CPPUNIT_NS::TestFixture {

	CPPUNIT_TEST_SUITE(PacketIntArr_test);
	CPPUNIT_TEST( testGetSize );
	CPPUNIT_TEST( testContents );
	CPPUNIT_TEST( testStream );
	CPPUNIT_TEST( testPrint );
	CPPUNIT_TEST( testGetSize64 );
	CPPUNIT_TEST( testContents64 );
	CPPUNIT_TEST( testStream64 );
       	CPPUNIT_TEST( testPrint64 );
	CPPUNIT_TEST( testGetSize64b );
	CPPUNIT_TEST( testContents64b );
       	CPPUNIT_TEST( testStream64b );
       	CPPUNIT_TEST( testPrint64b );
       	CPPUNIT_TEST( testCopyConstructor );
	CPPUNIT_TEST( testBmuFormat );
	CPPUNIT_TEST( testHash );
	CPPUNIT_TEST_SUITE_END();

protected:

		void testGetSize(){
			cout << "Test for getSize" << endl;

			// Prepare the original stream
			uint32_t *stream;
			stream=(uint32_t *)malloc(35*4);
			for(int i=0; i<35; i++){
				stream[i] = i*15;
			}
	
			// Create packet
		 	PacketIntArr<uint32_t> pac=PacketIntArr<uint32_t>(stream, 35);

			// Get the size of packet, and compare to the original size.
			CPPUNIT_ASSERT_EQUAL(35*4, (int) pac.getSize());
			CPPUNIT_ASSERT_EQUAL(35, (int) pac.getIndexSize());
		};

        void testContents(){
		cout << "Test for Contents" << endl;

		// Prepare the original stream
		uint32_t stream[94];
		for(int i=0; i<94; i++){
			stream[i] = i+(1<<(i%32));
		}
	
		// Create packet
	 	PacketIntArr<uint32_t> pac(stream, 94);

		// Obtain the stream
		uint32_t* pacStream=pac.getValues();

		// Assert the consistency to the original stream
		for(int i=0; i<94; i++){
			CPPUNIT_ASSERT_EQUAL((int)stream[i], (int)pacStream[i]);
			CPPUNIT_ASSERT_EQUAL((int)stream[i], (int)pac.getValue(i));
		}

		// Explicitly delete the obtained stream
		delete[] pacStream;
        }

	void testStream(){
		cout << "Test for Contents" << endl;

		// Prepare the original stream
		uint32_t stream[94];
		for(int i=0; i<94; i++){
			stream[i] = i+(1<<(i%32));
		}
	
		// Create packet
	 	PacketIntArr<uint32_t> pac(stream, 94);

		uint8_t* byteStream=pac.getStream();

		delete[] byteStream;
	}

	void testPrint(){
		cout << "Test for Contents" << endl;

		// Prepare the original stream
		uint32_t stream[32];
		for(int i=0; i<32; i++){
			stream[i] = i+(1<<(i%32));
		}
	
		// Create packet
	 	PacketIntArr<uint32_t> pac(stream, 32);

		// Test print
		uint32_t size=pac.dumpHexStream(stdout);

		// Original one
		for(int i=0; i<32; i++){
			printf("%08x ", (int)stream[i]);
			if(i%4 == 3)printf("\n");
		}

		CPPUNIT_ASSERT_EQUAL(32*4, (int)size);


		FILE* fp=fopen("tmp.out", "wb");
		pac.dumpByteStream(fp);

	}

		void testGetSize64(){
			cout << "Test for getSize" << endl;

			// Prepare the original stream
			uint64_t *stream;
			stream=(uint64_t *)malloc(35*8);
			for(int i=0; i<35; i++){
				stream[i] = i*15+(1<<i);
			}
	
			// Create packet
		 	PacketU64Arr  pac=PacketU64Arr(stream, 35);

			//Get the size of packet, and compare to the original size.
			CPPUNIT_ASSERT_EQUAL(35*8, (int) pac.getSize());
			CPPUNIT_ASSERT_EQUAL(35, (int) pac.getIndexSize());
		};

        void testContents64(){
		cout << "Test for Contents" << endl;

		// Prepare the original stream
		uint64_t stream[94];
		for(int i=0; i<94; i++){
			stream[i] = i+(1<<(i%64));
		}
	
		// Create packet
	 	PacketU64Arr  pac(stream, 94);

		// Obtain the stream
		uint64_t* pacStream=pac.getValues();

		// Assert the consistency to the original stream
		for(int i=0; i<94; i++){
			CPPUNIT_ASSERT_EQUAL((long long)stream[i], (long long)pacStream[i]);
			CPPUNIT_ASSERT_EQUAL((long long)stream[i], (long long)pac.getValue(i));
		}

		// Explicitly delete the obtained stream
		delete[] pacStream;
        }

	void testStream64(){
		cout << "Test for Contents" << endl;

		// Prepare the original stream
		uint64_t stream[94];
		for(int i=0; i<94; i++){
			stream[i] = i+(1<<(i%64));
		}
	
		// Create packet
	 	PacketU64Arr  pac(stream, 94);

		uint8_t* byteStream=pac.getStream();

		delete[] byteStream;
	}

	void testPrint64(){
		cout << "Test for Contents 64" << endl;

		// Prepare the original stream
		uint64_t stream[64];
		for(int i=0; i<64; i++){
			stream[i] = i+(((long)1)<<(i%64));
		}
	
		// Create packet
	 	PacketU64Arr pac(stream, 64);

		// Test print
		uint32_t size=pac.dumpHexStream(stdout);

		// Original one
		for(int i=0; i<64; i++){
			printf("%016lx ", (long)stream[i]);
			if(i%4 == 3)printf("\n");
		}

		CPPUNIT_ASSERT_EQUAL(64*8, (int)size);


		FILE* fp=fopen("tmp.out", "wb");
		pac.dumpByteStream(fp);

	}


		void testGetSize64b(){
			cout << "Test for getSize" << endl;

			// Prepare the original stream
			uint64_t *stream;
			stream=(uint64_t *)malloc(35*8);
			for(int i=0; i<35; i++){
				stream[i] = i*15+(1<<i);
			}
	
			// Create packet
		 	PacketIntArr<uint64_t> pac=PacketIntArr<uint64_t>(stream, 35);

			//Get the size of packet, and compare to the original size.
			CPPUNIT_ASSERT_EQUAL(35*8, (int) pac.getSize());
			CPPUNIT_ASSERT_EQUAL(35, (int) pac.getIndexSize());
		};

        void testContents64b(){
		cout << "Test for Contents" << endl;

		// Prepare the original stream
		uint64_t stream[94];
		for(int i=0; i<94; i++){
			stream[i] = i+(1<<(i%64));
		}
	
		// Create packet
	 	PacketIntArr<uint64_t> pac(stream, 94);

		// Obtain the stream
		uint64_t* pacStream=pac.getValues();

		// Assert the consistency to the original stream
		for(int i=0; i<94; i++){
			CPPUNIT_ASSERT_EQUAL((long long)stream[i], (long long)pacStream[i]);
			CPPUNIT_ASSERT_EQUAL((long long)stream[i], (long long)pac.getValue(i));
		}

		// Explicitly delete the obtained stream
		delete[] pacStream;
        }

	void testStream64b(){
		cout << "Test for Contents" << endl;

		// Prepare the original stream
		uint64_t stream[94];
		for(int i=0; i<94; i++){
			stream[i] = i+(1<<(i%64));
		}
	
		// Create packet
	 	PacketIntArr<uint64_t> pac(stream, 94);

		uint8_t* byteStream=pac.getStream();

		delete[] byteStream;
	}

	void testPrint64b(){
		cout << "Test for Contents 64" << endl;

		// Prepare the original stream
		uint64_t stream[64];
		for(int i=0; i<64; i++){
			stream[i] = i+(((long)1)<<(i%64));
		}
	
		// Create packet
	 	PacketIntArr<uint64_t> pac(stream, 64);

		// Test print
		uint32_t size=pac.dumpHexStream(stdout);

		// Original one
		for(int i=0; i<64; i++){
			printf("%016lx ", (long)stream[i]);
			if(i%4 == 3)printf("\n");
		}

		CPPUNIT_ASSERT_EQUAL(64*8, (int)size);


		FILE* fp=fopen("tmp.out", "wb");
		pac.dumpByteStream(fp);

	}


	void testCopyConstructor(){
		uint8_t stream[100];
		for(size_t i=0; i<100; i++){
			stream[i] = (i*17)%256;
		}
		PacketU8Arr *arr=new PacketU8Arr(stream, 100);		
		printf("%p\n", arr);
		
		PacketU8Arr *arrcp=arr;
		printf("%p\n", arrcp);
		
		PacketU8Arr arrcp2 = *arr;
		printf("%p\n", &arrcp2);
		
		delete arr;
		
		
	
	}

	void testBmuFormat(){
		fprintf(stdout, "\n----\n");
		uint8_t stream[1000];
		for(size_t i=0; i<1000; i++){
			stream[i] = (i*17)%256;
		}

		PacketBmuFormatter* bmufmt=new PacketBmuFormatter();

		PacketU8Arr *arr=new PacketU8Arr(stream, 100);		
		bmufmt->format(stdout, arr);
		delete arr;

		arr=new PacketU8Arr(stream, 128);
		bmufmt->format(stdout, arr);
		delete arr;

		arr=new PacketU8Arr(stream, 1000);
		bmufmt->format(stdout, arr);
		delete arr;

		fprintf(stdout, "----\n");
	}

	void testHash(){
		uint8_t stream[10];	
		for(int i=0; i<10; i++){
			stream[i]='a'+i;
		}
		PacketU8Arr p(stream, 1);   // Contents is "a"

		std::string hash=p.hash();
		cout << hash.c_str() << "  = 0CC175B9C0F1B6A831C399E269772661"<< endl;

		CPPUNIT_ASSERT_EQUAL(hash, std::string("0cc175b9c0f1b6a831c399e269772661"));
		

		PacketU8Arr* p0=new PacketU8Arr(stream, 10);

		uint8_t stream2[10];
		for(int i=0; i<10; i++){
			stream2[i]='a'+i;
		}
// p1 should be equal contents to p0

		PacketU8Arr* p1=new PacketU8Arr(stream2, 10);
		CPPUNIT_ASSERT(p0!=p1);
		CPPUNIT_ASSERT(p0->equals(p1));
	}

};

CPPUNIT_TEST_SUITE_REGISTRATION(PacketIntArr_test);

