/**@file CenaBmuReader.cpp

*/
#include<iostream>
#include<cstdlib>
#include<IrfPac/irfpac.h>
#include<IrfPac/SaraPac/sarapac.h>
#include<IrfPac/SaraPac/AscFmt/ascfmt.h>
#include <time.h>

using namespace std;
using namespace IrfPac;
using namespace IrfPac::SaraPac;
using namespace IrfPac::SaraPac::AscFmt;

int main(int argc, char* argv[]){

	if(argc==1){
		fprintf(stdout, "USAGE:  %s   filename.cena\n", argv[0]);
		exit(-10);
	}

	CenaBmuReader sr;        // Instance the CENA BMU reader class
	sr.readFile(argv[1]);    // Specify the filename to BMU reader via readFile method.

	CenaPacketArray*  cenaPac = sr.getArray();	// getArray method will return the pointer to the CenaPacketArry, which is the storage of CenaArray.

    int hydrogen = 0;
    int oxygen = 0;
    int iE,iC,iM;


	for(int idx=0; idx<cenaPac->size(); idx++){
		CenaPacket* packet = cenaPac->get(idx);  // Get the CENA packet.

		// CenaPacket has a lot of method.  You can use whatever you want.

		// A sample to get UniversalTime (epoch from 1970-01-01T00:00:00Z)
		// This is already converted by time file.

        time_t utc = (time_t) packet->getUtc();
        tm *now=gmtime(&utc);
        char buffer[40];
        strftime(buffer, 40, "%Y-%m-%dT%H:%M:%S", now);
        std::cout << &buffer[0];
        
		//cout << (time_t) utc ;

		// A sample to get HK packet embedded in science stream
		CenaFastHkPacket* fastPac = packet->getFastHkPac();     // The returned packet points newly generated instance.  You have to delete explicitly.
		// A sample to get total start count in science-HK
        int start = fastPac->getTotalStartCounts();
        cout << " " << start;
        int stop = fastPac->getTotalStopCounts();
        cout << " " << stop;
        int coinc = fastPac->getTotalCoinCounts();
        cout << " " << coinc;

		delete fastPac;   //  Otherwise memory leaks.

		// A sample to get scaling matrix.
		//CenaAccumSclMatx *acc = packet->getAccumSclMatx();
		//delete acc;

		// A sample to get science data.
		// You have to decompress the packet first.
		CenaPacket* packet2 = packet->decompress();    // Returns NULL if failed, returns the copy of the packet.
		if(packet2 == NULL){
			fprintf(stderr, "Failed to decompress.\n");
            cout << endl;  // a newline is needed however
			continue;
		}

		// Each mode has each class and getter method.
		// Here we will show the sample for Mass Accumulation mode.

		CenaSciMassAcc* mass = packet2->getCenaSciMassAcc();   // You can get science matrix from decompressed CenaPacket.
		if(mass == NULL){
			// fprintf(stderr, "This is not MASS mode.\n");
		} else { 
    
		    // A sample to get value in matrix.
            hydrogen = 0;
            oxygen = 0;
            int maxE = mass->getNrEne();
            int maxC = mass->getNrCh();
            int maxM = mass->getNrMass(); 
            
            for (iE = 0; iE < maxE; iE++) {
                for (iC = 0; iC < maxC; iC++) {
                    for (iM = 0; iM < maxM/2; iM++) {
                        hydrogen += mass->get(iE, 0, iC, iM);   // Argument order is iE, iP, iC, and iM.
                    }
                    for (iM = maxM/2; iM < maxM; iM++) {
                        oxygen += mass->get(iE, 0, iC, iM);   // Argument order is iE, iP, iC, and iM.
                    }
                }
            }
            cout << " " << hydrogen << " " << oxygen;
            delete mass;
        }
		delete packet2;

		//  delete packet;      // Currently not needed, but in the future, maybe....  Memory leak related issues are under debugging.

		cout << endl;
	}

	cenaPac->destroy();     // You have to explicitly call destroy() method otherwise all the read CenaPacket* causes memory leak.

	return 0;
}

