/*
 *  SaraAscFile.h
 *  libsadppac
 *
 *  Created by Futaana Yoshifumi on 07/07/19.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include <cstring> 
#include "IrfPac/SaraPac/sarapac.h"

namespace IrfPac{
namespace SaraPac{
namespace AscFmt{

/**	@brief The class corresponds to a dump file of SARA BMU format.

	Science packet should start by an indicator of 8 continouts * (asterisc)
	followed by 896 byte hex-dumped string, each line has 16 word (64 chars)
	The sample of using this class can be found in AscFmtReader_sample.cpp file.

	<pre>

	********SCI-DATA at Time Mon Jul  2 17:49:20 2007
	cccc 0001 aaaa 5555 0000 0000 0000 0000 0000 0183 000e 0001 0000 0001 6000 0000 
	.... (continuing 27 residual lines)
	</pre>
*/
class SaraAscFile{
	char _filename[2048];
//	FILE* _fp;

	int _fd;

	public:
		/** @brief Instance class with specifying the dump file name.
		
		You have to open the stream by openStream() method to use the file.
		*/
		SaraAscFile(char* filename){
			strcpy(_filename, filename);
//			std::cout << _filename << std::endl;
			_fd=-1;
			
			
			remain_buffer=BUFFER_SIZE;
			current_buffer=0;
			pointer_buffer=0;
			total=0;
			
			cur_pos=-1;
			
			absense=0;
			
			max_absense=0;
		};
		
		~SaraAscFile(){
		}
		
		/** @brief Open the file stream.
		*/
		bool openStream();
		
		/** @brief Close the file stream.
		*/
		bool closeStream();

		/** @brief Check the file stream is end-of-file or not.
		*/
		bool eof(){
//			if(feof(_fp)){
//				return true;
//			} else{
//				return false;
//			}
//			throw "Not support";
#if 0
			printf("ABS=%d\n", absense);
#endif
			if(absense > max_absense)return true;
			return false;
		}

		/** @brief Clear the flag of the file.

			The method clears all the error flag (including
			eof flag).
		*/
		void clearerror(){
//			clearerr(_fp);
//			throw "Not support";
			fprintf(stderr, "WARNING: ********************************************************\n");
			fprintf(stderr, "WARNING: ****** SaraAscFile::clearerror() do nothing so far. ****\n");
			fprintf(stderr, "WARNING: ********************************************************\n");
		}

		/** @brief Get next SaraPacket read from the file.
		
			If the file is not open, or reach end, NULL is returned.
			The returned packet should be deleted explicitly.
		*/
		SaraPacket* getSaraPacket();

		/**@brief Determine how long the program wait for the file stream.
		*/
		void setTimeout(int wait_msec){
			max_absense = wait_msec/100;
			fprintf(stderr, "Time out set to %d\n", max_absense);
		}
		
		
	private:	
		static const int BUFFER_SIZE=429;
		char buffer[BUFFER_SIZE];   // buffer of the data

		int remain_buffer;   // Remain number characters in the buffer
		int current_buffer;                // The position of the buffer to be read from the file
		int pointer_buffer;
		int total;              // Total character read.

		int cur_pos;
		uint8_t packet[896];

		int absense;

		int max_absense;  // this value should be a settable....
		
		void showBuffer(){
			printf("<SHOWBUFFER>\n");
			printf("BUFFER: POI=%d CUR=%d REM=%d TOT=%d\n", pointer_buffer, current_buffer, remain_buffer, total);
			for(int i=0; i<pointer_buffer; i++){
				printf("%c", buffer[i]);
			}
			printf("|");
			for(int i=pointer_buffer; i<current_buffer; i++){
				printf("%c", buffer[i]);
			}
			printf("&");
			for(int i=current_buffer; i<BUFFER_SIZE; i++){
				printf("%c", buffer[i]);
			}
			printf("\n</SHOWBUFFER>\n");

		}

		void shrinkBuffer(int nword){   // set the pointer buffer to be 0
			if(nword > pointer_buffer)throw this;
		
			if(nword != 0){
				for(int i=nword; i<BUFFER_SIZE; i++){
					buffer[i-nword] = buffer[i];
				}
				remain_buffer += nword;
				current_buffer -= nword;
				pointer_buffer -= nword;
			
				for(int i=current_buffer; i<BUFFER_SIZE; i++){
					buffer[i] = '0'+(i%10);
				}
			
			}
		}
		
		int readLine(char* buf){
#ifdef DEBUG
			printf("First, shrink the buffer.\n");
			showBuffer();
#endif
			shrinkBuffer(pointer_buffer);
#ifdef DEBUG
			printf("End of shrink the buffer.\n");
			showBuffer();
#endif
			int n=0;
			
			// First, copy the buffer to buf
			for(int i=0; i<current_buffer; i++){
				buf[i] = buffer[i];
			}
			buf[current_buffer] = '\0';
#ifdef DEBUG
			printf("<BUFFER>\n%s\n</BUFFER>\n", buf);
#endif
			
			// Second, look for '\n'
			for(n=0; n<current_buffer; n++){
				if(buf[n] == '\n') break;
			}

#ifdef DEBUG	
			printf("N=%d\n", n);
#endif

			// No data of '\n'
			if(n==current_buffer){
#ifdef DEBUG
			printf("No line completed.  Return null line.");
			showBuffer();
#endif

				buf[0]='\0';
				return -1;
			}
			
			buf[n]='\0';  // Set null instead of \n.

			pointer_buffer += (n+1);

#ifdef DEBUG			
			printf("Last, shrink the buffer.\n");
			showBuffer();
#endif
			shrinkBuffer(n+1);
#ifdef DEBUG
			printf("End of shrink the buffer.\n");
			showBuffer();
#endif		
			return n;
		
		}
};

}}}///NAME SPACE

