/**@file SaraDemultiplex.cpp
 * @brief A command line tool to divide BMU stream to SWIM and CENA stream.  The output is in the package format.
 * See {@link IrfPac::SaraPac::SwimSciencePackage} and {@link CenaSciencePackage} for details.
 *
 * Usage is
 * <pre>
 *    SaraDemultiplex   [options]   input_file
 * </pre>
 * The output is input_file.sarapkg/
 */

#include<iostream>
#include <cstdlib>
#include<libgen.h>
#include<sys/stat.h>
#include<getopt.h>

#include<IrfPac/irfpac.h>
#include<IrfPac/SaraPac/sarapac.h>
#include<IrfPac/SaraPac/AscFmt/ascfmt.h>


using namespace std;
using namespace IrfPac;
using namespace IrfPac::SaraPac;
using namespace IrfPac::SaraPac::AscFmt;

class CenaMassDump{
private:
	FILE *fp1, *fp2, *fp3, *fp4, *fp5, *fp6, *fp7, *fp8;
	CenaSciMassAccFormatter *f2, *f3, *f4, *f5, *f6, *f7;

public:
	CenaMassDump(char* dirname){
		char name[256];
		sprintf(name, "%s/01hk", dirname); fp1=fopen(name, "w");
		sprintf(name, "%s/02et", dirname); fp2=fopen(name, "w");
		f2=new CenaSciMassAccFormatterEspec();
		sprintf(name, "%s/03mt", dirname); fp3=fopen(name, "w");
		f3=new CenaSciMassAccFormatterMspec();
		sprintf(name, "%s/04ct", dirname); fp4=fopen(name, "w");
		f4=new CenaSciMassAccFormatterCspec();
		sprintf(name, "%s/05met", dirname); fp5=fopen(name, "w");
		f5=new CenaSciMassAccFormatterMspecEne();
		sprintf(name, "%s/06lcet", dirname); fp6=fopen(name, "w");
		f6=new CenaSciMassAccFormatterLightEspecC();
		sprintf(name, "%s/07hcet", dirname); fp7=fopen(name, "w");
		f7=new CenaSciMassAccFormatterHeavyEspecC();
		sprintf(name, "%s/08reserved", dirname); fp8=fopen(name, "w");
	}
	~CenaMassDump(){
		delete f2;
		delete f3;
		delete f4;
		delete f5;
		delete f6;
		delete f7;
	}
	void flush(){fflush(fp1); fflush(fp2); fflush(fp3); fflush(fp4); fflush(fp5); fflush(fp6); fflush(fp7); fflush(fp8);}
	void close(){fclose(fp1); fclose(fp2); fclose(fp3); fclose(fp4); fclose(fp5); fclose(fp6); fclose(fp7); fclose(fp8);}
	void add(CenaPacket* cena, TimeConversion* tconv){
		if(!cena->isValid()){
			return;
		}
		// Cena decompression check.  If it fails, nothing to do.
		CenaPacket* ucena = cena->decompress();
		fprintf(stdout, "DEMUL_DUMP_CEMS: %p\n", ucena);
		if(ucena==NULL) return;

		// 01hk HK table is the same as cenahk.
		cena->printParamsTbl2(fp1, tconv);
		// 02 to 04 is science data, only when counter mode is active.



		CenaSciMassAcc* pac=ucena->getCenaSciMassAcc();
		if(pac != NULL){
    			fprintf(fp2, "0x%04x %14.2f ", cena->get4sCycleCntr(), cena->getUniversalTime(tconv)*1000);
			f2->dump(fp2, pac);
			fprintf(fp2, "\n");

    			fprintf(fp3, "0x%04x %14.2f ", cena->get4sCycleCntr(), cena->getUniversalTime(tconv)*1000);
			f3->dump(fp3, pac);
			fprintf(fp3, "\n");

    			fprintf(fp4, "0x%04x %14.2f ", cena->get4sCycleCntr(), cena->getUniversalTime(tconv)*1000);
			f4->dump(fp4, pac);
			fprintf(fp4, "\n");

    			fprintf(fp5, "0x%04x %14.2f ", cena->get4sCycleCntr(), cena->getUniversalTime(tconv)*1000);
			f5->dump(fp5, pac);
			fprintf(fp5, "\n");

    			fprintf(fp6, "0x%04x %14.2f ", cena->get4sCycleCntr(), cena->getUniversalTime(tconv)*1000);
			f6->dump(fp6, pac);
			fprintf(fp6, "\n");

    			fprintf(fp7, "0x%04x %14.2f ", cena->get4sCycleCntr(), cena->getUniversalTime(tconv)*1000);
			f7->dump(fp7, pac);
			fprintf(fp7, "\n");

			delete pac;
		}
		delete ucena;
	}



};

class CenaCntrDump{
private:
	FILE *fp1, *fp2, *fp3, *fp4, *fp5, *fp6, *fp7, *fp8;
	CenaSciCntrAccFormatter *f2, *f3, *f4, *f5;
public:
	CenaCntrDump(char* dirname){
		char name[256];
		sprintf(name, "%s/01hk", dirname); fp1=fopen(name, "w");
		sprintf(name, "%s/02startnc", dirname); fp2=fopen(name, "w");
		f2=new CenaSciCntrAccFormatterStartNC();
		sprintf(name, "%s/03startcoin", dirname); fp3=fopen(name, "w");
		f3=new CenaSciCntrAccFormatterCoin();
		sprintf(name, "%s/04stop", dirname); fp4=fopen(name, "w");
		f4=new CenaSciCntrAccFormatterStopPlate();
		sprintf(name, "%s/05mesh", dirname); fp5=fopen(name, "w");
		f5=new CenaSciCntrAccFormatterStopMesh();
		sprintf(name, "%s/06reserved", dirname); fp6=fopen(name, "w");
		sprintf(name, "%s/07reserved", dirname); fp7=fopen(name, "w");
		sprintf(name, "%s/08reserved", dirname); fp8=fopen(name, "w");
	}
	~CenaCntrDump(){
		delete f2;
		delete f3;
		delete f4;
		delete f5;
	}
	void flush(){fflush(fp1); fflush(fp2); fflush(fp3); fflush(fp4); fflush(fp5); fflush(fp6); fflush(fp7); fflush(fp8);}
	void close(){fclose(fp1); fclose(fp2); fclose(fp3); fclose(fp4); fclose(fp5); fclose(fp6); fclose(fp7); fclose(fp8);}
	void add(CenaPacket* cena, TimeConversion *tconv){
		if(!cena->isValid()){
			return;
		}
		CenaPacket* ucena = cena->decompress();
		if(ucena==NULL){
			return;
		}

		// 01hk HK table is the same as cenahk.
		cena->printParamsTbl2(fp1, tconv);
		// 02 to 05 is science data, only when counter mode is active.
		CenaSciCntrAcc* pac=ucena->getCenaSciCntrAcc();
		if(pac != NULL){
    			fprintf(fp2, "0x%04x %14.2f ", cena->get4sCycleCntr(), cena->getUniversalTime(tconv)*1000);
			f2->dump(fp2, pac);
			fprintf(fp2, "\n");

    			fprintf(fp3, "0x%04x %14.2f ", cena->get4sCycleCntr(), cena->getUniversalTime(tconv)*1000);
			f3->dump(fp3, pac);
			fprintf(fp3, "\n");

    			fprintf(fp4, "0x%04x %14.2f ", cena->get4sCycleCntr(), cena->getUniversalTime(tconv)*1000);
			f4->dump(fp4, pac);
			fprintf(fp4, "\n");

    			fprintf(fp5, "0x%04x %14.2f ", cena->get4sCycleCntr(), cena->getUniversalTime(tconv)*1000);
			f5->dump(fp5, pac);
			fprintf(fp5, "\n");


			delete pac;
		}
		delete ucena;
	}


};

class SwimSciDump{
public:
  SwimSciDump(char* dirname){
    char name[256];
    sprintf(name, "%s/01hk", dirname); fp1=fopen(name, "w");
    sprintf(name, "%s/02sclstr", dirname); fp2=fopen(name, "w");
    sprintf(name, "%s/03sclstp", dirname); fp3=fopen(name, "w");
    sprintf(name, "%s/04scltof", dirname); fp4=fopen(name, "w");
    sprintf(name, "%s/05accmt", dirname); fp5=fopen(name, "w");
    sprintf(name, "%s/06accdt", dirname); fp6=fopen(name, "w");
    sprintf(name, "%s/07accet", dirname); fp7=fopen(name, "w");
    sprintf(name, "%s/08sclstrdef", dirname); fp8=fopen(name, "w");
    sprintf(name, "%s/09sclstrd", dirname); fp9=fopen(name, "w");
  }
	void flush(){fflush(fp1); fflush(fp2); fflush(fp3); fflush(fp4); fflush(fp5); fflush(fp6); fflush(fp7); fflush(fp8); fflush(fp9);}
	void close(){fclose(fp1); fclose(fp2); fclose(fp3); fclose(fp4); fclose(fp5); fclose(fp6); fclose(fp7); fclose(fp8); fclose(fp9);}
  void add(SwimPacket* swim, TimeConversion* tconv){
    if(!swim->isValid())return;
    SwimPacket* swim_decmp = swim->decompress();
    if(swim_decmp==NULL)return;

    // 01hk HK table is the same as swimhk.
    swim->printParamsTbl2(fp1, tconv);

    SwimAccumTotMatx *tot=swim->getAccumTotMatx();
    // 02sclstr is the start matrix.
    SwimAccumTotMatxEspecDump dp2(SwimAccumTotMatx::START_COUNTER);
    fprintf(fp2, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    tot->dump(fp2, &dp2);

    // 08sclstrdef
    SwimAccumTotMatxDeflDump dp8(SwimAccumTotMatx::START_COUNTER);
    fprintf(fp8, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    tot->dump(fp8, &dp8);

    // 09sclstrd
    SwimAccumTotMatxEspecDump8Dir dp9(SwimAccumTotMatx::START_COUNTER);
    fprintf(fp9, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    tot->dump(fp9, &dp9);

    // 03sclstr is the start matrix.
    SwimAccumTotMatxEspecDump dp3(SwimAccumTotMatx::STOP_COUNTER);
    fprintf(fp3, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    tot->dump(fp3, &dp3);

    // 04sclstr is the start matrix.
    SwimAccumTotMatxEspecDump dp4(SwimAccumTotMatx::TOF_COUNTER);
    fprintf(fp4, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    tot->dump(fp4, &dp4);

    delete tot;
    
    // Science data
    SwimSciAcc *sci=NULL;
    sci=swim_decmp->getSwimSciAcc();
    delete swim_decmp;

    fprintf(fp5, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    SwimSciAccDump* dp5=new SwimSciAccMSpectraDump();
    dp5->dump(fp5, sci);


    fprintf(fp6, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000); 
    SwimSciAccDump* dp6=new SwimSciAccDSpectraDump();
    dp6->dump(fp6, sci);

    fprintf(fp7, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    SwimSciAccDump* dp7=new SwimSciAccESpectraDump();
    dp7->dump(fp7, sci);

    delete dp5;
    delete dp6;
    delete dp7;

  }
private:
	FILE *fp1,*fp2,*fp3,*fp4,*fp5,*fp6,*fp7,*fp8, *fp9;
};


int main(int argc, char** argv){

	int result;
	char timefilename[256]="";

	FILE* tfp=NULL;
	TimeConversion* tconv=NULL;
	TimeFile tf;

	while((result = getopt(argc, argv, "t:"))!=-1){
		switch(result){
		case 't':
			fprintf(stdout, "Option -t is set.\n");
			fprintf(stdout, "Using the time file: %s\n", optarg);
			sprintf(timefilename, "%s\n", optarg);
			tfp=fopen(optarg, "rb");
			if(tfp==NULL){
				fprintf(stderr, "Failed to open the time file %s\n", optarg);
				fprintf(stderr, "Aborting...\n");
				exit(-3);
			}
			tf.readFile(tfp);
//			tconv=new TimeConversionTimeFile(&tf);
			tconv=new TimeConversionTimeFileExtpol(&tf);
			break;
		}
	}

	if(tconv==NULL)tconv=new TimeConversionOffset(0);

	if(argc==optind){
		cerr << "USAGE:  " << argv[0] << "  filename" << endl;
		exit(-1);
	}

	char dirout[256];
	sprintf(dirout, "%s.sarapkg", basename(argv[argc-1]));
	
	mode_t mode=0000755;
	mkdir(dirout, mode);

	char  swimout[256];
	sprintf(swimout, "%s/swim", dirout);
	fprintf(stderr, "OUTPUT(SWIM): %s\n", swimout);
	FILE* fps = fopen(swimout, "w");
	if(fps==NULL){
	  fprintf(stderr, "Failed to open file %s for SWIM output\n", swimout);
	  exit(-1);
	}

	char  swimhkout[256];
	sprintf(swimhkout, "%s/swimhk", dirout);
	fprintf(stderr, "OUTPUT(SWIM-HK): %s\n", swimhkout);
	FILE* fpsh = fopen(swimhkout, "w");
	if(fpsh==NULL){
	  fprintf(stderr, "Failed to open file %s for SWIM HK output\n", swimhkout);
	  exit(-1);
	}
	SwimPacket::printParamsTblHeader(fpsh);

	char swimsciout[256];
	sprintf(swimsciout, "%s/swimsci", dirout);
	fprintf(stderr, "OUTPUT(SWIM-SCI): %s\n", swimsciout);
	mkdir(swimsciout, mode);

	SwimSciDump ssd(swimsciout);

	char  cenaout[256];
	sprintf(cenaout, "%s/cena", dirout);
	fprintf(stderr, "OUTPUT(CENA): %s\n", cenaout);
	FILE* fpc = fopen(cenaout, "w");
	if(fpc==NULL){
	  fprintf(stderr, "Failed to open file %s for CENA output\n", cenaout);
	  exit(-1);
	}

	char  cenahkout[256];
	sprintf(cenahkout, "%s/cenahk", dirout);
	fprintf(stderr, "OUTPUT(CENA-HK): %s\n", cenahkout);
	FILE* fpch = fopen(cenahkout, "w");
	if(fpch==NULL){
	  fprintf(stderr, "Failed to open file %s for CENA-HK output\n", cenahkout);
	  exit(-1);
	}
	CenaPacket::printParamsTblHeader(fpch);

	char cenacntrout[256];
	sprintf(cenacntrout, "%s/cenacntr", dirout);
	fprintf(stderr, "OUTPUT(CENA-cntr): %s\n", cenacntrout);
	mkdir(cenacntrout, mode);

	char cenamassout[256];
	sprintf(cenamassout, "%s/cenamass", dirout);
	fprintf(stderr, "OUTPUT(CENA-mass): %s\n", cenamassout);
	mkdir(cenamassout, mode);

	CenaCntrDump ccd(cenacntrout);
	CenaMassDump cmd(cenamassout);

	// Specify a new input file of SARA-DPU dump format.
	SaraAscFile *asc = new SaraAscFile(argv[argc-1]);
	
	// Instance the SaraBufferedDemultiplexor
	SaraBufferedDemultiplexor* demul=new SaraBufferedDemultiplexor();
	
	int iCena=0, iSwim=0;

	PacketBmuFormatter bmuformatter;

	// Open the stream of input file.
	if(asc->openStream()){
		
		// Check the input stream is not eof.
		while(! asc->eof() ){
			
			// Obtain SaraPacket from the input file
			SaraPacket* saraPac = asc->getSaraPacket();
			
			// Check if SaraPacket can be read
			if(saraPac != NULL){

				// SaraPacket will be appended to Demultiplexor.
				demul->appendSaraPacket(saraPac);

				// If CenaPacket is buffered
				if(demul->getNrCena() > 0){
					
					CenaPacket* cena=demul->getCena();
					double utc=cena->getUniversalTime(tconv);
					time_t utc_i = (time_t)utc;
					struct tm* tm=gmtime(&utc_i);
					fprintf(fpc, "******** CENA-DATA at Time %04d-%02d-%02dT%02d:%02d:%02dZ %ld %d\n", tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec,  (long)utc, (int) cena->getSize());
					bmuformatter.format(fpc, cena);
					iCena++;

					fprintf(stdout, "CenaPacket [%4x]\n", cena->get4sCycleCntr());
					cena->dumpAll(stdout);
					cena->printParamsTbl2(fpch, tconv);

					ccd.add(cena, tconv);
					ccd.flush();

					cmd.add(cena, tconv);
					cmd.flush();
					
					// You must delete cena instance.
					delete cena;
				}
				
				if(demul->getNrSwim() > 0){

					SwimPacket* swim=demul->getSwim();

					double utc=swim->getUniversalTime(tconv);
					time_t utc_i = (time_t)utc;
					struct tm* tm=gmtime(&utc_i);
					fprintf(fps, "******** SWIM-DATA at Time %04d-%02d-%02dT%02d:%02d:%02dZ %ld %d\n", tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec,  (long)utc, (int) swim->getSize());
					bmuformatter.format(fps, swim);
					iSwim++;

					swim->dumpAll(stdout);
					swim->printParamsTbl2(fpsh, tconv);

					fprintf(stdout, "SwimPacket [%4x]\n", swim->get8sCycleCntr());

					ssd.add(swim, tconv);
					ssd.flush();

					// Delete swim instance
					delete swim;
				}
				
				fprintf(stdout, "SaraPacket [%4x] (%4x)\n", saraPac->getCounter(), saraPac->getPacketId());
				delete saraPac;
			}

		}	
	}
	else{  // Opening the input file failed.
		cerr << "Failed to open " << argv[argc-1] << endl;
		exit(-2);
	}

	// Deleting file and demultiplexor.
	delete asc;
	delete demul; 
	fclose(fpc);
	fclose(fps);
	fclose(fpch);
	fclose(fpsh);

	return 0;
}

void swim_science_table(char* dirname, SwimPacket* pac){
  char hkfile[256];
  sprintf(hkfile, "%s/hk.dat", dirname);
  FILE* fp=fopen(hkfile, "w");

  // 1.counter 2.time
  fprintf(fp, "0x%04x ", pac->get8sCycleCntr());
  fprintf(fp, "%12f ", pac->getUniversalTime(NULL)*1000);
  
  // 3.nD 4.nE 5.nM
  fprintf(fp, "%2d ", pac->getDeflectionBins());
  fprintf(fp, "%2d ", pac->getEnergyBins());
  fprintf(fp, "%2d ", pac->getMassBins());

  // 6.HVPSoscSts 7.TdcLatchUpSts 8.TdcLatchUpProt 9.TdcPwrSts

  fprintf(fp, "%1d ", pac->isHvpsOscOn());
  fprintf(fp, "%1d ", pac->isLatchUp());
  fprintf(fp, "%1d ", pac->isLatchUpProtectOn());
  fprintf(fp, "%1d ", pac->isTdcPowerOn());

  SwimFastHkPacket *hk=pac->getFastHkPac();

  fprintf(fp, "%04x ", hk->getHV());
  fprintf(fp, "%04x ", hk->getVCeem());
  fprintf(fp, "%04x ", hk->getVUpDef());
  fprintf(fp, "%04x ", hk->getVLoDef());
  fprintf(fp, "%04x ", hk->getVAnaCel());
  fprintf(fp, "%04x ", hk->getVTofCel());
  fprintf(fp, "%04x ", hk->getTempTofBrd());



  fprintf(fp, "\n");
  fflush(fp);
  delete hk;

}

