/** @file SaraDemultiplexor_sample.cpp
	@brief This is a sample code for demultiplex.
*/

#include<iostream>
#include <cstdlib>
using namespace std;

#include"IrfPac/SaraPac/SaraPacket.h"
#include"IrfPac/SaraPac/AscFmt/ascfmt.h"

using namespace IrfPac;
using namespace IrfPac::SaraPac;
using namespace IrfPac::SaraPac::AscFmt;

int main(int argc, char** argv){

	cout << "SaraDemultiplexor_sample" << endl;

	if(argc == 1){
		cerr << "USAGE:    " << argv[0] << "   filename" << endl;
		exit(-1);
	}

	SaraAscFile* asc=new SaraAscFile(argv[1]);  // Input file

	SaraDemultiplexor* demul=new SaraDemultiplexor();
	CenaPacket* cenaPac=NULL;
	SwimPacket* swimPac=NULL;

	int i=0, iNull=0, iCena=0, iSwim=0;

	if(asc->openStream()){
		while(! asc->eof() ){
			SaraPacket* saraPac = asc->getSaraPacket();
			if(saraPac != NULL){
				cout << "---" << endl;
				fprintf(stdout, "SARA PAC ARRIVED: %d : %04x : %d\n",i ,saraPac->getPacketId(), saraPac->getCounter());

				///////////////////////////////////////////
				// THIS LOGIC SHOULD NOT BE CHANGED.
				if(!demul->sequenceCheck(saraPac)){  // Note that the first packet is considered as false!
					cerr << "*** SEQUENCIAL CHECK NG : FLUSH BUFFER ***" << endl;
					demul->flushAll();
				}
				cenaPac=demul->appendCena(saraPac);   // Append the SaraPacket to CENA buffer.  If the buffer contents are complete, it will be returned.
				swimPac=demul->appendSwim(saraPac);   // Append the SaraPacket to SWIM buffer.  If the buffer contents are complete, it will be returned.
				// END OF THE UNCHANGABLE IF-BLOCK.
				///////////////////////////////////////////
				
				
				if(cenaPac!=NULL){   // If CenaPacket arrived.
					cout << "+++ CENA PACKET ARRIVED +++ " << iCena << endl;
					iCena++;
					
					//////////////////////////////////////////////////
					// Here you can put processing for CENA packet.
					//////////////////////////////////////////////////
					
					cenaPac->dumpAll(stdout);
					
					cout << "+++ COMPRESSION CHECK +++" << endl;
					if(cenaPac->isMatrixCompressed()){
						cout << "CENA IS COMPRESSED!" << endl;

						CenaPacket* cenaDecmp=cenaPac->decompress();
						cout << "DECOMPRESSION FINISHED" << endl;
						cenaDecmp->dumpAll();
						delete cenaDecmp;
						cout << "CENA COMPRESSED PACKET PROCESSING END" << endl;
					}
					else{
						cout << "CENA IS NOT COMPRESSED!" << endl;
						// You can also decompress the packet.
						CenaPacket* cenaDecmp=cenaPac->decompress();
						// Contents should be the same
						cenaDecmp->dumpAll();
						delete cenaDecmp;
						cout << "CENA NON-COMPRESSED PACKET PROCESSING END" << endl;
					}
					


					delete cenaPac;
				}


				if(swimPac!=NULL){   // If SwimPacket arrived.
					cout << "+++ SWIM PAC : " << iSwim << endl;
					iSwim++;

					//////////////////////////////////////////////////
					// Here you can put processing for SWIM packet.
					//////////////////////////////////////////////////

					swimPac->dumpAll(stdout);
					
					SwimPacket* swimDecmp;

					cout << "+++ COMPRESSION CHECK +++" << endl;
					if(swimPac->isMatrixCompressed()){
						cout << "SWIM IS COMPRESSED!" << endl;
						swimDecmp=swimPac->decompress();
						cout << "COMPRESSION FINISHED" << endl;

						swimDecmp->dumpAll();
						delete swimDecmp;
						cout << "SWIM COMPRESSION PACKET END" << endl;
					}
					else{
						cout << "SWIM IS NOT COMPRESSED!" << endl;
						uint8_t *pac=swimPac->getStream();
						uint32_t size=swimPac->getSize();
						swimDecmp=new SwimPacket(pac, size);
						delete[] pac;
					}
					
					// Interpretation check
					SwimMatrix *sMatx=swimDecmp->getSwimMatrix();
					SwimScienceDataPacket* sciPac=sMatx->interpret(swimDecmp->getEnergyBins(), swimDecmp->getDeflectionBins(), swimDecmp->getMassBins());

					delete sMatx;
					delete sciPac;

					delete swimDecmp;
					delete swimPac;
				}

				delete saraPac;
				i++;
			}
			else{
				cout << "SARA PAC NULL: " << iNull << endl;
				iNull++;
			}
		}
		asc->closeStream();
	}
	else{
		cerr << "Failed to open " << argv[1] << endl;
		exit(-2);
	}

	return 0;


}


