/*
 *  SaraSpacePac2Bmu.h
 *  libsadppac
 *
 *  Created by futaana on 08/03/28.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef __IRFPAC__SARAPAC__ASCFMT
#define __IRFPAC__SARAPAC__ASCFMT

#include<string>
#include <inttypes.h>

namespace IrfPac{
namespace SaraPac{
namespace AscFmt{

/** @brief Implementation of the converter class of 'SpacePacket' to BMU format.

	The space packet is the packet that the project team is treating.
	Because the definition is not clear, the implementation is done
	by 'hacking' way of the sample source.

	Current definition (2008-10-23) is here.
	<pre>
	Pre-header 33 byte + header 12 byte + contents 896 byte + trail 6 byte
	</pre>
	This definition is derived from the 'hacking' way.

	See SaraSpacePac2BmuApp.cpp for the usage of this class.
	
	(Below is obsolete, but it is left intentionally as a reminder.)
	During the AIT of delivery (2008-04), the definition is as follows.
	<pre>
	Header 12 byte + Contents 896 byte
	</pre>
	The header will be reduced to be 6 byte for the nominal mission.
*/
class SaraSpacePac2Bmu {

	bool detailedCheck;
	bool isFirstPacket;
	uint16_t counter;

	public:
		SaraSpacePac2Bmu(){
			detailedCheck=false;
			isFirstPacket=true;
			counter=0;
		}
		
#if 0
		// For nominal missions.  But it should be directly made from the satellite packet (what to call? 24 byte header's format )
		// 08-10-23: current understanding is during the nominal mission,
		// the datasize would be 908 but they have added 33 byte header in advance.
		static const int SPACE_PAC_SIZE=902;
		static const int SPACE_PAC_HEADER_SIZE=6;
#endif
		// During AIT, we need to use this program.  Format is different from mission SpacePacket.
		// 08-10-23: It looks like that there are 'other' header and trailer, each constitute of 33 and 6 byte, respectiviely, are added.
		static const int SPACE_PAC_HEADER_SKIP_BYTE=33;
		static const int SPACE_PAC_SIZE=908;
		static const int SPACE_PAC_HEADER_SIZE=12;  //NOTE: For AIT, 1	2 byte header exists.
		static const int SPACE_PAC_TRAIL_SKIP_BYTE=6;

		/**@brief Convert space packet to BMU format.
		
		Convert space packet (byte array) to BMU format.
		The spacepac size should be 902.  Otherwise fatal error may occur.
		No boundary check is conducted in this method.  BE CAREFUL OF THE BUFFER OVERFLOW!!
		*/
		std::string spacepac2bmu(uint8_t* spacepac);

		void setDetailedCheckEnabled(bool b){detailedCheck=b;}
		void resetCounter(){isFirstPacket=true;}
};

}}}//Namespace
#endif

