#include<iostream>
using namespace std;

#include<cppunit/extensions/HelperMacros.h>

#include"IrfPac/irfpac.h"
#include"IrfPac/SaraPac/sarapac.h"
#include"IrfPac/SaraPac/AscFmt/ascfmt.h"

using namespace IrfPac;
using namespace IrfPac::SaraPac;
using namespace IrfPac::SaraPac::AscFmt;

class TimeFile_test : public CPPUNIT_NS::TestFixture {

    public:

	CPPUNIT_TEST_SUITE(TimeFile_test);
	CPPUNIT_TEST(testRead);
	CPPUNIT_TEST(testTimeConversion);
	CPPUNIT_TEST(testTimeConversionRO);
	CPPUNIT_TEST_SUITE_END();
		
	void testRead(){
		char* filename="src/IrfPac/SaraPac/AscFmt/timefile.sample";
		FILE* fp=fopen(filename, "rb");
		TimeFile tf;
		tf.readFile(fp);
	}
	
	void testTimeConversion(){
		char* filename="src/IrfPac/SaraPac/AscFmt/timefile.sample";
		FILE* fp=fopen(filename, "rb");
		TimeFile tf;
		tf.readFile(fp);

		TimeConversionTimeFile tc(&tf);
		cout << tc.obt2utc(1145225640) << endl;
		CPPUNIT_ASSERT_DOUBLES_EQUAL(1233036249, tc.obt2utc(1145225640), 1);
	}

	void testTimeConversionRO(){
		char* filename="src/IrfPac/SaraPac/AscFmt/timefile_ro.sample";
		FILE* fp=fopen(filename, "rb");
		TimeFile tf;
		tf.readFile(fp);
		TimeConversionTimeFile tc(&tf);
		cout << tc.obt2utc(1814390000) << endl;
		CPPUNIT_ASSERT_DOUBLES_EQUAL(1235519809, tc.obt2utc(1814390000), 2);
		cout << tc.obt2utc(0) << endl;
		CPPUNIT_ASSERT_DOUBLES_EQUAL(1235519819, tc.obt2utc(0), 2);
		cout << tc.obt2utc(5000) << endl;
		CPPUNIT_ASSERT_DOUBLES_EQUAL(1235519824, tc.obt2utc(5000), 2);

		TimeConversionTimeFileExtpol te(&tf);
		fprintf(stdout, "OBT:UTC[DIRECT]:UTC[EXTPOL] = %15.3f  %15.3f  %15.3f\n", 1814390000., tc.obt2utc(1814390000), te.obt2utc(1814390000));
		fprintf(stdout, "OBT:UTC[DIRECT]:UTC[EXTPOL] = %15.3f  %15.3f  %15.3f\n", 0., tc.obt2utc(0), te.obt2utc(0));
		fprintf(stdout, "OBT:UTC[DIRECT]:UTC[EXTPOL] = %15.3f  %15.3f  %15.3f\n", 5000., tc.obt2utc(5000), te.obt2utc(5000));

		CPPUNIT_ASSERT_DOUBLES_EQUAL(1235519809, te.obt2utc(1814390000), 4000);
		CPPUNIT_ASSERT_DOUBLES_EQUAL(1235519819, te.obt2utc(0), 4000);
		CPPUNIT_ASSERT_DOUBLES_EQUAL(1235519824, te.obt2utc(5000), 4000);
	}

};

CPPUNIT_TEST_SUITE_REGISTRATION(TimeFile_test);

