#include<iostream>
#include<cmath>
#include<ctime>

#include<IrfPac/irfpac.h>
#include<IrfPac/SaraPac/sarapac.h>
#include<IrfPac/SaraPac/AscFmt/ascfmt.h>

using namespace std;
using namespace IrfPac;
using namespace IrfPac::SaraPac;
using namespace IrfPac::SaraPac::AscFmt;

/** @file TimeSync_sample.cpp
    @brief This is a source code of a simple tool to investigate how the time is treated for SARA/DPU output.
 */

std::string tostr(double msec){
  time_t buftime=(time_t)msec;
  struct tm* utc = gmtime(&buftime);
  char strtime[128];

  strftime(strtime, 127, "%Y-%m-%dT%H:%M:%S (%s)", utc);

  std::string retstr(strtime);
  return retstr;
}

int main(int argc, char** argv){

	if(argc==1){
		cerr << "USAGE:   " << argv[0] << "  filename" << endl;
	}

	SaraAscFile *asc=new SaraAscFile(argv[1]);

	SaraBufferedDemultiplexor *demul = new SaraBufferedDemultiplexor();

	signed long obt0=0;
	long dpu0=0;
	long data0=0;
  double ut0=0;
        fprintf(stdout, "OBTtime@M [msec] DPUtime@M DPUtime@M_rsd [msec] IntegCycTime IntegCycTime_rsd [msec] UT D_OBTtime D_DPUtime D_IntegCycTime D_UT\n");

	if(asc->openStream()){
	  while(! asc->eof()){

	    SaraPacket* saraPac = asc->getSaraPacket();
	    if(saraPac == NULL)break;

	    demul->appendSaraPacket(saraPac);

	    if(demul->getNrCena() > 0){
	      CenaPacket* cpac=demul->getCena();

	      if(obt0==0){
		obt0=cpac->getObtTime();
		dpu0=cpac->getDpuTime();
		data0=cpac->getEndIntegCycleTime();
    ut0=cpac->getUniversalTime(NULL)*1000;
	      }

	      //	      cpac->dumpAll();

	      fprintf(stdout, "0x%012llx [%f] 0x%lx 0x%lx [%f] 0x%lx 0x%lx [%f] %f [%s] %ld %ld %ld %f\n",
		      (signed long long)cpac->getObtTime(),
		      cpac->getObtTimeInMsec(),
		      (long)cpac->getDpuTime(),
		      (long)cpac->getTcountDpu(),
		      cpac->getDpuTimeMarkerInMsec(),
		      (long)cpac->getEndIntegCycleTime(),
		      (long)cpac->getTcountAcq(),
		      cpac->getDataAcquisTimeInMsec(),
		      cpac->getUniversalTime(NULL)*1000,
		      tostr(cpac->getUniversalTime(NULL)*1000).c_str(),
		      ((signed long)(cpac->getObtTime() - obt0)),
		      cpac->getDpuTime()-dpu0,
		      cpac->getEndIntegCycleTime()-data0, 
          cpac->getUniversalTime(NULL)*1000-ut0);

		///// OBT CONVERSION TEST
	      //		double obt = convert(cpac->getObtTime());
	      //		printf("Converted: %e\n", obt);

	      delete cpac;
	    }

	  }
	}

	
	return 0;


}


