#include<iostream>
#include<cppunit/extensions/HelperMacros.h>

#include"IrfPac/irfpac.h"
#include"IrfPac/SaraPac/sarapac.h"

using namespace std;
using namespace IrfPac;
using namespace IrfPac::SaraPac;

class CenaAccumSclMatx_test : public CPPUNIT_NS::TestFixture {
	
public:
	
	CPPUNIT_TEST_SUITE(CenaAccumSclMatx_test);
	CPPUNIT_TEST(testDirectInstance);
	CPPUNIT_TEST(testInstance);
	CPPUNIT_TEST(testValue);
	CPPUNIT_TEST_SUITE_END();
	
	void testDirectInstance();
	void testInstance();
	void testValue();
};

void CenaAccumSclMatx_test::testDirectInstance(){
	int n_e=8;
	int n_p=32;
	
	uint8_t *stream=(uint8_t*) malloc (n_e*n_p*4*2);
	
	CenaAccumSclMatx matx=CenaAccumSclMatx(stream, n_e, n_p);		
	
	delete[] stream;
}

void CenaAccumSclMatx_test::testInstance(){
	// Prepare
	uint8_t *stream=(uint8_t*) malloc (800);
	// Initialize by a sequence.  Importance stream should be overwritten.
	for(int i=0; i<800; i++){
		stream[i]=i;
	}
	// Magic number
	stream[0]=stream[1]=0xaa;
	stream[2]=stream[3]=0x55;
	
	// Assumption:  CH=7 (1) Ene=4 (011) Phase=8 (00111) Mass=32 (0011111)
	// Word 11 (20,21 byte) ->  1011001110011111 = 0xb39f
	stream[20] = 0xb3;
	stream[21] = 0x9f;

	// Mode should be one of coincidence mode: e.g. mass acc.
	// Word 13 (24 byte) 11001000 = 0xc8
	stream[24] = 0xc8;
	

	
	CenaPacket cena=CenaPacket(stream, 800);
	free(stream);
	
	CPPUNIT_ASSERT_EQUAL(800, (int)cena.getSize());
	
	printf("%lx\n", (long)cena.getBinningPrm());
	CPPUNIT_ASSERT_EQUAL(7, (int)cena.getNrCh());
	CPPUNIT_ASSERT_EQUAL(4, (int)cena.getNrEne());
	CPPUNIT_ASSERT_EQUAL(8, (int)cena.getNrPhase());
	CPPUNIT_ASSERT_EQUAL(32, (int)cena.getNrMass());
	
	CenaAccumSclMatx *matx=cena.getAccumSclMatx();		
	
	matx->dumpHexStream(stdout);
	
	CPPUNIT_ASSERT_EQUAL(0x90, (int)matx->getValue((size_t)0));
	CPPUNIT_ASSERT_EQUAL(4*8*4, (int)matx->getSize());
	
	delete matx;
};

void CenaAccumSclMatx_test::testValue(){
	// Prepare
	uint8_t *stream=(uint8_t*) malloc (800);
	// Initialize by a sequence.  Importance stream should be overwritten.
	for(int i=0; i<800; i++){
		stream[i]=i;
	}
	// Magic number
	stream[0]=stream[1]=0xaa;
	stream[2]=stream[3]=0x55;
	
	// Assumption:  CH=7 (1) Ene=4 (011) Phase=8 (00111) Mass=32 (0011111)
	// Word 11 (20,21 byte) ->  1011001110011111 = 0xb39f
	stream[20] = 0xb3;
	stream[21] = 0x9f;
	
	// Mode should be one of coincidence mode: e.g. mass acc.
	// Word 13 (24 byte) 11001000 = 0xc8
	stream[24] = 0xc8;
	


	CenaPacket cena=CenaPacket(stream, 800);
	free(stream);
	
	// Get CenaAccumSclMatx
	CenaAccumSclMatx *matx=cena.getAccumSclMatx();

	// Get raw value
	uint8_t rawvalue=matx->getRaw(0,0,0);   // 0x90
	cout << "value (raw) = " << (long)rawvalue << endl; 
	CPPUNIT_ASSERT_EQUAL(0x90, (int)rawvalue);
	
	rawvalue=matx->getRaw(0,7,0);   // 0x90
	cout << "value (raw) = " << (long)rawvalue << endl; 
	CPPUNIT_ASSERT_EQUAL(0xac, (int)rawvalue);
	
	// Get decompressed value.
	uint32_t value=matx->get(0,1,0);   // 0x94 for raw -- 393216
	cout << "value (dec) = " << (long)value << endl; 
	CPPUNIT_ASSERT_EQUAL(393216, (int)value);
	
	// Get decompressed value.
	rawvalue=matx->getRaw(1,0,0);
	value=matx->get(1,0,0);   // 0xc2 for raw -- 20971520
	cout << "values = " << (long)rawvalue << " " << (long)value << endl; 
	CPPUNIT_ASSERT_EQUAL(0xb0, (int)rawvalue);
	CPPUNIT_ASSERT_EQUAL(4194304, (int)value);
	
	// try catch
	try{
		matx->get(4,5,3);
		CPPUNIT_ASSERT(false);
	} catch (const char* mess){
		printf("%s\n", mess);
	}
	
	delete matx;
	
};

CPPUNIT_TEST_SUITE_REGISTRATION(CenaAccumSclMatx_test);
