#include<iostream>
#include<string>
#include<stdexcept>
#include "IrfPac/SaraPac/CenaFastHkPacket.h"

namespace IrfPac {

namespace SaraPac {

CenaFastHkPacket::CenaFastHkPacket(uint8_t* stream, size_t size)
 : PacketU8Arr(stream, size)
{
#ifdef DEBUG
	std::cout << "CenaFastPacket Generating" << std::endl;
#endif
}

CenaFastHkPacket::CenaFastHkPacket(uint8_t* stream)
 : PacketU8Arr(stream, 119)
{
#ifdef DEBUG
	std::cout << "CenaFastPacket Generating" << std::endl;
#endif
}

CenaFastHkPacket::~CenaFastHkPacket()
{
}

void CenaFastHkPacket::dumpAll(FILE* fp){

	fprintf(fp, "FLAGS:     %02x\n", IrfPac::SaraPac::getByteAt(*this, 0));
	fprintf(fp, "HVmain(P)  %1d\n", getFlagHvMain());
	fprintf(fp, "HVmain(S)  %1d\n", getFlagHvSafety());
	fprintf(fp, "HVmain(PE) %1d\n", getFlagHvOnEna());
	fprintf(fp, "HVmain(SE) %1d\n", getFlagHvLvlEna());
	fprintf(fp, "HVmain(ER) %1d\n", getFlagError());

	fprintf(fp, "HV main:1-2:%02x %02x: %04x\n", getByteAt(*this, 1), getByteAt(*this, 2), getHvMain());
	fprintf(fp, "HV startMCP:3-4:%02x %02x: %04x\n", getByteAt(*this, 3), getByteAt(*this, 4), getHvStartMcp());
	fprintf(fp, "HV stopMCP:5-6:%02x %02x: %04x\n", getByteAt(*this, 5), getByteAt(*this, 6), getHvStopMcp());
	fprintf(fp, "HV tof:7-8:%02x %02x: %04x\n", getByteAt(*this, 7), getByteAt(*this, 8), getHvTof());
	fprintf(fp, "HV def:9-10:%02x %02x: %04x\n", getByteAt(*this, 9), getByteAt(*this, 10), getHvDef());
	fprintf(fp, "TEMP IFE:11-12:%02x %02x: %04x\n", getByteAt(*this, 11), getByteAt(*this, 12), getTempIFE());
	fprintf(fp, "TEMP HVPS:13-14:%02x %02x: %04x\n", getByteAt(*this, 13), getByteAt(*this, 14), getTempHVPS());
	fprintf(fp, "SV_TABLE(2A/1/LENS/2B):15-16:%02x%02x: %1x %1x %1x %1x\n", getByteAt(*this, 15), getByteAt(*this, 16), getSv1Table(), getSv2Table(), getSv3Table(), getSv4Table());
	fprintf(fp, "PsyncCntr:33-34:%02x %02x: %04x\n", getByteAt(*this, 33), getByteAt(*this, 34), getPsyncCntr());
	fprintf(fp, "BaseCntr:35-36:%02x %02x: %04x\n", getByteAt(*this, 35), getByteAt(*this, 36), getBaseCntr());
	fprintf(fp, "ObsMode(Sync):17:%02x (B7=%01x) :%01x\n", getByteAt(*this, 17),  getByteAt(*this, 17) >> 7, getObsModeSync());
	fprintf(fp, "ObsMode(Sensor):17:%02x (B1=%01x,B0=%01x) :%01x\n", getByteAt(*this, 17),  (getByteAt(*this, 17)&0x2) >> 1, (getByteAt(*this,17)&0x1), getObsModeSensor());
	fprintf(fp, "TotalStartCount:101-102:%02x %02x: %04x\n", getByteAt(*this, 101), getByteAt(*this, 102), getTotalStartCounts());

	fprintf(fp, "..... HEX DUMP\n");
	dumpHexStream(fp);
}

uint16_t CenaFastHkPacket::getHvMain(){return get12bitValue(1,2);};
uint16_t CenaFastHkPacket::getHvStartMcp(){return get12bitValue(3,4);};
uint16_t CenaFastHkPacket::getHvStopMcp(){return get12bitValue(5,6);};
uint16_t CenaFastHkPacket::getHvTof(){return get12bitValue(7,8);};
uint16_t CenaFastHkPacket::getHvDef(){return get12bitValue(9,10);};
uint16_t CenaFastHkPacket::getTempIFE(){return get12bitValue(11,12);};
uint16_t CenaFastHkPacket::getTempHVPS(){return get12bitValue(13,14);};
uint16_t CenaFastHkPacket::getPsyncCntr(){return get16bitValue(33,34);};
uint16_t CenaFastHkPacket::getBaseCntr(){return get16bitValue(35,36);};

uint8_t CenaFastHkPacket::getSv1Table(){
	uint8_t tbl=getByteAt(*this, 15);
	return (tbl & 0x0f);
}
uint8_t CenaFastHkPacket::getSv2Table(){
	uint8_t tbl=getByteAt(*this, 15);
	return (tbl & 0xf0) >> 4;
}
uint8_t CenaFastHkPacket::getSv3Table(){
	uint8_t tbl=getByteAt(*this, 16);
	return (tbl & 0x0f);
}
uint8_t CenaFastHkPacket::getSv4Table(){
	uint8_t tbl=getByteAt(*this, 16);
	return (tbl & 0xf0) >> 4;
}
uint8_t CenaFastHkPacket::getSvTable(int SvName){
	switch(SvName){
	case SV_WAVE1:
		return getSv1Table();
	case SV_WAVE2A:
		return getSv2Table();
	case SV_WAVE2B:
		return getSv3Table();
	case SV_LENS:
		return getSv4Table();
	default:
		char buf[256];
		sprintf(buf, "CenaFastHkPacket::getSvTable(svname=%d [must be 0,1,2,3])", SvName);
		std::string mesg(buf);
		throw std::invalid_argument(mesg);
	}
}


uint8_t CenaFastHkPacket::getObsModeRaw(){return getByteAt(*this,17);}
uint8_t CenaFastHkPacket::getObsModeSync(){return getByteAt(*this,17)>>7;}
uint8_t CenaFastHkPacket::getObsModeSensor(){return getByteAt(*this, 17)&0x3;}

uint16_t CenaFastHkPacket::getTotalStartCounts(){return get16bitValue(101,102);};
uint16_t CenaFastHkPacket::getTotalStopCounts(){return get16bitValue(103,104);};
uint16_t CenaFastHkPacket::getTotalCoinCounts(){return get16bitValue(105,106);};

uint16_t CenaFastHkPacket::getSvMonitor(int svname, int level){
	if(level<1 || level>8){
		char buf[256];
		sprintf(buf, "CenaFastHkPacket::getSvMonitor(svname=%d [must be 0,1,2,3], level=%d [must be 1-8])", svname, level);
		std::string mesg(buf);
		throw std::invalid_argument(mesg);
	}

	// W1L1 is DV38 and DV39.   Offset should be 37 and 38.

	int index = 37+ svname*16+(level-1)*2;

#ifdef DEBUG
	std::cout << svname << " " << level << " " << index << std::endl;
#endif

	return get12bitValue(index, index+1);
}


}

}
