#include<iostream>
using namespace std;

#include<cppunit/extensions/HelperMacros.h>

#include"IrfPac/irfpac.h"
#include"IrfPac/SaraPac/sarapac.h"

using namespace IrfPac;
using namespace IrfPac::SaraPac;

class CenaFastHkPacket_test : public CPPUNIT_NS::TestFixture {
	public:

	    CPPUNIT_TEST_SUITE(CenaFastHkPacket_test);
            CPPUNIT_TEST(testGeneration);
            CPPUNIT_TEST(test12b);
            CPPUNIT_TEST(testbit);
            CPPUNIT_TEST(testSv);
	    CPPUNIT_TEST_SUITE_END();

        void testGeneration();
	void testSv();
        void test12b(){
#ifdef DEBUG
	  for(uint8_t u=0; u<=0x0f; u++){
	    for(uint8_t l=0; l<0xff; l++){
	      fprintf(stdout, "%x %x %x\n", 
		      u, l, CenaFastHkPacket::test12b(u, l));
	      CPPUNIT_ASSERT_EQUAL(((int)u)*256+l, (int)(CenaFastHkPacket::test12b(u,l)));
	    }

	    uint8_t l = 0xff;
	    fprintf(stdout, "%x %x %x\n", 
		    u, l, CenaFastHkPacket::test12b(u, l));
	    CPPUNIT_ASSERT_EQUAL(((int)u)*256+l, (int)(CenaFastHkPacket::test12b(u,l)));
	
	  }
#else
	  fprintf(stdout, "get12Value test ...skipped... Recompile with DEBUG option.\n");
#endif
        }


        void testbit(){


#ifdef DEBUG
	  uint8_t val = 0xa3;   // B 1010 0011
	  CPPUNIT_ASSERT( CenaFastHkPacket::testbit(val, 0));
	  CPPUNIT_ASSERT( CenaFastHkPacket::testbit(val, 1));
	  CPPUNIT_ASSERT(!CenaFastHkPacket::testbit(val, 2));
	  CPPUNIT_ASSERT(!CenaFastHkPacket::testbit(val, 3));
	  CPPUNIT_ASSERT(!CenaFastHkPacket::testbit(val, 4));
	  CPPUNIT_ASSERT( CenaFastHkPacket::testbit(val, 5));
	  CPPUNIT_ASSERT(!CenaFastHkPacket::testbit(val, 6));
	  CPPUNIT_ASSERT( CenaFastHkPacket::testbit(val, 7));

#else
	  fprintf(stdout, "getBitAt test ...skipped... Recompile with DEBUG option.\n");
#endif
        }


};

void CenaFastHkPacket_test::testSv(){
	// First, prepare byte array for generation of CenaPacket.
	uint8_t *stream=(uint8_t*) malloc (800);
	stream[0]=stream[1]=0xaa;
	stream[2]=stream[3]=0x55;
	for(int i=4; i<800; i++){
		stream[i]=i;
	}

	CenaPacket cena=CenaPacket(stream, 800);
	delete[] stream;

	CenaFastHkPacket *hk=cena.getFastHkPac();

	hk->dumpAll();

	uint16_t val=hk->getSvMonitor(CenaFastHkPacket::SV_WAVE1, 1);  // This is S1L1, which is DV37-38.  Value should be 0e3f
	CPPUNIT_ASSERT_EQUAL(0x0e3f, (int)val);

	val=hk->getSvMonitor(CenaFastHkPacket::SV_LENS, 8);  // This is SLL8, which is DV100-101.  Value should be 0c7d
	CPPUNIT_ASSERT_EQUAL(0x0c7d, (int)val);

}

void CenaFastHkPacket_test::testGeneration(){
	// Normally CENA subcommuted HK packet is generated from CenaPacket.
	
	// First, prepare byte array for generation of CenaPacket.
	uint8_t *stream=(uint8_t*) malloc (800);
	stream[0]=stream[1]=0xaa;
	stream[2]=stream[3]=0x55;
	for(int i=4; i<800; i++){
		stream[i]=i;
	}

	CenaPacket cena=CenaPacket(stream, 800);
	delete[] stream;

	CPPUNIT_ASSERT_EQUAL(800, (int)cena.getSize());

	// Second obtain CenaFastHkPac class
	CenaFastHkPacket *hk=cena.getFastHkPac();

	// Check the size
	CPPUNIT_ASSERT_EQUAL(119, (int)(hk->getSize()));
	
	// Chekc the contents
	uint8_t* str=hk->getStream();

	// Start from 25th byteCenaPacket_test
	CPPUNIT_ASSERT_EQUAL(25, (int)str[0]);
	
	delete hk;

	delete[] str;

};

CPPUNIT_TEST_SUITE_REGISTRATION(CenaFastHkPacket_test);

