/*
 *  CenaMatrix.cpp
 *  libsadppac
 *
 *  Created by Futaana Yoshifumi on 07/06/24.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include<stdexcept>
#include "IrfPac/SaraPac/CenaMatrix.h"
#include <IrfPac/SaraPac/saradecomp.h>
#include <IrfPac/SaraPac/CenaSciCntrAcc.h>
#include <IrfPac/SaraPac/CenaSciMassAcc.h>
#include <IrfPac/SaraPac/CenaSciTofAcc.h>
#include<string>

namespace IrfPac{
namespace SaraPac{
	CenaMatrix::CenaMatrix(uint8_t* stream, int size) : PacketU8Arr(stream, size){
	  _tm_mode=TMMODE_UNKNWON;
	  _nc=-1;
	  _ne=-1;
	  _np=-1;
	  _nm=-1;
	}
	
	CenaMatrix::~CenaMatrix(){
	}


	CenaMatrix* CenaMatrix::decompress(int verbose){	
		size_t decomp_size;
		char* formattedInput=preformat(verbose);
		uint8_t* decompressedContent=decomp_main(formattedInput, &decomp_size, verbose);
		delete[] formattedInput;
		if(decompressedContent==NULL){
			 if (verbose >= VERBOSE_NORMAL) {
				uint8_t* contents=getStream();
				size_t size=getSize();
				fprintf(stderr, "<c.txt>\n");
				for(size_t i=0; i<size; i++){
					fprintf(stderr, "%02x", contents[i]);
					if(i%2==1)fprintf(stderr, " ");
				}
				fprintf(stderr, "\n</c.txt>\n");
				delete contents;
			 }
			return NULL;  // decompression failed.
		}
		CenaMatrix* retMatx=new CenaMatrix(decompressedContent, decomp_size);
		delete[] decompressedContent;
		return retMatx;
	}

	char* CenaMatrix::preformat(int verbose){
		// input shoudl be "xxxx xxxx xxxx ....."
		size_t size=getSize();
		if (verbose >= VERBOSE_DEBUG) 
			fprintf(stdout, "Size of packet to be decompressed= %d\n", (int)size);

		char* str=new char[5*size/2+1];

		uint8_t* bStream=getStream();
		for(size_t i=0; i<size; i+=2){   // Even index number
			sprintf(&(str[(i/2)*5]), "%02x%02x ", bStream[i], bStream[i+1]);
		}
		
		if (verbose >= VERBOSE_DEBUG) {
			fprintf(stdout, "Decompressed packet contents (in ascii)\n");
			fprintf(stdout, "%s\n", str);
		}
		
		delete[] bStream;
		return str;
	}
	uint8_t* CenaMatrix::decomp_main(char* inputStream, size_t* decompressedSize, int verbose){
		Decompression* decomp=new Decompression();
		if (verbose >= VERBOSE_DEBUG) {
			fprintf(stdout, "Compressed packet size = %d\n", (int)(5*getSize()/2+1));
			for(size_t iii=0; iii<5*getSize()/2+1; iii++){
				fprintf(stdout, "%c", inputStream[iii]);
			}
			fprintf(stdout, "\n");
		}
		try{
			uint8_t* decompPac=decomp->decompressionAll(inputStream, 5*getSize()/2+1, decompressedSize);  
			delete decomp;
			return decompPac;
		}
		catch(const std::runtime_error &exc){
			if (verbose >= VERBOSE_NORMAL)
					fprintf(stderr, "Failed to decompress:  %s\n", exc.what());
			return NULL;
		}
	}

  int CenaMatrix::getExpectedMatrixSize(uint8_t tm_mode, uint8_t nc, uint8_t ne, uint8_t np, uint8_t nm){

    // Up to the science mode, the returned value is different.
    // For Mass Accumulation mode.
    if(tm_mode == TMMODE_MASS_ACC){
      return ne*np*nc*nm;
    }
    else if (tm_mode == TMMODE_TOF_ACC){
      return ne*1024;
    }
    else if (tm_mode == TMMODE_COUNT_ACC){
      return ne*np*55;
    }
    else if (tm_mode == TMMODE_NON_PROCESS){
      fprintf(stderr, "HOW TO TREAT NON PROCESS MODE?\n");
      return -1;
    }
    return -1;
  }

	CenaScienceDataPacket* CenaMatrix::interpret(uint8_t tm_mode, uint8_t nc, uint8_t ne, uint8_t np, uint8_t nm)
	{

				// Obtain the raw stream to be added to the CenaScienceDataPacket.
				uint8_t* stream=getStream();   // Note: stream+4 should be the contents.

				// Check if it is compressed or not.
				uint32_t header=(((((((uint32_t)stream[0])<<8)|stream[1])<<8)|stream[2])<<8)|stream[3];
#ifdef DEBUG
				fprintf(stdout, "Header: %lx\n", (long)header);
#endif

				if(header == COMPRESSED_HEADER){
					fprintf(stderr, "COMPRESSED PACKET IS NOT YET SUPPORTED.  PLEASE DO compress() FIRST.\n");
					throw ILLEGAL_PACKET;
				}

				// Up to the science mode, the returned class is different.
				// For Mass Accumulation mode.
				if(tm_mode == TMMODE_MASS_ACC){
					CenaSciMassAcc* matx=new CenaSciMassAcc(stream+4, ne, np, nc, nm);
					delete[] stream;
					return matx;
				}
				else if (tm_mode == TMMODE_TOF_ACC){
					CenaSciTofAcc* matx=new CenaSciTofAcc(stream+4, ne, 1024);
					delete[] stream;
					return matx;
				}
				else if (tm_mode == TMMODE_COUNT_ACC){
					CenaSciCntrAcc* matx=new CenaSciCntrAcc(stream+4, ne, np, 55);
					delete[] stream;
					return matx;
				}
				else if (tm_mode == TMMODE_NON_PROCESS){
					fprintf(stderr, "HOW TO TREAT NON PROCESS MODE?\n");
					delete[] stream;
					throw UNKNOWN_TMMODE;
				}
				delete[] stream;
				throw UNKNOWN_TMMODE;				

			};

}
}

