#include<iostream>
#include<string>
#include<sstream>
#include "IrfPac/SaraPac/CenaPacket.h"

using namespace std;

namespace IrfPac {

namespace SaraPac {

CenaPacket::CenaPacket(uint8_t* stream, size_t size)
 : PacketU8Arr(stream, size)
{
//	std::cout << "CenaPacket Generating" << std::endl;
	_savedUtc=0;
	_qflag=QFLAG_NOTSET;
}

CenaPacket::~CenaPacket()
{
}

CenaPacket* CenaPacket::decompress(int verbose){
	CenaPacket* decompPac=NULL;
	if(!isMatrixCompressed()){ // When CENA compression is OFF
		if (verbose >= VERBOSE_DEBUG)
			fprintf(stderr, "Matrix is already decompressed.  Return the new packet instance with identical contents.\n");
		decompPac=new CenaPacket(*this);  // Just copy the contents by calling copy constructor.
 		decompPac->_savedUtc=_savedUtc;
		decompPac->_qflag=_qflag;
	}
	else{  // CENA COMPRESSION ON
		CenaMatrix* compMatx=getCenaMatrix();               // Original cena matrix
		CenaMatrix* decompMatx=compMatx->decompress(verbose);      // Decompressed cena matrix
		if(decompMatx==NULL){  // Failed to decompress
			if (verbose >= VERBOSE_MIN) 
				fprintf(stderr, "Failed to decompress CENA packet: (4s-cntr: 0x%x)\n", get4sCycleCntr());
			decompPac=NULL;  // Returns NULL.
		}
		else{  // Successfully decompressed.
			// Create a new stream.
			size_t matxSizeDecmp = decompMatx->getSize();  // Decompressed matrix size

			size_t offset = offsetByteToCenaMatrix();

			size_t packetSize = offset + 4 + matxSizeDecmp;   // Decompressed packet size.  4 is for 0xaaddaadd
			uint8_t* contDecmp=new uint8_t[packetSize];

			if (verbose >= VERBOSE_DEBUG) {
				size_t matxSizeCmp = compMatx->getSize();      // Original matrix size
				fprintf(stdout, "CenaMatrix decompression succeeded: %d -> %d (Packet size %d)\n", (int)matxSizeCmp, (int)matxSizeDecmp, (int)packetSize);
			}

			// Copy the same part (STUFF, HK and SCL_MATX)
			uint8_t* contCmp=getStream();
			for(size_t i=0; i<offset; i++){
				contDecmp[i] = contCmp[i];
			}
			delete[] contCmp;

			// aaddaadd is added

			contDecmp[offset]=0xaa;
			contDecmp[offset+1]=0xdd;
			contDecmp[offset+2]=0xaa;
			contDecmp[offset+3]=0xdd;

			// Copy the decompressed CenaMatrix
			uint8_t* contMatx=decompMatx->getStream();
			for(size_t i=0; i<matxSizeDecmp; i++){
				contDecmp[i+offset+4] = contMatx[i];
			}
			decompPac=new CenaPacket(contDecmp, packetSize);
 			decompPac->_savedUtc=_savedUtc;
 			decompPac->_qflag=_qflag;

			delete[] contMatx;
			delete[] contDecmp;

		}
		if(compMatx!=NULL)delete compMatx;
		if(decompMatx!=NULL)delete decompMatx;

	}
	return decompPac;

}



CenaSciCntrAcc* CenaPacket::getCenaSciCntrAcc(){
		if(isMatrixCompressed()){
			fprintf(stderr, "Science Matrix is still Compressed.  Use decompress() first.  (%s:%d)\n", __FILE__, __LINE__);
			throw "Science Matrix is still Compressed.  Use decompress() first.";
		}
		if(getTMmode() != TMMODE_COUNT_ACC){
			return NULL;
		}
		CenaMatrix* matx = getCenaMatrix();
		CenaSciCntrAcc* sci=(CenaSciCntrAcc*)(matx->interpret(getTMmode(), getNrCh(), getNrEne(), getNrPhase(), getNrMass()));

		delete matx;

		return sci;
}

CenaSciMassAcc* CenaPacket::getCenaSciMassAcc(){
		if(isMatrixCompressed()){
			fprintf(stderr, "Science Matrix is still Compressed.  Use decompress() first.  (%s:%d)\n", __FILE__, __LINE__);
			throw "Science Matrix is still Compressed.  Use decompress() first.";
		}
		if(getTMmode() != TMMODE_MASS_ACC){
			return NULL;
		}
		CenaMatrix* matx = getCenaMatrix();
		CenaSciMassAcc* sci=(CenaSciMassAcc*)(matx->interpret(getTMmode(), getNrCh(), getNrEne(), getNrPhase(), getNrMass()));

		delete matx;

		return sci;
}

CenaSciMassAccE16* CenaPacket::getCenaSciMassAccE16(){

	if(getNrEne() != 8){
		ostringstream oss;
		oss << "Energy step should be 8. (Data) " << getNrEne() << endl;
		string emesg = oss.str();
		cerr << emesg << endl;
		throw std::runtime_error(emesg);
	}

	// You have to get the SV table.
	CenaFastHkPacket* hk = getFastHkPac();

	int sv = hk->getSv1Table();

	if(sv != hk->getSv2Table() || sv != hk->getSv3Table() || sv != hk->getSv4Table()){
		ostringstream oss;
		oss << "Table inconsistency.  " << sv  << ", " << hk->getSv2Table() <<
				", " << hk->getSv3Table() << ", " << hk->getSv4Table() << endl;
		string emesg = oss.str();
		cerr << emesg << endl;
		throw std::runtime_error(emesg);
	}

	delete hk;

	CenaSciMassAcc *pac1 = getCenaSciMassAcc();
	if(pac1 == NULL){
		ostringstream oss;
		oss << "Not a mass mode." << endl;
		string emesg = oss.str();
		cerr << emesg << endl;
		throw std::runtime_error(emesg);
	}

	// New instance of E16.
	//
	CenaSciMassAccE16 *pac = new CenaSciMassAccE16(pac1, sv);
	delete pac1;


	return pac;
}

CenaSciTofAcc* CenaPacket::getCenaSciTofAcc(){
		if(isMatrixCompressed()){
			fprintf(stderr, "Science Matrix is still Compressed.  Use decompress() first.  (%s:%d)\n", __FILE__, __LINE__);
			throw "Science Matrix is still Compressed.  Use decompress() first.";
		}
		if(getTMmode() != TMMODE_TOF_ACC){
			return NULL;
		}
		CenaMatrix* matx = getCenaMatrix();
		CenaSciTofAcc* sci=(CenaSciTofAcc*)(matx->interpret(getTMmode(), getNrCh(), getNrEne(), getNrPhase(), getNrMass()));

		delete matx;

		return sci;
}

void CenaPacket::dumpAll(FILE* fp){

		fprintf(fp, "*** CENA PACKET (%d byte) ***\n", (int)this->getSize());
		fprintf(fp, "*** VALIDITY = %d ***\n", (int)this->isValid());

		fprintf(fp, "** STUFF\n");
		fprintf(fp, "MagicNum:1-2: %04x/%04x: %08x\n",
			getWordAt(*this,1), getWordAt(*this,2), this->getHeadMagicNumber());
		fprintf(fp, "OBT:3-5: %04x/%04x/%04x: 0x%012lx\n",
			getWordAt(*this,3), getWordAt(*this,4),getWordAt(*this,5),
			(signed long)(this->getObtTime()));
		fprintf(fp, "DPUtime:6: %04x: %04x\n",
			getWordAt(*this,6), this->getDpuTime());
		fprintf(fp, "Tcount:7: %04x: %04x\n",
			getWordAt(*this,7), this->getTcountDpu());
		fprintf(fp, "DataAcq:8: %04x: %04x\n",
			getWordAt(*this,8), this->getDataAcquisTime());
		fprintf(fp, "TcountTimer:9: %04x: %04x\n",
			getWordAt(*this,9), this->getTcountAcq());
		fprintf(fp, "4S cycle:10: %04x: %04x\n",
			getWordAt(*this,10), this->get4sCycleCntr());
		fprintf(fp, "BinPrm:11: %04x: %04x\n",
			getWordAt(*this,11), this->getBinningPrm());
		fprintf(fp, "\tC:B15:%04x:%2d\n",
			getWordAt(*this,11), this->getNrCh());
		fprintf(fp, "\tE:B14-B12:%04x:%2d\n",
			getWordAt(*this,11), this->getNrEne());
		fprintf(fp, "\tP:B11-B7:%04x:%2d\n",
			getWordAt(*this,11), this->getNrPhase());
		fprintf(fp, "\tM:B6-B0:%04x:%2d\n",
			getWordAt(*this,11), this->getNrMass());
		fprintf(fp, "NoCycle:12: %04x: %04x\n",
			getWordAt(*this,12), this->getNrCycles());

		fprintf(fp, "Modes:13:%04x: %04x\n",
			getWordAt(*this,13), this->getModes());
		fprintf(fp, "\tSWon:B15:%04x:%2d\n",
			getWordAt(*this,13), (int)this->isSwimOn());
		fprintf(fp, "\tCEon:B14:%04x:%2d\n",
			getWordAt(*this,13), (int)this->isCenaOn());
		fprintf(fp, "\tCCmp:B13:%04x:%2d\n",
			getWordAt(*this,13), (int)this->isCenaCompOn());

		fprintf(fp, "\tTM:B12-10:%04x:%2d (%s)\n",
			getWordAt(*this,13), this->getTMmode(), getTMmodeStr().c_str());

		fprintf(fp, "\tSN:B9-8:%04x:%2d (%s)\n",
			getWordAt(*this,13), this->getSensorMode(), getSensorModeStr().c_str());


		CenaFastHkPacket *hkpac=this->getFastHkPac();
		fprintf(fp, "** HK\n");
		hkpac->dumpAll(fp);
		delete hkpac;

		fprintf(fp, "** CENA_ACC_MATX\n");
		CenaAccumSclMatx *acMat=this->getAccumSclMatx();
		acMat->dumpAll(fp);
		delete acMat;

		fprintf(fp, "** CENA_MATX\n");
		fprintf(fp, "\tCompression:%d\n", (int)this->isMatrixCompressed());
		CenaMatrix *mat=this->getCenaMatrix();
		mat->dumpAll(fp);
		delete mat;

		fprintf(fp, "******\n");

		// To be continued.



}

void CenaPacket::printParamsTblHeader(FILE* fp){
	fprintf(fp, "#   CENA PACKET TABLE (HEX) VERSION 2.0\n");
	fprintf(fp, "#1:VALIDITY  2:CNTR  3:OBT  4:DPUTIME  5:UT[msec]  6:CH  7:EN  8:PH  9:MAS  10:CYC  11:SW-ON  12:CE-ON  13:CE-CMP  14:SNS-MOD  15:TML-MOD  ");
	fprintf(fp, "16:HV-MA-PW  17:HV-SFTY-EN  18:HV-ON-EN  19:HV-LVL-EN  20:ERR  21:HV-MAIN  22:HV-STA  23:HV-STO  24:HV-TOF  25:HV-DEF  26:IFE-TEM  27:HVPS-TEM  ");
	fprintf(fp, "28:PSYN 29:BASE 30:MODE 31:SYNC 32:SENS 33:START 34:STOP 35:COIN  36:W1L1  37:W1L8\n");  // MODE should be removed in future.
}

CenaSubcHkPacket* CenaPacket::getSubcomHkPac(){
	uint8_t* str=getStream();
	CenaSubcHkPacket *pac=new CenaSubcHkPacket(str+25);
	delete[] str;
	return pac;
}

void CenaPacket::printParamsTbl(FILE* fp){
	printParamsTbl2(fp, NULL);
}

void CenaPacket::printParamsTbl2(FILE* fp, TimeConversion* tc){
	printParamsTbl3(fp, tc, false);
}

void CenaPacket::printParamsTbl3(FILE* fp, TimeConversion* tc, bool useUtc){

	fprintf(fp, "%1d ", (int)this->isValid());
	fprintf(fp, "0x%04x ", this->get4sCycleCntr());
	fprintf(fp, "%12f ", (this->getMarkerObt()));
	fprintf(fp, "%9.1f ", this->getObsTimeDpu());
	if (useUtc) {
		fprintf(fp, "%16.4f ", this->getUtc()*1000);
	} else	 {
		fprintf(fp, "%16.4f ", this->getUniversalTime(tc)*1000);
	}	
	fprintf(fp, "0x%02x ", this->getNrCh());
	fprintf(fp, "0x%02x ", this->getNrEne());
	fprintf(fp, "0x%02x ", this->getNrPhase());
	fprintf(fp, "0x%02x ", this->getNrMass());
	fprintf(fp, "0x%04x ", this->getNrCycles());
	fprintf(fp, "%1d ", (int)this->isSwimOn());
	fprintf(fp, "%1d ", (int)this->isCenaOn());
	fprintf(fp, "%1d ", (int)this->isCenaCompOn());
	fprintf(fp, "0x%04x ", this->getTMmode());
	fprintf(fp, "0x%02x ", this->getSensorMode());

	CenaFastHkPacket *hkpac=this->getFastHkPac();

	fprintf(fp, "%1d ", (int)hkpac->getFlagHvMain());
	fprintf(fp, "%1d ", (int)hkpac->getFlagHvSafety());
	fprintf(fp, "%1d ", (int)hkpac->getFlagHvOnEna());
	fprintf(fp, "%1d ", (int)hkpac->getFlagHvLvlEna());
	fprintf(fp, "%1d ", (int)hkpac->getFlagError());

	fprintf(fp, "0x%04x ", hkpac->getHvMain());
	fprintf(fp, "0x%04x ", hkpac->getHvStartMcp());
	fprintf(fp, "0x%04x ", hkpac->getHvStopMcp());
	fprintf(fp, "0x%04x ", hkpac->getHvTof());
	fprintf(fp, "0x%04x ", hkpac->getHvDef());
	fprintf(fp, "0x%04x ", hkpac->getTempIFE());
	fprintf(fp, "0x%04x ", hkpac->getTempHVPS());

	fprintf(fp, "0x%04x ", hkpac->getPsyncCntr());
	fprintf(fp, "0x%04x ", hkpac->getBaseCntr());

	fprintf(fp, "0x%02x ", hkpac->getObsModeRaw());   // This should be removed in future
	fprintf(fp, "0x%1x ", hkpac->getObsModeSync());
	fprintf(fp, "0x%1x ", hkpac->getObsModeSensor());

	fprintf(fp, "0x%04x ", hkpac->getTotalStartCounts());
	fprintf(fp, "0x%04x ", hkpac->getTotalStopCounts());
	fprintf(fp, "0x%04x ", hkpac->getTotalCoinCounts());

	fprintf(fp, "0x%04x ", hkpac->getSvMonitor(CenaFastHkPacket::SV_WAVE1, 1));
	fprintf(fp, "0x%04x ", hkpac->getSvMonitor(CenaFastHkPacket::SV_WAVE1, 8));


	delete hkpac;

	fprintf(fp, "\n");
}

}

}
