#include<iostream>

#include<time.h>

using namespace std;

#include<cppunit/extensions/HelperMacros.h>

#include"IrfPac/irfpac.h"
#include"IrfPac/SaraPac/sarapac.h"

using namespace IrfPac;
using namespace IrfPac::SaraPac;

class CenaPacket_test : public CPPUNIT_NS::TestFixture {

    private:
	CenaPacket* readCenaMassSample(){
		fprintf(stdout, "HOGEHOGE\n");
		FILE* fp = fopen("cenapacket_mass_sample.txt", "r");
		fprintf(stdout, "FP = %p\n", fp);

		if(fp == NULL){
			fp = fopen("src/IrfPac/SaraPac/cenapacket_mass_sample.txt", "r");
			if(fp == NULL){
				fprintf(stderr, "Failed to load sample file.  return NULL.\n");
				throw "No file found.";
			}
		}

		fprintf(stdout, "FP = %p\n", fp);

		int size = 10000;
		uint8_t* arr = new uint8_t[10000];

		int idx=0;
		int a0;

		while(!feof(fp)){
			fscanf(fp, "%4x", &a0);
			arr[idx] =   (a0 & 0xff00) >> 8;
			arr[idx+1] = (a0 & 0x00ff);
			idx += 2;
		}

		delete[] arr;
		fclose(fp);
		return new CenaPacket(arr, idx);
	}

    public:

	    CPPUNIT_TEST_SUITE ( CenaPacket_test );
            CPPUNIT_TEST(testInstance );
            CPPUNIT_TEST(testMagicNumber );
            CPPUNIT_TEST(testdump );
            CPPUNIT_TEST(testDecomp );
            CPPUNIT_TEST(testCenaSciCntrAcc );
            CPPUNIT_TEST(testCenaSciCntrAcc2 );
            CPPUNIT_TEST(testCenaMassAccE16 );
            CPPUNIT_TEST(testTimeConversion );
//            CPPUNIT_TEST(testCopyConstructor );  // Disabled for some environment.  Using pointer is recommended.
            CPPUNIT_TEST(testCenaPacketArray );
            CPPUNIT_TEST(testCenaPacketArray2 );
            CPPUNIT_TEST(testHash);
	    CPPUNIT_TEST_SUITE_END();

        void testInstance(){
		uint8_t *stream=(uint8_t*) malloc (800);
		for(int i=0; i<800; i++){
			stream[i]=i;
		}

		CenaPacket cena=CenaPacket(stream, 800);

		CPPUNIT_ASSERT_EQUAL(800, (int)cena.getSize());

		free(stream);
        };

        void testMagicNumber(){
		uint8_t *stream=(uint8_t*) malloc (800);
		stream[0]=stream[1]=0xaa;
		stream[2]=stream[3]=0x55;
		for(int i=4; i<800; i++){
			stream[i]=i;
		}

		CenaPacket cena=CenaPacket(stream, 800);

		CPPUNIT_ASSERT_EQUAL(800, (int)cena.getSize());
		CPPUNIT_ASSERT_EQUAL((long)0xAAAA5555, (long)cena.getHeadMagicNumber());
 		CPPUNIT_ASSERT_EQUAL((long)CenaPacket::CENA_SCI_ID, (long)cena.getHeadMagicNumber());

		free(stream);
        }


	void testdump(){
		printf("\nDUMP A STREAM");

		// Prepare
		uint8_t *stream=(uint8_t*) malloc (800);
		// Initialize by a sequence.  Importance stream should be overwritten.
		for(int i=0; i<800; i++){
			stream[i]=i;
		}
		// Magic number
		stream[0]=stream[1]=0xaa;
		stream[2]=stream[3]=0x55;

		// Instance
		CenaPacket cena=CenaPacket(stream, 800);

		cena.dumpAll();

	}

	void testDecomp(){
		std::cout << "TEST DECOMP" << std::endl;
		uint8_t *stream=(uint8_t*) malloc (1000);
		stream[0]=stream[1]=0xaa;  // Set a 0magic number
		stream[2]=stream[3]=0x55;
		for(int i=4; i<1000; i++){
			stream[i]=i%256;
		}
		stream[24]=0xcd;  // All sensors ON, Comp OFF, and TM=CNT, SE=CNT
		stream[0x90]=stream[0x92]=0xaa;
		stream[0x91]=stream[0x93]=0xdd;
		CenaPacket* cena=new CenaPacket(stream, 800);

//		std::cout << cena->getModes() << std::endl;
//		cena->dumpAll();

		CenaPacket* deccena=cena->decompress();

		printf("%p %p\n", cena, deccena);

		CPPUNIT_ASSERT(cena!=deccena);
		
		delete cena;
		delete deccena;
		
		/// Compression on for original stream
		
		stream[0x90]=stream[0x92]=0xaa;
		stream[0x91]=stream[0x93]=0x55;
		stream[24]=0xed;  // All sensors ON, Comp ON, and TM=CNT, SE=CNT

		cena=new CenaPacket(stream, 1000);
		
		cena->dumpAll();
		deccena=cena->decompress();

		printf("%p %p\n", cena, deccena);

		deccena->dumpAll();

	}
	void test003();       // Implementation in .cpp file

	void testCenaMassAccE16(){
		std::cout << "E16" << std::endl;

		CenaPacket* cena=readCenaMassSample();
		
		cena->dumpAll();
		std::cout << cena->getSize() << std::endl;


		// Check the data read.
		CenaFastHkPacket *hk = cena->getFastHkPac();
		int hvtbl = hk->getSv1Table();

		printf("###### hvtbl ##### %d \n", hvtbl);
		CPPUNIT_ASSERT_EQUAL(2, hvtbl);

		CenaSciMassAccE16* ce16=cena->getCenaSciMassAccE16();

		CPPUNIT_ASSERT_EQUAL(16, ce16->getNrEne());

		int v0=ce16->get(0, 0, 0, 0);
		int v1=ce16->get(1, 0, 0, 0);
		int v2=ce16->get(2, 0, 0, 0);
		int v3=ce16->get(3, 0, 0, 0);
		int v4=ce16->get(4, 0, 0, 0);
		int v5=ce16->get(5, 0, 0, 0);
		int v6=ce16->get(6, 0, 0, 0);
		int v7=ce16->get(7, 0, 0, 0);
		int v8=ce16->get(8, 0, 0, 0);
		int v9=ce16->get(9, 0, 0, 0);
		int v10=ce16->get(10, 0, 0, 0);
		int v11=ce16->get(11, 0, 0, 0);
		int v12=ce16->get(12, 0, 0, 0);
		int v13=ce16->get(13, 0, 0, 0);
		int v14=ce16->get(14, 0, 0, 0);
		int v15=ce16->get(15, 0, 0, 0);

		// For SV table 4-11 is used.
		CPPUNIT_ASSERT_EQUAL(63488, v0);
		CPPUNIT_ASSERT_EQUAL(63488, v1);
		CPPUNIT_ASSERT_EQUAL(63488, v2);
		CPPUNIT_ASSERT_EQUAL(63488, v3);
		CPPUNIT_ASSERT_EQUAL(0, v4);
		CPPUNIT_ASSERT_EQUAL(0, v5);
		CPPUNIT_ASSERT_EQUAL(0, v6);
		CPPUNIT_ASSERT_EQUAL(0, v7);
		CPPUNIT_ASSERT_EQUAL(0, v8);
		CPPUNIT_ASSERT_EQUAL(0, v9);
		CPPUNIT_ASSERT_EQUAL(0, v10);
		CPPUNIT_ASSERT_EQUAL(0, v11);
		CPPUNIT_ASSERT_EQUAL(63488, v12);
		CPPUNIT_ASSERT_EQUAL(63488, v13);
		CPPUNIT_ASSERT_EQUAL(63488, v14);
		CPPUNIT_ASSERT_EQUAL(63488, v15);


		ce16->dumpAll();


		delete ce16;
	}
		
	void testCenaSciCntrAcc(){
		
		std::cout << "testCenaSciCntrAcc" << std::endl;
		// Test data
		uint8_t *stream=(uint8_t*) malloc (1500);
		stream[0]=stream[1]=0xaa;
		stream[2]=stream[3]=0x55;
		
		for(int i=4; i<1500; i++){
			stream[i]=i;
		}
		
		stream[24]=0xcd;  // All sensors ON, Comp OFF, and TM=CNT, SE=CNT
		stream[0x90]=stream[0x92]=0xaa;
		stream[0x91]=stream[0x93]=0xdd;
		
		CenaPacket cena=CenaPacket(stream, 1500);
		cena.dumpAll();
		
		CenaSciCntrAcc* sciData = cena.getCenaSciCntrAcc();
		
		std::cout << "SCIENCE DATA: size=" << sciData->getSize() << std::endl;
		sciData->dumpAll();
		
		CPPUNIT_ASSERT_EQUAL(990, (int)sciData->getSize());

	// If different mode is called NULL is returned.
		CenaSciMassAcc* sciMass = cena.getCenaSciMassAcc();
		CPPUNIT_ASSERT(sciMass==NULL);

		CenaSciTofAcc* sciTof = cena.getCenaSciTofAcc();
		CPPUNIT_ASSERT(sciTof==NULL);
		
	// If compressed data...
		stream[0x91]=stream[0x93]=0x55;
		CenaPacket cena2=CenaPacket(stream, 1500);
		
		try{
			CenaSciCntrAcc* sciData2 = cena2.getCenaSciCntrAcc();
			fprintf(stdout, "CenaPacket.getCenaSciCntrAcc() should throw exception (%p)\n", sciData2);
			CPPUNIT_ASSERT(false);
		} catch(char const* e){
			std::cout << "Exception caught!  " << e << std::endl;
		}

						
	}

	void testCenaSciCntrAcc2(){
		
		std::cout << "testCenaSciCntrAcc2" << std::endl;
		// Test data
		uint8_t *stream=(uint8_t*) malloc (1500);
		stream[0]=stream[1]=0xaa;
		stream[2]=stream[3]=0x55;
		
		for(int i=4; i<1500; i++){
			stream[i]=i;
		}
		
		stream[24]=0xcd;  // All sensors ON, Comp OFF, and TM=CNT, SE=CNT
		stream[0x90]=stream[0x92]=0xaa;
		stream[0x91]=stream[0x93]=0xdd;

		// nc=1, ne=4, np=8, nm=1
		// So binprm is 0 011 00111 0000000
		stream[20] = 0x33;
		stream[21] = 0x80;
		
		CenaPacket cena=CenaPacket(stream, 1500);
		cena.dumpAll();

		CenaMatrix* matx = cena.getCenaMatrix();

		CenaSciCntrAcc* sciData = (CenaSciCntrAcc*) matx->interpret();
		
		std::cout << "SCIENCE DATA: size=" << sciData->getSize() << std::endl;
		sciData->dumpAll();

		// Size should be 55(fix) * 4(ne) * 8(np) = 1760
		CPPUNIT_ASSERT_EQUAL(1760, (int)sciData->getSize());
		CPPUNIT_ASSERT_EQUAL(1760, (int)matx->getExpectedMatrixSize());
						
	}

	void testTimeConversion(){
		// Test data
		uint8_t *stream=(uint8_t*) malloc (1500);
		stream[0]=stream[1]=0xaa;
		stream[2]=stream[3]=0x55;
		
		for(int i=4; i<1500; i++){
			stream[i]=i;
		}

		// These values are taken from the actual data observed on 2009-01-30.
		stream[4]=0x54;  // W3
		stream[5]=0xfc;
		stream[6]=0x32;  // W4
		stream[7]=0x1f;
		stream[8]=0xbf;  // W5
		stream[9]=0x49;

		stream[10] = 0x61;
		stream[11] = 0xb4;

		stream[12] = 0x67;
		stream[13] = 0x00;

		CenaPacket cena=CenaPacket(stream, 1500);
//		cena.dumpAll();

		TimeConversionOffset* tc = new TimeConversionOffset(TimeConversionOffset::OFFSET(4));  // Start from 09-01-13
		double obstime=cena.getUniversalTime(tc);
		cout << obstime << endl;
		cout << epoch2str(obstime) << endl;
		
//		CPPUNIT_ASSERT_EQUAL(std::string("2009-01-30T12:00:37.207Z"), epoch2str(obstime));  // Disabled because of uncertain result....


	}
	
	/** This test case is not used because the copy constructor is not recommended, and not officially surpported.
 	*/
	void testCopyConstructor(){
		uint8_t *stream=(uint8_t*) malloc (1500);
		stream[0]=stream[1]=0xaa;
		stream[2]=stream[3]=0x55;
		
		for(int i=4; i<1500; i++){
			stream[i]=i;
		}

		// These values are taken from the actual data observed on 2009-01-30.
		stream[4]=0x54;  // W3
		stream[5]=0xfc;
		stream[6]=0x32;  // W4
		stream[7]=0x1f;
		stream[8]=0xbf;  // W5
		stream[9]=0x49;

		stream[10] = 0x61;
		stream[11] = 0xb4;

		stream[12] = 0x67;
		stream[13] = 0x00;

		CenaPacket cena(stream, 1500);

		try{
			cena.getUtc();
			CPPUNIT_ASSERT(false);
		}
		catch(std::runtime_error& e){
			std::cout << "Exception caught... OK" << std::endl;
		}
		cena.setUtc(1000000);
		double utc=cena.getUtc();
		std::cout << "UTC set =" << utc << std::endl;
		CPPUNIT_ASSERT_DOUBLES_EQUAL(1000000, utc, 1e-10);
		
		CenaPacket cena2(stream+100,1300);

		fprintf(stdout, "&CENA =%p\n", &cena);
		fprintf(stdout, "&CENA2=%p\n", &cena2);

		cena2=cena;

		fprintf(stdout, "&CENA2=%p\n", &cena2);

		CPPUNIT_ASSERT_EQUAL( (int) 1500, (int) cena2.getSize());
		CPPUNIT_ASSERT_DOUBLES_EQUAL(1000000, cena2.getUtc(), 1e-10);
		
		uint8_t* s=cena.getStream();
		uint8_t* s2=cena2.getStream();

		fprintf(stdout, "&s = %p\n", s);
		fprintf(stdout, "&s2 = %p\n", s2);
		for(int i=0; i<150; i++){
			CPPUNIT_ASSERT_EQUAL((int)s[i], (int)s2[i]);
		}

		CPPUNIT_ASSERT( (&cena)!=(&cena2));

		fprintf(stdout, "End of the test copy constructor\n");
		fflush(stdout);

	}
	void testHash(){
		int n_trial=1000;
		double clk=0;
		for(int i=0; i<n_trial; i++){
			// Prepare the stream.  7000 kbyte.
			uint8_t stream[7000];
			for(int j=0; j<7000; j++){
				stream[i] = random();
			}
			CenaPacket c(stream, 7000);
			clock_t start=clock();
			c.hash();
			clock_t stop=clock();
			clk += (stop-start);
		}
		cout << "Time of calculation of hash (sec/packet)  " << clk/CLOCKS_PER_SEC/n_trial << endl;

	}

	void testCenaPacketArray(){  // Old way.
                uint8_t *stream=(uint8_t*) malloc (1500);
                for(int i=0; i<1500; i++){
                        stream[i]=i;
                }

		CenaPacket* c1=new CenaPacket(stream, 100);
		c1->setUtc(1000);
		CenaPacket* c2=new CenaPacket(stream+100, 200);
		c2->setUtc(2000);
		CenaPacket* c3=new CenaPacket(stream+300, 300);
		c3->setUtc(3000);
		CenaPacket* c4=new CenaPacket(stream+600, 400);
		c4->setUtc(4000);
		CenaPacket* c5=new CenaPacket(stream+1000, 500);
		c5->setUtc(5000);

		CenaPacketArray cpa;
		cpa.append(c1);
		cpa.append(c2);
		cpa.append(c3);
		cpa.append(c4);
		cpa.append(c5);
	
		CPPUNIT_ASSERT_EQUAL(5, (int)cpa.size());		

		CenaPacket* p0=cpa.get(0);
		CenaPacket* p3=cpa.get(3);

		CPPUNIT_ASSERT_DOUBLES_EQUAL(1000, p0->getUtc(), 1e-5);
		CPPUNIT_ASSERT_DOUBLES_EQUAL(4000, p3->getUtc(), 1e-5);
		
		cpa.destroy();

		CPPUNIT_ASSERT_EQUAL(0, (int)cpa.size());
	}
	
	void testCenaPacketArray2(){  // New
                uint8_t *stream=(uint8_t*) malloc (1500);
                for(int i=0; i<1500; i++){
                        stream[i]=i;
                }

		CenaPacket* c1=new CenaPacket(stream, 100);
		c1->setUtc(1000);
		CenaPacket* c2=new CenaPacket(stream+100, 200);
		c2->setUtc(2000);
		CenaPacket* c3=new CenaPacket(stream+300, 300);
		c3->setUtc(3000);
		CenaPacket* c4=new CenaPacket(stream+600, 400);
		c4->setUtc(4000);
		CenaPacket* c5=new CenaPacket(stream+1000, 500);
		c5->setUtc(5000);

		CenaPacketArray cpa;
		cpa.append(c1);
		cpa.append(c2);
		cpa.append(c3);
		cpa.append(c4);
		cpa.append(c5);
	
		CPPUNIT_ASSERT_EQUAL(5, (int)cpa.size());		

		CenaPacket* p0=cpa.getPacket(0);
		CenaPacket* p3=cpa.getPacket(3);

		CPPUNIT_ASSERT_DOUBLES_EQUAL(1000, p0->getUtc(), 1e-5);
		CPPUNIT_ASSERT_DOUBLES_EQUAL(4000, p3->getUtc(), 1e-5);

		delete p0;
		delete p3;
		cpa.destroy();

		CPPUNIT_ASSERT_EQUAL(0, (int)cpa.size());

		CPPUNIT_ASSERT_DOUBLES_EQUAL(1000, c1->getUtc(), 1e-5);
	}
	
};

CPPUNIT_TEST_SUITE_REGISTRATION (CenaPacket_test);
