#include<iostream>

#include<cppunit/extensions/HelperMacros.h>

#include"IrfPac/irfpac.h"
#include"IrfPac/SaraPac/sarapac.h"

using namespace std;
using namespace IrfPac;
using namespace IrfPac::SaraPac;

class CenaSciCntrAcc_test : public CPPUNIT_NS::TestFixture {
	
public:
	CPPUNIT_TEST_SUITE(CenaSciCntrAcc_test);
	CPPUNIT_TEST(testDirectInstance);
	CPPUNIT_TEST(testIndex);
	CPPUNIT_TEST(testZeroIndex);
	CPPUNIT_TEST_SUITE_END();
	
	void testDirectInstance();

	void testZeroIndex(){
		CPPUNIT_ASSERT_EQUAL(28, CenaSciCntrAcc::IDX0_START_RING(0));
		CPPUNIT_ASSERT_EQUAL(31, CenaSciCntrAcc::IDX0_START_RING(3));
		try{
			CenaSciCntrAcc::IDX0_START_RING(4);
			fprintf(stdout, "** Failed to throw exception.\n");
			CPPUNIT_ASSERT(false);
		}
		catch(const std::runtime_error& e){
			fprintf(stdout, "Correct exception caught (%s)\n", e.what());
		}
		catch(...){
			fprintf(stdout, "** Failed to catch exception.\n");
			CPPUNIT_ASSERT(false);
		}

		CPPUNIT_ASSERT_EQUAL(0, CenaSciCntrAcc::IDX0_START_RING_SECTOR(0,0));
		CPPUNIT_ASSERT_EQUAL(1, CenaSciCntrAcc::IDX0_START_RING_SECTOR(1,0));
		CPPUNIT_ASSERT_EQUAL(4, CenaSciCntrAcc::IDX0_START_RING_SECTOR(0,1));
		CPPUNIT_ASSERT_EQUAL(27, CenaSciCntrAcc::IDX0_START_RING_SECTOR(3,6));
		try{
			CenaSciCntrAcc::IDX0_START_RING_SECTOR(3,7);
			fprintf(stdout, "** Failed to throw exception.\n");
			CPPUNIT_ASSERT(false);
		}
		catch(const std::runtime_error& e){
			fprintf(stdout, "Correct exception caught (%s)\n", e.what());
		}
		catch(...){
			fprintf(stdout, "** Failed to catch exception.\n");
			CPPUNIT_ASSERT(false);
		}

		CPPUNIT_ASSERT_EQUAL(32, CenaSciCntrAcc::IDX0_START_SECTOR(0));
		CPPUNIT_ASSERT_EQUAL(38, CenaSciCntrAcc::IDX0_START_SECTOR(6));
		try{
			CenaSciCntrAcc::IDX0_START_SECTOR(7);
			fprintf(stdout, "** Failed to throw exception.\n");
			CPPUNIT_ASSERT(false);
		}
		catch(const std::runtime_error& e){
			fprintf(stdout, "Correct exception caught (%s)\n", e.what());
		}
		catch(...){
			fprintf(stdout, "** Failed to catch exception.\n");
			CPPUNIT_ASSERT(false);
		}


		CPPUNIT_ASSERT_EQUAL(39, CenaSciCntrAcc::IDX0_STOP_PLATE(0));
		CPPUNIT_ASSERT_EQUAL(46, CenaSciCntrAcc::IDX0_STOP_PLATE(7));
		CPPUNIT_ASSERT_EQUAL(47, CenaSciCntrAcc::IDX0_STOP_MESH());
		try{
			CenaSciCntrAcc::IDX0_STOP_PLATE(8);
			fprintf(stdout, "** Failed to throw exception.\n");
			CPPUNIT_ASSERT(false);
		}
		catch(const std::runtime_error& e){
			fprintf(stdout, "Correct exception caught (%s)\n", e.what());
		}
		catch(...){
			fprintf(stdout, "** Failed to catch exception.\n");
			CPPUNIT_ASSERT(false);
		}

		CPPUNIT_ASSERT_EQUAL(48, CenaSciCntrAcc::IDX0_COIN_SECTOR(0));
		CPPUNIT_ASSERT_EQUAL(54, CenaSciCntrAcc::IDX0_COIN_SECTOR(6));
		try{
			CenaSciCntrAcc::IDX0_COIN_SECTOR(7);
			fprintf(stdout, "** Failed to throw exception.\n");
			CPPUNIT_ASSERT(false);
		}
		catch(const std::runtime_error& e){
			fprintf(stdout, "Correct exception caught (%s)\n", e.what());
		}
		catch(...){
			fprintf(stdout, "** Failed to catch exception.\n");
			CPPUNIT_ASSERT(false);
		}


	}
	
	void testIndex(){
		CPPUNIT_ASSERT_EQUAL(0, CenaSciCntrAcc::START_RING_SECTOR(1,1));
		CPPUNIT_ASSERT_EQUAL(1, CenaSciCntrAcc::START_RING_SECTOR(2,1));
		CPPUNIT_ASSERT_EQUAL(4, CenaSciCntrAcc::START_RING_SECTOR(1,2));
		CPPUNIT_ASSERT_EQUAL(27, CenaSciCntrAcc::START_RING_SECTOR(4,7));
		CPPUNIT_ASSERT_EQUAL(28, CenaSciCntrAcc::START_RING(1));
		CPPUNIT_ASSERT_EQUAL(31, CenaSciCntrAcc::START_RING(4));
		CPPUNIT_ASSERT_EQUAL(32, CenaSciCntrAcc::START_SECTOR(1));
		CPPUNIT_ASSERT_EQUAL(38, CenaSciCntrAcc::START_SECTOR(7));
		CPPUNIT_ASSERT_EQUAL(39, CenaSciCntrAcc::STOP_PLATE(1));
		CPPUNIT_ASSERT_EQUAL(46, CenaSciCntrAcc::STOP_PLATE(8));
		CPPUNIT_ASSERT_EQUAL(47, CenaSciCntrAcc::STOP_MESH());
		CPPUNIT_ASSERT_EQUAL(48, CenaSciCntrAcc::COIN_SECTOR(1));
		CPPUNIT_ASSERT_EQUAL(54, CenaSciCntrAcc::COIN_SECTOR(7));
		
		try{
			CenaSciCntrAcc::START_RING_SECTOR(0,0);
			CPPUNIT_ASSERT(false);
		}
		catch(char const* er){
			std::cout << "FINE!" << er << std::endl;
		}
		catch(...){
			CPPUNIT_ASSERT(false);
		}
		
	}
	
};

void CenaSciCntrAcc_test::testDirectInstance(){
	// Assuming E=4, P=8, C=55

	uint8_t stream[4*8*55];
	for(int i=0; i<4*8*55; i++){
		stream[i]=i;
	}

	CenaSciCntrAcc cena=CenaSciCntrAcc(stream, 4, 8, 55);
	
	cout << (int)cena.getSize() << endl;
	CPPUNIT_ASSERT_EQUAL(4*8*55, (int)cena.getSize());

	cena.dumpAll();
	
}

CPPUNIT_TEST_SUITE_REGISTRATION(CenaSciCntrAcc_test);
