/*
 *  CenaSciMassAcc.cpp
 *  libsadppac
 *
 *  Created by Futaana Yoshifumi on 07/06/26.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "IrfPac/SaraPac/CenaSciMassAcc.h"
#include <linlog++.h>

namespace IrfPac{
namespace SaraPac{

CenaSciMassAcc::CenaSciMassAcc(uint8_t *pac, int nEnergy, int nPhase, int nC, int nMass)
: CenaScienceDataPacket(pac, nEnergy*nPhase*((nC==7)?8:nC)*nMass){
	_ne=nEnergy;
	_np=nPhase;
	_nc=nC;
	_nm=nMass;
    _nc_padd=(nC==7)?8:nC;  // ugly fix for channel padding for case with 7 channels.  
}

int CenaSciMassAcc::indexOf(int iE, int iP, int iC, int iM){
	return iE*_np*_nc_padd*_nm + iP*_nc_padd*_nm + iC*_nm + iM;
}

int CenaSciMassAccE16::indexOf(int iE16, int iP, int iC, int iM){
	if(iE16 < 0 || iE16 >= 16 || iP < 0 || iP >= _np || iC < 0 || iC >= _nc || iM < 0 || iM>= _nm){
		throw std::range_error("Given parameter is out of range");
	}
	
	// For all (1-3) the table.
	if(iE16 == 0){
		return -1;
	}

//	printf("E16::  iE16 = %d\n", iE16);	
//	printf("E16::  hv = %d\n", _hv);	

	int iE=-1;
	// For table 1
	if(_sv == 0){  // _sv 0 should not throw an exception.
		return -1;
	}
	else if(_sv == 1){
		if(iE16 >= 9){
			return -1;
		}
		iE = 8 - iE16;
	}
	else if (_sv == 2){
		if(iE16 >= 12 || iE16 <= 3){
			return -1;
		}
		iE = 11 - iE16;
	}
	else if (_sv == 3){
		if(iE16 <= 7){
			return -1;
		}
		iE = 15 - iE16;
	}
	else{
		std::cerr << "Invalid SV table is specified :	" << _sv << std::endl;
		throw std::runtime_error("Invalid SV is specified");
	}
	return iE*_np*_nc_padd*_nm + iP*_nc_padd*_nm + iC*_nm + iM;
	
}

std::vector<double> CenaSciMassAcc::getCountAsArray(){
	int size=_ne*_np*_nc_padd*_nm;
#ifdef DEBUG
	fprintf(stderr, "size= %d\n", size);
#endif
	std::vector<double> arr(size);

	for(int ie=0; ie<_ne; ie++){
		for(int ip=0; ip<_np; ip++){
			for(int ic=0; ic<_nc; ic++){
				for(int im=0; im<_nm; im++){
					arr[indexOf(ie,ip,ic,im)] = get(ie,ip,ic,im);
					
				}
			}
		}
	}

	return arr;
}


std::vector<double> CenaSciMassAccE16::getCountAsArray(){
	throw std::runtime_error("Not yet supported.");
	int ne = getNrEne();

	int size=ne*_np*_nc_padd*_nm;
#ifdef DEBUG
	fprintf(stderr, "size= %d\n", size);
#endif
	std::vector<double> arr(size);

	for(int ie=0; ie<ne; ie++){
		for(int ip=0; ip<_np; ip++){
			for(int ic=0; ic<_nc; ic++){
				for(int im=0; im<_nm; im++){
					arr[ie*_np*_nc_padd*_nm+ip*_nc_padd*_nm+ic*_nm+im] = get(ie,ip,ic,im);
				}
			}
		}
	}
	return arr;
}

uint8_t CenaSciMassAcc::getRaw(int iE, int iP, int iC, int iM){
	int index = indexOf(iE, iP, iC, iM);
	return getValue(index);
}

uint16_t CenaSciMassAcc::get(int iE, int iP, int iC, int iM){
	uint8_t raw=getRaw(iE, iP, iC, iM);
	return log8tolinear16(raw);
}


// End of namespaces
}}

