#include<iostream>
using namespace std;
#include<linlog++.h>

#include<cmath>
#include<cppunit/extensions/HelperMacros.h>

#include"IrfPac/irfpac.h"
#include"IrfPac/SaraPac/sarapac.h"

using namespace IrfPac;
using namespace IrfPac::SaraPac;

class CenaSciMassAcc_test : public CPPUNIT_NS::TestFixture {
	
public:
	CPPUNIT_TEST_SUITE(CenaSciMassAcc_test);
	CPPUNIT_TEST(testDirectInstance);
	CPPUNIT_TEST(testValarray);
	CPPUNIT_TEST(testValarrayC7);
	CPPUNIT_TEST(testValarrayE16);
	CPPUNIT_TEST_SUITE_END();
	
	void testDirectInstance();
	void testValarray();
	void testValarrayC7();
	void testValarrayE16();
	
};

void CenaSciMassAcc_test::testValarrayC7(){
	// Assuming the maximum mode for Cena. nC=7, but internal expression nCpadd=8.
	int ne=8, np=1, nc=7, nci=8, nm=128;
	int pacsize=ne*np*nci*nm;

	uint8_t stream[pacsize];
	for(int i=0; i<pacsize; i++){
		stream[i]=i;
	}

	CenaSciMassAcc cena=CenaSciMassAcc(stream, ne, np, nci, nm);
	
	cout << (int)cena.getSize() << endl;
	CPPUNIT_ASSERT_EQUAL(pacsize, (int)cena.getSize());

	cena.dumpAll();

	std::vector<double> matx = cena.getCountAsArray();
	CPPUNIT_ASSERT_EQUAL(pacsize, (int)matx.size());

	for(int ie=0; ie<ne; ie++){
		for(int ip=0; ip<np; ip++){
			for(int ic=0; ic<nc; ic++){
				for(int im=0; im<nm; im++){
					fprintf(stdout, "%d %d %d %d: %d: %f\n", ie, ip, ic, im, cena.indexOf(ie,ip,ic,im), matx[ cena.indexOf(ie,ip,ic,im) ]);
					CPPUNIT_ASSERT_EQUAL(log8tolinear16(ie*np*nci*nm+ip*nci*nm+ic*nm+im), (int)matx[cena.indexOf(ie,ip,ic,im)]);
				}
			}
		}
	}
}


void CenaSciMassAcc_test::testValarrayE16(){
	// Assuming the maximum mode for Cena. nC=7, but internal expression nCpadd=8.
	int ne=8, np=1, nc=7, nci=8, nm=128;
	int pacsize=ne*np*nci*nm;

	uint8_t stream[pacsize];
	for(int i=0; i<pacsize; i++){
		stream[i]=i % 100;
	}

	try{
		CenaSciMassAcc *cenaFail = new CenaSciMassAcc(stream, 1, 1, 7, 128);
		CenaSciMassAccE16 *cenaFail16 = new CenaSciMassAccE16(cenaFail, 1);
		cerr << "You cannot come here!!!" << endl;
		CPPUNIT_ASSERT(false);
	} catch(std::runtime_error& e){
	}

	CenaSciMassAcc *cena=new CenaSciMassAcc(stream, ne, np, nc, nm);
	
	cout << (int)cena->getSize() << endl;
	CPPUNIT_ASSERT_EQUAL(pacsize, (int)cena->getSize());

	CenaSciMassAccE16 *cenaE16 = new CenaSciMassAccE16(cena, 1);   // 1 is the table 1.

	for(int ic=0; ic<7; ic++){
		for(int im=0; im<128; im++){
			// For table 1, the energy bin 0 (E0) is the energy index 8.
			CPPUNIT_ASSERT_EQUAL(cena->get(0, 0, ic, im), cenaE16->get(8, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->get(1, 0, ic, im), cenaE16->get(7, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->get(2, 0, ic, im), cenaE16->get(6, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->get(3, 0, ic, im), cenaE16->get(5, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->get(4, 0, ic, im), cenaE16->get(4, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->get(5, 0, ic, im), cenaE16->get(3, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->get(6, 0, ic, im), cenaE16->get(2, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->get(7, 0, ic, im), cenaE16->get(1, 0, ic, im));

			CPPUNIT_ASSERT_EQUAL(cena->getRaw(0, 0, ic, im), cenaE16->getRaw(8, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->getRaw(1, 0, ic, im), cenaE16->getRaw(7, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->getRaw(2, 0, ic, im), cenaE16->getRaw(6, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->getRaw(3, 0, ic, im), cenaE16->getRaw(5, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->getRaw(4, 0, ic, im), cenaE16->getRaw(4, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->getRaw(5, 0, ic, im), cenaE16->getRaw(3, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->getRaw(6, 0, ic, im), cenaE16->getRaw(2, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL(cena->getRaw(7, 0, ic, im), cenaE16->getRaw(1, 0, ic, im));

			CPPUNIT_ASSERT_EQUAL((int)255, (int)cenaE16->getRaw(0, 0, ic, im));
			CPPUNIT_ASSERT_EQUAL((int)63488, (int)cenaE16->get(0, 0, ic, im));
			for(int ie=9; ie<16; ie++){
				CPPUNIT_ASSERT_EQUAL((int)255, (int)cenaE16->getRaw(ie, 0, ic, im));
			}
		}
	}

	try{
		cenaE16->getRaw(16, 0, 0, 0);
		CPPUNIT_ASSERT(false);
	} catch (std::runtime_error &e){
	}
	
	cenaE16->dumpAll();

}

void CenaSciMassAcc_test::testValarray(){
	// Assuming the snmple case.  nc=4 means no padding.
	int ne=8, np=1, nc=4, nm=128;
	int pacsize=ne*np*nc*nm;

	uint8_t stream[pacsize];
	for(int i=0; i<pacsize; i++){
		stream[i]=(uint8_t)sqrt(i);
	}

	CenaSciMassAcc cena=CenaSciMassAcc(stream, ne, np, nc, nm);
	
	cout << (int)cena.getSize() << endl;
	CPPUNIT_ASSERT_EQUAL(pacsize, (int)cena.getSize());

	std::vector<double> matx = cena.getCountAsArray();
	CPPUNIT_ASSERT_EQUAL(pacsize, (int)matx.size());

	for(int ie=0; ie<ne; ie++){
		for(int ip=0; ip<np; ip++){
			for(int ic=0; ic<nc; ic++){
				for(int im=0; im<nm; im++){
					fprintf(stdout, "%d %d %d %d: %f\n", ie, ip, ic, im, matx[ cena.indexOf(ie,ip,ic,im) ]);
				}
			}
		}
	}


}

void CenaSciMassAcc_test::testDirectInstance(){
	// Assuming E=4, P=8, C=2, M=16

	uint8_t stream[4*8*2*16];
	for(int i=0; i<4*8*2*16; i++){
		stream[i]=i;
	}

	CenaSciMassAcc cena=CenaSciMassAcc(stream, 4, 8, 2, 16);
	
	cout << (int)cena.getSize() << endl;
	CPPUNIT_ASSERT_EQUAL(4*8*2*16, (int)cena.getSize());
	
}

CPPUNIT_TEST_SUITE_REGISTRATION(CenaSciMassAcc_test);
