#ifndef IRFPAC_SARAPACCENASUBCHKPACKET_H
#define IRFPAC_SARAPACCENASUBCHKPACKET_H

#include <IrfPac/irfpac.h>
#include <IrfPac/SaraPac/SaraPacket.h>

namespace IrfPac {

namespace SaraPac {

	/**	@brief Implementation of CENA HK packet subcommuted in
		CENA science packet.

		The definition of the packet contents is described
		in Table 19 in CHA-SARA-DS-0006.

		@deprecated CenaSubcHkPacket class is deprecated because
		the name 'subc' confuses users.
		The new class CenaFastHkPacket can be used.
	*/
        class CenaSubcHkPacket : public PacketU8Arr
        {
        public:
		DEPRECATED
	        CenaSubcHkPacket(uint8_t* stream, size_t size);
		DEPRECATED
	        CenaSubcHkPacket(uint8_t* stream);

		DEPRECATED
	        ~CenaSubcHkPacket();

		/** @brief Dump a detailed information into stdout.
		*/
		DEPRECATED
		void dumpAll(void){dumpAll(stdout);}
		/** @brief Dump a detailed information into specified file.
		*/
		DEPRECATED
		void dumpAll(FILE* fp);

		/** @brief return Flag P */
		DEPRECATED
		bool getFlagHvMain(){return getBitFlag(0);}
		/** @brief return Flag S */
		DEPRECATED
		bool getFlagHvSafety(){return getBitFlag(1);} 
		/** @brief return Flag PE */
		DEPRECATED
		bool getFlagHvOnEna(){return getBitFlag(4);} 
		/** @brief return Flag SE */
		DEPRECATED
		bool getFlagHvLvlEna(){return getBitFlag(5);} 
		/** @brief return Flag ER */
		DEPRECATED
		bool getFlagError(){return getBitFlag(7);} 

		DEPRECATED
		uint16_t getHvMain(); //!< DV2-3, H1Level
		DEPRECATED
		uint16_t getHvStartMcp();  //!< DV4-5, H2Level
		DEPRECATED
		uint16_t getHvStopMcp();   //!< DV6-7, H3Level
		DEPRECATED
		uint16_t getHvTof();      //!< DV8-9, H4Level
		DEPRECATED
		uint16_t getHvDef();      //!< DV10-11, H5Level
		DEPRECATED
		uint16_t getTempIFE();    //!< DV12-13, TempIFE
		DEPRECATED
		uint16_t getTempHVPS();   //!< DV14-15,TempHVPS

		DEPRECATED
		uint16_t getPsyncCntr();  //!< DV34-35, Number of sync pulses received from DPU since IFE power on
		DEPRECATED
		uint16_t getBaseCntr();   //!< DV36-37, Number of sync pulse intervals clapsed since IFE power on

		///////// Below (DV16-) is not yet implemented.
		///////// Implementation would also be needed to swig interface file

#ifdef DEBUG
		DEPRECATED
		static uint16_t test12b(uint8_t u, uint8_t l){
		  return get12bValue(u,l);
		} 
		DEPRECATED
		static bool testbit(uint8_t val, size_t pos){
		  return getBitAt(val,pos);
		} 
		
#endif

	private:
		static uint16_t get12bValue(uint8_t upper, uint8_t lower){
		  uint16_t val=0;
		  val=val|(((uint16_t)upper & 0x000f) << 8);
		  val=val|(((uint16_t)lower & 0x00ff));
		  return val;
		}

		static uint16_t get16bValue(uint8_t upper, uint8_t lower){
		  uint16_t val=0;
		  val=val|(((uint16_t)upper & 0x00ff) << 8);
		  val=val|(((uint16_t)lower & 0x00ff));
		  return val;
		}

		/**@brief Returns 12 bit value constitute of idx0-th and idx1-th bytes.
		 */
		uint16_t get12bitValue(int idx0, int idx1){
		  uint8_t u = IrfPac::SaraPac::getByteAt(*this, (size_t)idx0);
		  uint8_t l = IrfPac::SaraPac::getByteAt(*this, (size_t)idx1);
		  return get12bValue(u,l);
		}

		uint16_t get16bitValue(int idx0, int idx1){
		  uint8_t u = IrfPac::SaraPac::getByteAt(*this, (size_t)idx0);
		  uint8_t l = IrfPac::SaraPac::getByteAt(*this, (size_t)idx1);
		  return get16bValue(u,l);
		}

		static bool getBitAt(uint8_t value, size_t position){
		  return (value & (1 << position)) >> position;
		}	// Only used from getBitFlag


		/**@brief Returns the bit flag of the DV1 */
		bool getBitFlag(int position){
		  uint8_t v=IrfPac::SaraPac::getByteAt(*this, (size_t)0);
		  return getBitAt(v, (size_t)position);
		}

        };

}

}

#endif
