/*
 *  SaraBufferedDemultiplexor.cpp
 *  libsadppac
 *
 *  Created by Futaana Yoshifumi on 07/10/23.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "IrfPac/SaraPac/SaraBufferedDemultiplexor.h"

using namespace std;

namespace IrfPac{
namespace SaraPac{

SaraBufferedDemultiplexor::SaraBufferedDemultiplexor(){
	demul = new SaraDemultiplexor();
#ifdef DEBUG
	cout << "--- " << cenaBuffer.size() << endl;
#endif
}

SaraBufferedDemultiplexor::~SaraBufferedDemultiplexor(){
	flushCena();
	flushSwim();
	delete demul;
}
		
void SaraBufferedDemultiplexor::appendSaraPacket(SaraPacket* saraPac){
	#ifdef DEBUG
		cout << "Appending SARA Packet" << endl;
	#endif
	
	if(!demul->sequenceCheck(saraPac)){
		cerr << "*** SEQUENCIAL CHECK NG : FLUSH BUFFER ***" << endl;
		demul->flushAll();
		return;
	}
	CenaPacket* cenaPac=demul->appendCena(saraPac);
	SwimPacket* swimPac=demul->appendSwim(saraPac);

	if(cenaPac != NULL){
		#ifdef DEBUG
			cout << "+++ Adding CenaPacket to cenaBuffer +++" << endl;
		#endif
		cenaBuffer.push_back(cenaPac);
	}
	if(swimPac != NULL){
		swimBuffer.push_back(swimPac);
	}
}

int SaraBufferedDemultiplexor::getNrCena(){
	return cenaBuffer.size();
}

int SaraBufferedDemultiplexor::getNrSwim(){
	return swimBuffer.size();
}

CenaPacket* SaraBufferedDemultiplexor::getCena(){
	if(cenaBuffer.size()==0){  // If queue size is 0, NULL is returned.
		return NULL;
	}
	CenaPacket* cPac=cenaBuffer.front();  // Obtain the first element
	#ifdef DEBUG
		cout << "+++ CENA PACKET +++" << endl;
	#endif
	cenaBuffer.pop_front();  // Remove the first element
	return cPac;
}

CenaPacket* SaraBufferedDemultiplexor::getLatestCena(){
	if(cenaBuffer.size()==0){  // If queue size is 0, NULL is returned.
		return NULL;
	}
	CenaPacket* cPac=cenaBuffer.back();  // Obtain the latest element
	cenaBuffer.pop_back();  // Remove the latest element
	#ifdef DEBUG 
		cout << "+++ CENA PACKET +++" << endl;
	#endif
	flushCena();  // Destroy all the buffered elements and flush the buffer.
	return cPac;
}

SwimPacket* SaraBufferedDemultiplexor::getSwim(){
	if(swimBuffer.size()==0){  // If queue size is 0, NULL is returned.
		return NULL;
	}
	SwimPacket* cPac=swimBuffer.front();
	swimBuffer.pop_front();
	return cPac;
}
SwimPacket* SaraBufferedDemultiplexor::getLatestSwim(){
	if(swimBuffer.size()==0){  // If queue size is 0, NULL is returned.
		return NULL;
	}
	SwimPacket* cPac=swimBuffer.back();
	swimBuffer.pop_back();
	flushSwim();
	return cPac;
}

void SaraBufferedDemultiplexor::flushCena(void){
	// Flushing the cenaBuffer, it is important to delete CenaPacket instances
	// in order to avoid memory leakage.

	while(cenaBuffer.size() > 0){
		CenaPacket* cPac=cenaBuffer.front();
		cenaBuffer.pop_front();
		delete cPac;
	}
	
}

void SaraBufferedDemultiplexor::flushSwim(void){
	while(swimBuffer.size() > 0){
		SwimPacket* cPac=swimBuffer.front();
		swimBuffer.pop_front();
		delete cPac;
	}
}


}}//namespace

