/*
 *  SaraDemultiplexor.cpp
 *  libsadppac
 *
 *  Created by Futaana Yoshifumi on 07/06/28.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "IrfPac/SaraPac/SaraDemultiplexor.h"

////////////////// Definition fo SasCache


namespace IrfPac{
namespace SaraPac{

SasCache::SasCache(){
//	_pac=(PacketU8Arr*)NULL;
	__pac=new uint8_t[0];
	__pac_size=0;
	__pac_max_size=0;
}

SasCache::~SasCache(){
	if(__pac!=NULL){
		delete[] __pac;
	}
}

bool SasCache::isEmpty(){
	return (__pac_size==0);
}

uint8_t* SasCache::flush(){
	uint8_t* pac=getPacket();
	delete[] __pac;
	__pac=new uint8_t[0];
	__pac_size=0;
	__pac_max_size=0;
	
	return pac;
}

uint8_t* SasCache::getPacket(){
	uint8_t *pac=new uint8_t[__pac_size];
	for(size_t i=0; i<__pac_size; i++){
		pac[i] = __pac[i];
	}
	return pac;
}

size_t SasCache::append(GeneralPacket* saraPac){
	uint8_t* contToAdd=saraPac->getStream();
	int sizeToAdd=saraPac->getSize();
	return append(contToAdd, sizeToAdd);
}

size_t SasCache::append(uint8_t *contToAdd, size_t sizeToAdd){
	
	if(__pac_size + sizeToAdd > __pac_max_size){
		uint8_t *neopac=new uint8_t[__pac_size+sizeToAdd];
		for(size_t i=0; i<__pac_size; i++){
			neopac[i] = __pac[i];
		}
		delete[] __pac;
		for(size_t i=__pac_size; i<__pac_size+sizeToAdd; i++){
			neopac[i] = contToAdd[i-__pac_size];
		}
		__pac=neopac;
		__pac_size=__pac_size+sizeToAdd;
		__pac_max_size=__pac_size;
	} else {
		for(size_t i=__pac_size; i<__pac_size+sizeToAdd; i++){
			__pac[i] = contToAdd[i-__pac_size];
		}
		__pac_size=__pac_size+sizeToAdd;
	}

	return __pac_size;
}

// End of namespace 
}}

////////////////// Definition fo SaraDemultiplexor

namespace IrfPac{
namespace SaraPac{


SaraDemultiplexor::SaraDemultiplexor(){
	cenaCache=new SasCache();
	swimCache=new SasCache();
	previousPacketCntr=0xffff;
	firstpacket=true;
};

SaraDemultiplexor::~SaraDemultiplexor(){
	delete cenaCache;
	delete swimCache;
};

/** @brief Append the SaraPacket to the CENA cache.  Returns the completed packet that has been cached.

	This method controls caching and returning the completed packet.
	We know the completion of packet only when the new packet arrived.
	No sequencial check is done in this method.  That should be done manually.
	Look the sample code in SaraDemultiplexor_sample.cpp file.
*/
CenaPacket* SaraDemultiplexor::appendCena(SaraPacket* saraPac){

//	uint16_t pacId=saraPac->getPacketId();
	uint8_t *cena_buff=saraPac->getCenaByteStream();
	size_t cena_buff_size=saraPac->getCenaByteStreamSize();
	bool cacheEmpty = cenaCache->isEmpty();  // Check whether the cache is empty
	CenaPacket *cenaPac = (CenaPacket*) NULL;

	if(cena_buff != NULL){
		uint32_t head=(cena_buff[0]<<24) + (cena_buff[1] <<16) +(cena_buff[2]<<8) + cena_buff[3];

		#ifdef DEBUG
			fprintf(stderr, "CENA APPEND: 0x%08x\n", head);
		#endif

		bool isStart=(head == CenaPacket::CENA_SCI_ID);  // Check whether the packet is the start
		
		// If arrival packet is the start, and cache is empty ....
		//		We can consider that this is the start of the CenaPacket.
		//				->Just append the contents 
		if(isStart && cacheEmpty){
			#ifdef DEBUG
				std::cout << "%%%C START/EMPTY (append) %%% " ;
				std::cout << cenaCache->getCacheSize() << " -> ";
			#endif

			cenaCache->append(cena_buff, cena_buff_size);

			#ifdef DEBUG
				std::cout << cenaCache->getCacheSize() << std::endl;
			#endif
		}
		// If arrival packet is the start, and cache is not empty ....
		//		The arrival packet is a part of new CenaPacket, and the existing cache is the completion of CenaPacket.
		//				->Create new CenaPacket to return from the cache, and flush the cache, and append buffer
		else if(isStart && !cacheEmpty){
			#ifdef DEBUG
				std::cout << "%%%C START/EXIST (flush/append) %%% " ;
				std::cout << cenaCache->getCacheSize() << " -> ";
			#endif

			size_t cenaStreamSize = cenaCache->getSize();
			uint8_t* cenaStream = cenaCache->flush();
			cenaPac = new CenaPacket(cenaStream, cenaStreamSize);
			delete[] cenaStream;
			cenaCache->append(cena_buff, cena_buff_size);

			#ifdef DEBUG
				std::cout << cenaCache->getCacheSize() << std::endl;
			#endif
		}
		// If arrival packet is not start, and cache is empty
		//		This should not be the case.
		//			-> Ignore the packet.
		else if(!isStart && cacheEmpty){
			#ifdef DEBUG
				std::cout << "%%%C CONT/EMPTY %%% should not occur..." << std::endl;
			#endif
//			std::cerr << "Illegal packet arrived to CENA.  Packet " << saraPac->getCounter() << std::endl;
			fprintf(stderr, "Illegal packet arrived to CENA, PACKET %x\n", saraPac->getCounter());
			fprintf(stderr, "*** CONTENTS OF PACKET ***\n");
			saraPac->dumpHexStream(stderr);
			fprintf(stderr, "**************************\n");
		}
		// If arrival packet is not start, and cache is not empty
		//      This is the continuation of the packet.  We don't know the completion of packet.
		//          -> Append to the cache.
		else if(!isStart && !cacheEmpty){
			#ifdef DEBUG
				std::cout << "%%%C CONT/EXIST (append) %%% " ;
				std::cout << cenaCache->getCacheSize() << " -> ";
			#endif

			cenaCache->append(cena_buff, cena_buff_size);

			#ifdef DEBUG
				std::cout << cenaCache->getCacheSize() << std::endl;
			#endif
		}
		
		
		delete[] cena_buff;
	}

	return cenaPac;
}

SwimPacket* SaraDemultiplexor::appendSwim(SaraPacket* saraPac){

//	uint16_t pacId=saraPac->getPacketId();
	uint8_t *swim_buff=saraPac->getSwimByteStream();
	size_t swim_buff_size=saraPac->getSwimByteStreamSize();
	bool cacheEmpty = swimCache->isEmpty();  // Check whether the cache is empty
	
	SwimPacket *swimPac = (SwimPacket*) NULL;

	if(swim_buff != NULL){
		uint32_t head=(swim_buff[0]<<24) + (swim_buff[1] <<16) +(swim_buff[2]<<8) + swim_buff[3];

		#ifdef DEBUG
			fprintf(stderr, "SWIM APPEND: 0x%08x\n", head);
		#endif

		bool isStart=(head == SwimPacket::SWIM_SCI_ID);  // Check whether the packet is the start
		
		// If arrival packet is the start, and cache is empty ....
		//		We can consider that this is the start of the SwimPacket.
		//				->Just append the contents 
		if(isStart && cacheEmpty){
			#ifdef DEBUG
				std::cout << "%%%S START/EMPTY (append) %%% " ;
				std::cout << swimCache->getCacheSize() << " -> ";
			#endif

			swimCache->append(swim_buff, swim_buff_size);

			#ifdef DEBUG
				std::cout << swimCache->getCacheSize() << std::endl;
			#endif
		}
		// If arrival packet is the start, and cache is not empty ....
		//		The arrival packet is a part of new SwimPacket, and the existing cache is the completion of SwimPacket.
		//				->Create new SwimPacket to return from the cache, and flush the cache, and append buffer
		else if(isStart && !cacheEmpty){
			#ifdef DEBUG
				std::cout << "%%%S START/EXIST (flush/append) %%% " ;
				std::cout << swimCache->getCacheSize() << " -> ";
			#endif

			size_t swimStreamSize = swimCache->getSize();
			uint8_t* swimStream = swimCache->flush();
			swimPac = new SwimPacket(swimStream, swimStreamSize);
			delete[] swimStream;
			swimCache->append(swim_buff, swim_buff_size);

			#ifdef DEBUG
				std::cout << swimCache->getCacheSize() << std::endl;
			#endif
		}
		// If arrival packet is not start, and cache is empty
		//		This should not be the case.
		//			-> Ignore the packet.
		else if(!isStart && cacheEmpty){
			#ifdef DEBUG
				std::cout << "%%%S CONT/EMPTY %%% should not occur..." << std::endl;
			#endif
			std::cerr << "Illegal packet arrived to SWIM.  Packet " << saraPac->getCounter() << std::endl;
		}
		// If arrival packet is not start, and cache is not empty
		//      This is the continuation of the packet.  We don't know the completion of packet.
		//          -> Append to the cache.
		else if(!isStart && !cacheEmpty){
			#ifdef DEBUG
				std::cout << "%%%S CONT/EXIST (append) %%% " ;
				std::cout << swimCache->getCacheSize() << " -> ";
			#endif

			swimCache->append(swim_buff, swim_buff_size);

			#ifdef DEBUG
				std::cout << swimCache->getCacheSize() << std::endl;
			#endif
		}
		
		
		delete[] swim_buff;
	}

	return swimPac;
}




// End of namespace 
}}






