/*
 *  SaraDemultiplexor.h
 *  libsadppac
 *
 *  Created by Futaana Yoshifumi on 07/06/28.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#ifndef _IRFPAC_SARAPAC_SARADEMULTIPLEXOR
#define _IRFPAC_SARAPAC_SARADEMULTIPLEXOR

#include<IrfPac/irfpac.h>
#include<IrfPac/SaraPac/CenaPacket.h>
#include<IrfPac/SaraPac/SwimPacket.h>
#include<IrfPac/SaraPac/SaraPacket.h>

namespace IrfPac{
namespace SaraPac{

/**	@brief Cache of byte stream.

	Actually, I believe that this class should be refactored as it extends GeneralPacket.
	
*/
class SasCache {
	public:
		SasCache();
		~SasCache();
		
		bool isEmpty();
		/** @brief Returns a copy of the contents of packet.
		*/
		uint8_t* getPacket();
		/** @brief Returns a copy of the contents of packet, and flush the cache.
		*/
		uint8_t* flush();
		/** @brief Append the GeneralPacket to the cache.
		*/
		size_t append(GeneralPacket* saraPac);
		/** @brief Append the byte stream to the cache.
		*/
		size_t append(uint8_t* byteToAdd, size_t sizeToAdd);
		size_t getSize(){return __pac_size;};  //<! This method returns the cache size.
		size_t getCacheSize(){return __pac_max_size;};  //<! This method returns the buffer size including preserved area.

	private:
//		PacketU8Arr* _pac;
		uint8_t *__pac;
		size_t __pac_size;
		size_t __pac_max_size;

};

/** @brief Factory class of SARA data stream to demultiplex to CENA/SWIM data stream

	This class receives series of SaraPacket, which is 448-word-fixed binary stream packet, and
	demultiplex the packet, and returns CenaPacket and SwimPacket.
	Only when SaraPacket shows the completion of one packet,
	CenaPacket and SwimPacket will be returned.  Otherwise,
	NULL is returned.

	An sample of the execution can be found in AscFmt/SaraDemultiplexor_sample.cpp
*/
class SaraDemultiplexor {
	public:
		SaraDemultiplexor();
		~SaraDemultiplexor();
		
		CenaPacket* appendCena(SaraPacket* saraPac);
		SwimPacket* appendSwim(SaraPacket* saraPac);
		
		/**	@brief Check the sequence of SaraPacket, and returns the packet is sequencially arrived.
		
		This sequence should not be called twice for the same SaraPacket because this updates the
		internal buffer of counter number to the current packet.
		
		@bug Potential bug: Because I do not know the maximum number of the packet counter,
		I have never checked the bug-freeness when loop occurs.  I implemented maximum counter as 1<<15,
		but should be checked!
		*/
		bool sequenceCheck(SaraPacket* saraPac){
			uint16_t packetCntr=saraPac->getCounter();

			bool b=false;

			if(firstpacket){  // First packet
				firstpacket=false;
				b=true;
			}
			else if(previousPacketCntr==MAX_CNTR && packetCntr==0){  // Roll over: MAX_CNTR should be revised if needed.
				b=true;
			}
			else if(packetCntr == previousPacketCntr+1){ // Normal count up
				b=true;
			}
			
			if(!b){
				std::cerr << "Sequencial check fails (" << previousPacketCntr << " to " << packetCntr << ")" << std::endl;
			}
			
			previousPacketCntr=packetCntr;
		
			return b;
		};
		
		void flushAll(){
			cenaCache->flush();
			swimCache->flush();
		};
	
	private:
		static const uint16_t MAX_CNTR=1<<15;
		SasCache* cenaCache;
		SasCache* swimCache;
		uint16_t previousPacketCntr; 
		bool firstpacket;
};

// End of namespace
}}

#endif
