#include<iostream>
using namespace std;

#include<cppunit/extensions/HelperMacros.h>

#include"IrfPac/irfpac.h"
#include"IrfPac/SaraPac/sarapac.h"

using namespace IrfPac;
using namespace IrfPac::SaraPac;

class SaraDemultiplexor_test : public CPPUNIT_NS::TestFixture {

public:
		CPPUNIT_TEST_SUITE(SaraDemultiplexor_test);
		CPPUNIT_TEST(testSasCacheAppend);
		CPPUNIT_TEST_SUITE_END();
	
	void testSasCacheAppend(){
		SasCache *sasCache=new SasCache();
	
		CPPUNIT_ASSERT_EQUAL(0, (int)(sasCache->getSize()));
		CPPUNIT_ASSERT_EQUAL(0, (int)(sasCache->getCacheSize()));
	
		// First test: add 896 byte packet
		uint8_t *p1=new uint8_t[896];
		for(int i=0; i<896; i++){
			p1[i] = i%256;
		}
		SaraPacket *sara=new SaraPacket(p1);
		int size=sasCache->append(sara);
		delete sara;
		
		CPPUNIT_ASSERT_EQUAL(896, size);
		
		CPPUNIT_ASSERT_EQUAL(896, (int)(sasCache->getSize()));
		CPPUNIT_ASSERT_EQUAL(896, (int)(sasCache->getCacheSize()));

		// Second test: add 896 byte packet again

		for(int i=0; i<896; i++){
			p1[i] = (i*3)%256;
		}

		sara=new SaraPacket(p1);
		delete p1;
		size=sasCache->append(sara);
		delete sara;

		CPPUNIT_ASSERT_EQUAL(896*2, size);
		

		CPPUNIT_ASSERT_EQUAL(896*2, (int)(sasCache->getSize()));
		CPPUNIT_ASSERT_EQUAL(896*2, (int)(sasCache->getCacheSize()));

		// Check contents:
		uint8_t *p3=sasCache->getPacket();

		for(int i=0; i<896; i++){
			CPPUNIT_ASSERT_EQUAL((int)p3[i], (int)(i%256));
		}
		for(int i=896; i<896*2; i++){
			CPPUNIT_ASSERT_EQUAL((int)p3[i],(int)((i-896)*3)%256);
		}
		delete p3;
		
		// Flushing
		p3=sasCache->flush();

		for(int i=0; i<896; i++){
			CPPUNIT_ASSERT_EQUAL((int)p3[i], (int)(i%256));
		}
		for(int i=896; i<896*2; i++){
			CPPUNIT_ASSERT_EQUAL((int)p3[i],(int)((i-896)*3)%256);
		}
		delete p3;

		CPPUNIT_ASSERT_EQUAL(0, (int)(sasCache->getSize()));
		CPPUNIT_ASSERT_EQUAL(0, (int)(sasCache->getCacheSize()));
		
		
		// Thrid test
		p1=new uint8_t[150];
		for(int i=0; i<150; i++){
			p1[i] = 255-i;
		}
		sasCache->append(p1, 150);
		CPPUNIT_ASSERT_EQUAL(150, (int)(sasCache->getSize()));
		CPPUNIT_ASSERT_EQUAL(150, (int)(sasCache->getCacheSize()));
		delete p1;
		
		p3=sasCache->getPacket();
		for(int i=0; i<150; i++){
			CPPUNIT_ASSERT_EQUAL((int)(255-i), (int)p1[i]);
		}
		
		delete sasCache;
	
	
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION(SaraDemultiplexor_test);
