#ifndef __IRFPAC__SARAPAC__SARAHKCALIBRATION
#define __IRFPAC__SARAPAC__SARAHKCALIBRATION

#include <string>

namespace IrfPac{
namespace SaraPac{
  /**  @brief Conversion table for SARA HK data.

  Conversion table for SARA HK data.

  */
  class SaraHkConversion{
    public:
      /** @brief Identifier of the calibration table.

      Identifier of the calibration table for ground calibration.
      Documentation in CHA-SARA-CR-0002-I1R2 (SAS Housekeeping Calibration Functions).

      In order to obtain monitor value, use SwimFastHkPacket class for SWIM and CenaFastHkPacket for CENA.

      */
      static const int TABLE_GROUND_CALIBRATION=1;
 
      static SaraHkConversion* getConversionFunctions(int id);

    protected:
      SaraHkConversion(){}

    public:
      virtual ~SaraHkConversion(){}
//      virtual std::string getName(){return "Unknown";};
      virtual std::string getName()=0;

      virtual double swimHV(uint32_t monitor)=0;  ///< SWIM Main in Volts
      virtual double swimVCeem(uint32_t monitor)=0;   ///< SWIM CEM in Volts
      virtual double swimVUpDef(uint32_t monitor)=0; ///< SWIM Defl. upper in Volts
      virtual double swimVLoDef(uint32_t monitor)=0; ///< SWIM Defl. lower in Volts
      virtual double swimVAnaCel(uint32_t monitor)=0;  ///< SWIM Anyz in Volts
      virtual double swimVTofCel(uint32_t monitor)=0;  ///< SWIM Cell in Volts
      virtual double swimTempTofBrd(uint32_t monitor)=0;  ///< SWIM Temp in degC

      virtual double cenaHvMain(uint32_t monitor)=0;  ///< CENA HV_Ref in Volts
      virtual double cenaHvStartMcp(uint32_t monitor)=0;  ///< CENA HV_StartMCP in Volts
      virtual double cenaHvStopMcp(uint32_t monitor)=0;  ///< CENA HV_StopMCP in Volts
      virtual double cenaHvDef(uint32_t monitor)=0;  ///< CENA HV_Def in Volts
      virtual double cenaHvTof(uint32_t monitor)=0;  ///< CENA HV_Tef in Volts
      virtual double cenaSvWave1(uint32_t monitor)=0;  ///< CENA SV_WAVE1 in Volts
      virtual double cenaSvWave2a(uint32_t monitor)=0;  ///< CENA SV_WAVE2A in Volts
      virtual double cenaSvWave2b(uint32_t monitor)=0;  ///< CENA SV_WAVE2B in Volts
      virtual double cenaSvLens(uint32_t monitor)=0;  ///< CENA HV_LENS in Volts
      virtual double cenaTempHVPS(uint32_t monitor)=0;  ///< CENA Temp_HVPS in degC
      virtual double cenaTempIFE(uint32_t monitor)=0;  ///< CENA Temp_IFE in degC




  };

} }



#endif

