#ifndef __IRFPAC__SARAPAC__SARAHKCALIBRATIONGRNDCAL__
#define __IRFPAC__SARAPAC__SARAHKCALIBRATIONGRNDCAL__

#include<string>
#include <inttypes.h>
#include"IrfPac/SaraPac/SaraHkConversion.h"

namespace IrfPac{
namespace SaraPac{
  /** @brief An implementation of the SaraHkConversion.

	The conversion function is defined by CHA-SARA-CR-0002-I1R2 (SAS Housekeeping Calibration Functions).
	You can instance it by
	<pre>
	SaraHkConversion.getConversionFunctions(SaraHkConversion::TABLE_GROUND_CALIBRATION);
	</pre>
  */
  class SaraHkConversionGrndCal : public SaraHkConversion{
    public:
      SaraHkConversionGrndCal(){}

      virtual std::string getName(){
        return "ConversionTableFromGroundCalibration";
      }

      virtual double swimHV(uint32_t monitor);  ///< SWIM Main in Volts
      virtual double swimVCeem(uint32_t monitor);   ///< SWIM CEM in Volts
      virtual double swimVUpDef(uint32_t monitor); ///< SWIM Defl. upper in Volts
      virtual double swimVLoDef(uint32_t monitor); ///< SWIM Defl. lower in Volts
      virtual double swimVAnaCel(uint32_t monitor);  ///< SWIM Anyz in Volts
      virtual double swimVTofCel(uint32_t monitor);  ///< SWIM Cell in Volts
      virtual double swimTempTofBrd(uint32_t monitor);  ///< SWIM Temp in degC

      virtual double cenaHvMain(uint32_t monitor);  ///< CENA HV_Ref in Volts
      virtual double cenaHvStartMcp(uint32_t monitor);  ///< CENA HV_StartMCP in Volts
      virtual double cenaHvStopMcp(uint32_t monitor);  ///< CENA HV_StopMCP in Volts
      virtual double cenaHvDef(uint32_t monitor);  ///< CENA HV_Def in Volts
      virtual double cenaHvTof(uint32_t monitor);  ///< CENA HV_Tef in Volts
      virtual double cenaSvWave1(uint32_t monitor);  ///< CENA SV_WAVE1 in Volts
      virtual double cenaSvWave2a(uint32_t monitor);  ///< CENA SV_WAVE2A in Volts
      virtual double cenaSvWave2b(uint32_t monitor);  ///< CENA SV_WAVE2B in Volts
      virtual double cenaSvLens(uint32_t monitor);  ///< CENA HV_LENS in Volts
      virtual double cenaTempHVPS(uint32_t monitor);  ///< CENA Temp_HVPS in degC
      virtual double cenaTempIFE(uint32_t monitor);  ///< CENA Temp_IFE in degC

    private:
      double polynoms(double a, double b, double c, double val);
  };

		
}
}

#endif
