#include<iostream>
#include<cmath>

#include "IrfPac/SaraPac/SaraPacket.h"


using namespace std;
using namespace IrfPac;

namespace IrfPac {
namespace SaraPac {

uint16_t getWordAt(GeneralPacket& pac, size_t pos){
	size_t size=pac.getSize();
	if(pos > size/2 || pos <=0){
		throw "Given size violation.";
	}
	uint8_t* stream = pac.getStream();
	uint16_t up=stream[2*(pos-1)];
	uint16_t dn=stream[2*pos-1];
	uint16_t retVal=(up << 8)+dn;
	delete[] stream;
	return retVal;
}

uint8_t getByteAt(GeneralPacket& pac, size_t pos){
	size_t size=pac.getSize();
//	cout << (int)size << endl;
	if(pos >= size || pos < 0){
		throw "Given size violation.";
	}
	uint8_t* stream=pac.getStream();
	uint8_t retByt=stream[pos];
	delete[] stream;
	return retByt;
}


  /*
		Algorithm is the message forwarded from Dhanya.
		The algorithm is confirmed by the sample program
		(Futaana memo: 080722-libsadppac-bmu-support, bmu_sample_files_test branch.)
		<pre>
			From: jagada@isac.gov.in
			To: skp@isac.dos.gov.in
			Cc: mb_dhanya@vssc.gov.in
			Sent: Thu, 10 Jul 2008 17:21:22 +0530
			Subject: Re: Fwd: BMU OBT format

			We have program to convert the same. But to run the program we need
			the system architecture - whether it is big endian or little endian?
			You can use the following deduction also:

			Assuming B0 is lsb
			Mantissa : B47 to B24 & B15 to B00
			Exponent : B23 to B16

			both mantissa and exponents are in 2'complement format.

			So value = decimal(40-bit 2'complement mantissa) * 2^(-39+
			decimal(8-bit 2'comp exp))

			With regards,
			Jagannath Das
			</pre>
   */
double obt2msec(uint64_t obt){
  double converted=0;

#ifdef DEBUG
  int msb = (obt & 0xffff00000000LL) >> 32;
  int mid = (obt & 0xffff0000) >> 16;
  int lsb = (obt & 0xffff) ;

  printf("%04x %04x %04x\n", msb, mid, lsb);
#endif

  uint64_t mantissa_m = obt & 0xffffff000000LL;
  uint64_t mantissa_l = obt & 0xffff;

  uint64_t mantissa = ( mantissa_m >> 8 ) | mantissa_l;
  uint16_t exponent   = (obt & 0xff0000) >> 16;

#ifdef DEBUG
  printf("MAN: %012llx \n", mantissa);
  printf("EXP: %02x \n", exponent);
#endif

  ///// I guess the 2-complement format means that
  ////    - For positive (MSB is 0) value, as is.
  ////    - For negative (MSB is 1) value, complement

  // for mantissa
  if ( (mantissa & ( 1LL<<39 ) ) != 0){  // negative
    mantissa = - ( (1LL<<39) - (mantissa & 0x7fffffffffLL));
  }

  // for exponent
  if ( (exponent & (1<<7)) != 0){
    exponent = - ( (1<<7) - (exponent & 0x7f));
  }

  converted = mantissa * pow((double)2, (double)exponent-39);

  return converted;
}


SaraPacket::SaraPacket(uint8_t* stream) : PacketU8Arr(stream, 448*2) {}
SaraPacket::~SaraPacket() {}

/** @brief Returns the packet ID, packetIdCenaOnly (0xCCCC), packetIdCenaSwim (0xCECE), or packetIdSwimOnly (0xEEEE).

	In case the packet ID was not those of them, packetIdUnknown (0xFFFF) is returned.
*/
uint16_t SaraPacket::getPacketId(){
	uint16_t id=getWordAt(*this, 1);
	if(id==packetIdCenaSwim || id==packetIdCenaOnly || id==packetIdSwimOnly)return id;
	return packetIdUnknown;
}

/** @brief Returns the counter of packet.

	The counter is located at WORD-2.
	
	@bug Note that I DO NOT KNOW THE MAXIMUM NUMBER YET!
	HOW LOOP LOOKS LIKE.  IT IS SIGNED? or UNSIGNED?
*/
uint16_t SaraPacket::getCounter(){
	uint16_t cntr=getWordAt(*this, 2);
	return cntr;
}

/** @brief Returns a copy of byte stream of CENA part.

	The CENA memory area is returned.  The area is up to the packet ID.
	If the packet ID is Cena only, it returns 446 words (WORD 3 to 448/Byte 4 to 895).
	If the packet ID is Cens-Swim, it returns 318 words (WORD 3 to 320/Byte 4 to 639).
	Otherwise, NULL is returned.
*/
uint8_t* SaraPacket::getCenaByteStream(){
	uint8_t *bStream=NULL;
	uint16_t pacId=getPacketId();

	// When the saraPac is CenaOnly packet
	if(pacId == SaraPacket::packetIdCenaOnly){
//		std::cout << "+++++ CENA ONLY PACKET" << std::endl;
		uint8_t *stream=this->getStream();
		bStream=new uint8_t[892];
		for(int i=0; i<892; i++){
			bStream[i] = stream[i+4];
		}
		delete[] stream;
	}
	// When the saraPac is CenaSwim packet
	else if(pacId == SaraPacket::packetIdCenaSwim){
//		std::cout << "+++++ CENA-SWIM PACKET" << std::endl;
		uint8_t *stream=this->getStream();
		bStream=new uint8_t[636];
		for(int i=0; i<636; i++){
			bStream[i] = stream[i+4];
		}
		delete[] stream;
	}

	return bStream;
}

/** @brief Returns the size in byte of the area that CENA data occupies.  Values are up to the packet ID.

	This must be the same as the size of the array of getCenaByteStream().  CENA_ONLY is 892, CENA_SWIM is 636, otherwize 0.
*/
size_t SaraPacket::getCenaByteStreamSize(){
	uint16_t pacId=getPacketId();
	if(pacId == SaraPacket::packetIdCenaOnly){
		return 892;
	}
	else if(pacId == SaraPacket::packetIdCenaSwim){
		return 636;
	}
	else{
		return 0;
	}
}

uint8_t* SaraPacket::getSwimByteStream(){
	uint8_t *bStream=NULL;
	uint16_t pacId=getPacketId();

	// When the saraPac is SwimOnly packet
	if(pacId == SaraPacket::packetIdSwimOnly){
//		std::cout << "+++++ SWIM ONLY PACKET" << std::endl;
		uint8_t *stream=this->getStream();
		bStream=new uint8_t[892];
		for(int i=0; i<892; i++){
			bStream[i] = stream[i+4];
		}
		delete[] stream;
	}
	// When the saraPac is CenaSwim packet
	else if(pacId == SaraPacket::packetIdCenaSwim){
//		std::cout << "+++++ CENA-SWIM PACKET" << std::endl;
		uint8_t *stream=this->getStream();
		bStream=new uint8_t[256];
		for(int i=0; i<256; i++){
			bStream[i] = stream[i+640];
		}
		delete[] stream;
	}

	return bStream;
}


size_t SaraPacket::getSwimByteStreamSize(){
	uint16_t pacId=getPacketId();
	if(pacId == SaraPacket::packetIdSwimOnly){
		return 892;
	}
	else if(pacId == SaraPacket::packetIdCenaSwim){
		return 256;
	}
	else{
		return 0;
	}
}


// End of namespace
}
}


