#ifndef IRFPAC_SARAPACSARAPACKET_H
#define IRFPAC_SARAPACSARAPACKET_H

#include"IrfPac/irfpac.h"

namespace IrfPac {

namespace SaraPac {
/*
	@author 
*/
//  class SaraPacket{
// public:
//     SaraPacket();
// 
//     ~SaraPacket();
// 
// };


/**	@brief Returns the contents of specified word.

	Word for SARA DPU is 2 byte starting from 1.
	Note that this function is discouraged to be used inside the for-loop because
	it is too slow in this version. (While no other methods are implemented:-(.)
*/
uint16_t getWordAt(GeneralPacket& pac, size_t pos);
/**	@brief Returns the contents of specified byte.

	Byte for SARA DPU is 1 byte starting from 0.
	This definition is natural for C/C++.
	Note that this function is discouraged to be used inside the for-loop because
	it is too slow in this version.

	@see GeneralPacket::getStream()
	@see PacketIntArr::getValue(size_t)

*/
uint8_t getByteAt(GeneralPacket& pac, size_t pos);

/**@brief A function to convert OBT (so-called 1750 format) to float value (unit in msec).
 */
double obt2msec(uint64_t obt);

/** @brief SARA packet.  See TMTC doc.

	The SARA packet is very complicated packet.
	Each packet consists of 448 word (896 byte).
	Combination of packets will re-create CenaPacket and SaraPacket.
	This process is called `demultiplex'.
	
	The SARA packet can be categolized into several types depending on
	the header value (first one-word).
	- 0xCECE is a packet contains both CENA and SWIM
	- 0xCCCC is a packet contains only CENA
	- 0xEEEE is a packet contains only SWIM
	
*/
class SaraPacket : public PacketU8Arr{

	public:
		SaraPacket(uint8_t* stream);
		~SaraPacket();
		
		uint16_t getPacketId();
		uint16_t getCounter();

		uint8_t *getCenaByteStream();
		size_t getCenaByteStreamSize();
		uint8_t *getSwimByteStream();
		size_t getSwimByteStreamSize();

		static const int saraPacketSize=448;

		static const uint16_t packetIdCenaSwim=0xcece;  //!< Specifies the packet is CENA_SWIM packet.  Real value is identical to the header value (1st word).
		static const uint16_t packetIdCenaOnly=0xcccc;  //!< Specifies the packet is CENA_ONLY packet.  Real value is identical to the header value (1st word).
		static const uint16_t packetIdSwimOnly=0xeeee;  //!< Specifies the packet is SWIM_ONLY packet.  Real value is identical to the header value (1st word).
		static const uint16_t packetIdUnknown =0xffff;  //!< Specifies the packet is not-known packet.
	
	
	private:

};


}

}

#endif
