/*
 *  SwimAccumTotMatx.cpp
 *  libsadppac
 *
 *  Created by Futaana Yoshifumi on 07/11/21.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */
#include<sstream>

#include "IrfPac/SaraPac/SwimAccumTotMatx.h"

namespace IrfPac{
namespace SaraPac{

SwimAccumTotMatx::SwimAccumTotMatx(uint8_t* stream, int n_e, int n_d) : PacketU8Arr(stream, n_e*n_d*5){
	_ne=n_e;
	_nd=n_d;
	_dump=new SwimAccumTotMatxAllDump();
}

SwimAccumTotMatx::~SwimAccumTotMatx(){
	delete _dump;
}


void SwimAccumTotMatx::dumpAll(FILE* fp){
	_dump->dump(fp, this);
}

void SwimAccumTotMatx::dump(FILE* fp, SwimAccumTotMatxDump *dump){
	dump->dump(fp, this);
}

void SwimAccumTotMatxAllDump::dump(FILE* fp, SwimAccumTotMatx* matx){
	int _ne=matx->getEnergyBins();
	int _nd=matx->getDeflectionBins();
	fprintf(fp, "Nene = %d\n", _ne);
	fprintf(fp, "Ndef = %d\n", _nd);

	for(int ic=0; ic<5; ic++){
		fprintf(fp, "xxx %s\n", SwimAccumTotMatx::getCounterName(ic));
		fprintf(fp, "EE   DD| ");
		for(int id=0; id<_nd; id++){
			fprintf(fp, "%4d ", id);
		}
		fprintf(fp, "| TOTAL\n");
		fprintf(fp, "----------------------------------------------------------\n");
		fprintf(fp, "SUM    | ");
		for(int id=0; id<_nd; id++){
			fprintf(fp, "%4d ", matx->collapseEnergyRaw(id, ic));
		}
		fprintf(fp, "| %4d\n", matx->collapseAllRaw(ic));
		fprintf(fp, "----------------------------------------------------------\n");
		for(int ie=0; ie<_ne; ie++){
			fprintf(fp, "%4d   | ", ie);
			for(int id=0; id<_nd; id++){
				fprintf(fp, "%4d ", matx->getRaw(ie,id,ic));
			}
			fprintf(fp, "| %4d\n", matx->collapseDeflbinRaw(ie, ic));
		}
	}

	fprintf(fp, "..... HEX DUMP .....\n");
	matx->dumpHexStream(fp);
}

SwimAccumTotMatx* SwimAccumTotMatx::reform16x16(){
	if(getEnergyBins() != 32 || getDeflectionBins() != 8){
		std::ostringstream s;
		s << "SwimAccumTotMatx::reform16x16 can only be used for Ne=32/Nd=8 (given: " << getEnergyBins() << "/" << getDeflectionBins() << ")";
		fprintf(stderr, "%s\n", s.str().c_str());
		throw std::runtime_error(s.str());
	}
	uint8_t *pac=getStream();
	SwimAccumTotMatx* swim16=new SwimAccumTotMatx16x16(pac);
	delete pac;
	return swim16;

}

SwimAccumTotMatx16x16::SwimAccumTotMatx16x16(uint8_t *pac) : SwimAccumTotMatx(pac, 16, 16){}
uint8_t SwimAccumTotMatx16x16::getRaw(int aE, int aD, int iC){
	// Index of the original matrix.
	int iE = (aE << 1) | (aD & 1);
	int iD = (aD >> 1);

#ifdef DEBUG
	fprintf(stderr, "Index (%d,%d) -> (%d,%d)\n", aE, aD, iE, iD);
#endif

	// Refer to the index of the original matrix (SwimAccumTotMatx).
	int index= iE * 8 * 5 + iD * 5 + iC;  // 8 is _nd in the original matrix.
	return getValue(index);
}

}
}
