/*
 *  SwimFastHkPacket.h
 *  libsadppac
 *
 *  Created by Futaana Yoshifumi on 07/10/19.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */
#ifndef __IRFPAC_SARAPAC_SWIMFASTHKPACKET_H
#define __IRFPAC_SARAPAC_SWIMFASTHKPACKET_H

#include<IrfPac/irfpac.h>
#include<IrfPac/SaraPac/SaraPacket.h>

namespace IrfPac{
namespace SaraPac{

/** @brief HK field of the science stream of SWIM.

Documentation is in Table 18  of DS0005.
*/
	class SwimFastHkPacket : public PacketU8Arr {
	
		public:
			SwimFastHkPacket(uint8_t* stream, size_t size);
			SwimFastHkPacket(uint8_t* stream);
			~SwimFastHkPacket();
			
			void dumpAll(void){dumpAll(stdout);};
			void dumpAll(FILE* fp);	

            /**@brief  Main HV voltage, HK0.  */
			uint16_t getHV(){return get12bitValue(0,1);};
		    /**@brief  CCEM voltage, HK1.  */
			uint16_t getVCeem(){return get12bitValue(2,3);};
			/**@brief  Upper deflector voltage, HK2.  */
			uint16_t getVUpDef(){return get12bitValue(4,5);};
            /**@brief  Lower deflector voltage, HK3.  */
			uint16_t getVLoDef(){return get12bitValue(6,7);};
            /**@brief  Analyzer voltage, HK4.  */
			uint16_t getVAnaCel(){return get12bitValue(8,9);};
		    /**@brief  Tof cell voltage, HK5.  */
			uint16_t getVTofCel(){return get12bitValue(10,11);};
		    /**@brief  Tof board temperature, HK6.  */
			uint16_t getTempTofBrd(){return get12bitValue(12,13);};

		private:
			static uint16_t get12bValue(uint8_t upper, uint8_t lower){
			  uint16_t val=0;
			  val=val|(((uint16_t)upper & 0x000f) << 8);
			  val=val|(((uint16_t)lower & 0x00ff));
			  return val;
			}
			/**@brief Returns 12 bit value constitute of idx0-th and idx1-th bytes.
			 */
			uint16_t get12bitValue(int idx0, int idx1){
			  uint8_t u = IrfPac::SaraPac::getByteAt(*this, (size_t)idx0);
			  uint8_t l = IrfPac::SaraPac::getByteAt(*this, (size_t)idx1);
			  return get12bValue(u,l);
			};



	};




}}  // namespace


#endif

