/*
 *  SwimPacket.cpp
 *  libsadppac
 *
 *  Created by Futaana Yoshifumi on 07/09/12.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include<iostream>
#include"IrfPac/SaraPac/SwimPacket.h"


namespace IrfPac{
namespace SaraPac{

SwimPacket::SwimPacket(uint8_t* stream, size_t size) : PacketU8Arr(stream, size){
#ifdef DEBUG
  std::cout << "SwimPacket Generating" << std::endl;
#endif
  _savedUtc=0;
  _qflag=SwimPacket::QFLAG_NOTSET;
}

SwimPacket::~SwimPacket(){
}

SwimSubcHkPacket* SwimPacket::getSubcomHkPac(){
	uint8_t* str=getStream();
	SwimSubcHkPacket *pac=new SwimSubcHkPacket(str+28);
	delete[] str;
	return pac;
}

SwimPacket* SwimPacket::decompress(int verbose){

	SwimPacket* decompPac=NULL;
	if(!isMatrixCompressed()){ // When Swim compression is OFF
		if (verbose >= VERBOSE_DEBUG)
			fprintf(stderr, "Matrix is already decompressed.  Return the new packet instance with identical contents.\n");

		decompPac=new SwimPacket(*this);  // Just copy the contents by calling copy constructor.
		decompPac->_savedUtc=_savedUtc;
		decompPac->_qflag=_qflag;
	}
	else{  // Swim COMPRESSION ON
		SwimMatrix* compMatx=getSwimMatrix();               // Original swim matrix
		SwimMatrix* decompMatx=compMatx->decompress(verbose);      // Decompressed swim matrix
		if(decompMatx==NULL){  // Failed to decompress
			if (verbose >= VERBOSE_MIN) 
				fprintf(stderr, "Failed to decompress SWIM packet: (8s-cntr: 0x%x)\n", get8sCycleCntr());
			decompPac=NULL;  // Returns NULL.
		}
		else{  // Successfully decompressed.
			// Create a new stream.
			size_t matxSizeDecmp = decompMatx->getSize();  // Decompressed matrix size
			
			size_t offset = offsetByteToSwimMatrix();
			
			size_t packetSize = offset + 4 + matxSizeDecmp;   // Decompressed packet size.  4 is for 0xaaddaadd
			uint8_t* contDecmp=new uint8_t[packetSize];
		
			if (verbose >= VERBOSE_DEBUG) {
				size_t matxSizeCmp = compMatx->getSize();      // Original matrix size 
				fprintf(stdout, "SwimMatrix decompression succeeded: %d -> %d (Packet size %d)\n", (int)matxSizeCmp, (int)matxSizeDecmp, (int)packetSize);
			}

			// Copy the same part (STUFF, HK and SCL_MATX)			
			uint8_t* contCmp=getStream();
			for(size_t i=0; i<offset; i++){
				contDecmp[i] = contCmp[i];
			}
			delete[] contCmp;

			// aaddaadd is added

			contDecmp[offset]=0xaa;
			contDecmp[offset+1]=0xdd;
			contDecmp[offset+2]=0xaa;
			contDecmp[offset+3]=0xdd;

			// Copy the decompressed SwimMatrix
			uint8_t* contMatx=decompMatx->getStream();
			for(size_t i=0; i<matxSizeDecmp; i++){
				contDecmp[i+offset+4] = contMatx[i];
			}
			decompPac=new SwimPacket(contDecmp, packetSize);
			decompPac->_savedUtc=_savedUtc;
			decompPac->_qflag=_qflag;
			
			delete[] contMatx;
			delete[] contDecmp;
			
			
		}
		if(compMatx!=NULL)delete compMatx;
		if(decompMatx!=NULL)delete decompMatx;

	}
	return decompPac;


}



void SwimPacket::printParamsTblHeader(FILE* fp){
	fprintf(fp, "#   SWIM PACKET TABLE (HEX) VERSION 2.0\n");
	fprintf(fp, "#1:VALIDITY  2:CNTR  3:OBT@M  4:DPUTIME@M  5:DATATIME  6:UT[msec]  7:PAC  8:MAS  9:DEFL  10:ENE  11:CYCLE  12:TDC-PW-ON  13:LUTCH-PROT  14:LUTCH  15:HVPS-OSC-ON  16:MEM-BAN-SEL  17:MODE-SAMP  18:SWIM-ON  19:CENA-ON  20:SWIM-CMP  21:HEALTH-CHK-EV  ");
	fprintf(fp, "22:MAIN-HV  23:CCEM-VOL  24:UP-DEF-V  25:LO-DEF-V  26:ANA-VOL  27:TOF-CEL-VOL  28:TOF-BRD-TMP\n");
}

void SwimPacket::printParamsTbl(FILE* fp){
	printParamsTbl2(fp, NULL);
}

void SwimPacket::printParamsTbl2(FILE*fp, TimeConversion* tconv){
	printParamsTbl3(fp, tconv, false);
}

void SwimPacket::printParamsTbl3(FILE*fp, TimeConversion* tconv, bool useUtc){
		fprintf(fp, "%1d ", (int)this->isValid());
		fprintf(fp, "0x%04x ", this->get8sCycleCntr());
		fprintf(fp, "0x%012lx ", (signed long)this->getObtTime());
		fprintf(fp, "0x%04x ", this->getDpuTime());
		fprintf(fp, "0x%04x ", this->getEndIntegCycleTime());
		if (useUtc) {
			fprintf(fp, "%19.4f ", this->getUtc()*1000);
		} else {
			fprintf(fp, "%19.4f ", this->getUniversalTime(tconv)*1000);
		}
		fprintf(fp, "0x%02x ", this->getPostAccIdx());
		fprintf(fp, "0x%02x ", this->getMassBins());
		fprintf(fp, "0x%02x ", this->getDeflectionBins());
		fprintf(fp, "0x%02x ", this->getEnergyBins());
		fprintf(fp, "0x%04x ", this->getNrCycles());
		fprintf(fp, "%1d ", (int)this->isTdcPowerOn());
		fprintf(fp, "%1d ", (int)this->isLatchUpProtectOn());
		fprintf(fp, "%1d ", (int)this->isLatchUp());
		fprintf(fp, "%1d ", (int)this->isHvpsOscOn());
		fprintf(fp, "%1d ", (int)this->isSelectedMemoryBank());
		fprintf(fp, "%1d ", (int)this->isModeSampling());
		fprintf(fp, "%1d ", (int)this->isSwimOn());
		fprintf(fp, "%1d ", (int)this->isCenaOn());
		fprintf(fp, "%1d ", (int)this->isSwimCompOn());
		fprintf(fp, "%1d ", (int)this->isHealthCheckEvent());

		SwimFastHkPacket* hkPac=this->getFastHkPac();

		fprintf(fp, "0x%04x ", hkPac->getHV());
		fprintf(fp, "0x%04x ", hkPac->getVCeem());
		fprintf(fp, "0x%04x ", hkPac->getVUpDef());
		fprintf(fp, "0x%04x ", hkPac->getVLoDef());
		fprintf(fp, "0x%04x ", hkPac->getVAnaCel());
		fprintf(fp, "0x%04x ", hkPac->getVTofCel());
		fprintf(fp, "0x%04x ", hkPac->getTempTofBrd());

		delete hkPac;

		fprintf(fp, "\n");
}


void SwimPacket::dumpAll(FILE* fp){
		fprintf(fp, "*** SWIM PACKET (%d byte) ***\n", (int)this->getSize());
		fprintf(fp, "*** VALIDITY = %d ***\n", (int)this->isValid());
		
		fprintf(fp, "** STUFF\n");
		fprintf(fp, "MagicNum:1-2: %04x/%04x: %08x\n",
			getWordAt(*this,1), getWordAt(*this,2), this->getHeadMagicNumber());
		fprintf(fp, "OBT:3-5: %04x/%04x/%04x: %12lx\n", 
			getWordAt(*this,3), getWordAt(*this,4),getWordAt(*this,5), 
			(signed long)this->getObtTime());
		fprintf(fp, "DPUtime:6: %04x: %04x\n", 
			getWordAt(*this,6), this->getDpuTime());
		fprintf(fp, "Tcount:7: %04x: %04x\n", 
			getWordAt(*this,7), this->getTcountDpu());
		fprintf(fp, "IntegCycTime:8: %04x: %04x\n", 
			getWordAt(*this,8), this->getEndIntegCycleTime());
		fprintf(fp, "TcountTimer:9: %04x: %04x\n", 
			getWordAt(*this,9), this->getTcountAcq());
		fprintf(fp, "8S cycle:10: %04x: %04x\n", 
			getWordAt(*this,10), this->get8sCycleCntr());
		fprintf(fp, "BinPrm1:11: %04x: %04x\n", 
			getWordAt(*this,11), this->getBinningPrm1());
		fprintf(fp, "\tPAC:B15-B8:%04x:%02x\n", 
			getWordAt(*this,11), this->getPostAccIdx());
		fprintf(fp, "\tM:B7-B0:%04x:%d\n", 
			getWordAt(*this,11), this->getMassBins());
		fprintf(fp, "BinPrm2:12: %04x: %04x\n", 
			getWordAt(*this,12), this->getBinningPrm2());
		fprintf(fp, "\tD:B15-B8:%04x:%d\n", 
			getWordAt(*this,12), this->getDeflectionBins());
		fprintf(fp, "\tE:B7-B0:%04x:%d\n", 
			getWordAt(*this,12), this->getEnergyBins());
		fprintf(fp, "NrCycle:13:%04x: %04x\n",
			getWordAt(*this,13), this->getNrCycles());
		fprintf(fp, "Mode:14:%04x: %04x\n",
			getWordAt(*this,14), this->getModes());
		fprintf(fp, "\tSTDCPWR:B14:%04x:%2d\n", 
			getWordAt(*this,14), this->isTdcPowerOn());
		fprintf(fp, "\tLUProtec:B13:%04x:%2d\n", 
			getWordAt(*this,14), this->isLatchUpProtectOn());
		fprintf(fp, "\tLU:B12:%04x:%2d\n", 
			getWordAt(*this,14), this->isLatchUp());
		fprintf(fp, "\tHVPS:B11:%04x:%2d\n", 
			getWordAt(*this,14), this->isHvpsOscOn());
		fprintf(fp, "\tMemB:B10:%04x:%2d\n", 
			getWordAt(*this,14), this->isSelectedMemoryBank());
		fprintf(fp, "\tMode:B9:%04x:%2d\n", 
			getWordAt(*this,14), this->isModeSampling());
		fprintf(fp, "\tSWon:B7:%04x:%2d\n", 
			getWordAt(*this,14), this->isSwimOn());
		fprintf(fp, "\tCEon:B6:%04x:%2d\n", 
			getWordAt(*this,14), this->isCenaOn());
		fprintf(fp, "\tSCmp:B5:%04x:%2d\n", 
			getWordAt(*this,14), this->isSwimCompOn());
		fprintf(fp, "\tHCE:B0:%04x:%2d\n", 
			getWordAt(*this,14), this->isHealthCheckEvent());


		fprintf(fp, "** HK\n");
		SwimFastHkPacket* hkPac=this->getFastHkPac();
		hkPac->dumpAll(fp);
		delete hkPac;
		
		fprintf(fp, "** SWIM_ACC_TOT_MATX\n");
		SwimAccumTotMatx* accTot=this->getAccumTotMatx();
		accTot->dumpAll(fp);
		delete accTot;
		
		fprintf(fp, "** SWIM_ACC_SCI_MATX\n");
		SwimMatrix* matx=this->getSwimMatrix();
		matx->dumpAll(fp);
		
		fprintf(fp, "** SWIM_SCIENCE_DATA_PACKET (converted ACC_SCI_MATX)\n");

		delete matx;

#ifdef DEBUG
		// This is for higher DEBUG level output.
		fprintf(fp, "** SwimPacket : HEX dump **\n");
		this->dumpHexStream(fp);
#endif

		fprintf(fp, "******\n");

		// To be continued.

}




// End of namespace
}}

