/*
 *  SwimSciAcc.cpp
 *  libsadppac
 *
 *  Created by futaana on 08/03/10.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

#include "IrfPac/SaraPac/SwimSciAcc.h"
#include <linlog++.h>

namespace IrfPac{
namespace SaraPac{

SwimSciAcc::SwimSciAcc(uint8_t *pac, int nEnergy, int nDefl, int nMass) : SwimScienceDataPacket(pac, nEnergy*nDefl*nMass){
	_ne=nEnergy;
	_nd=nDefl;
	_nm=nMass;
}

uint8_t SwimSciAcc::getRaw(int iE, int iD, int iM){
	int idx = iE * _nd*_nm + iD* _nm + iM;
	return getValue(idx);
}

uint16_t SwimSciAcc::get(int iE, int iD, int iM){
	uint8_t raw=getRaw(iE, iD, iM);
	return log8tolinear16(raw);
}

void SwimSciAcc::dumpAll(FILE*fp){
	fprintf(fp, "=== SwimSciAcc packet ===\n");
	fprintf(fp, "Nene = %d\n", _ne);
	fprintf(fp, "Ndef = %d\n", _nd);
	fprintf(fp, "Nmas = %d\n", _nm);
	
	for(int im=0; im<_nm; im++){
		for(int ie=0; ie<_ne; ie++){
			fprintf(fp, "M=%02d E=%02d | ", im, ie);
			for(int id=0; id<_nd; id++){
				fprintf(fp, " %02x -> %04x  ", getRaw(ie, id, im), get(ie,id,im));
			}
			fprintf(fp, "\n");
		}
	}
}

SwimSciAcc* SwimSciAcc::reform16x16(){
	if(getNrEne() != 32 || getNrDefl()!=8){
		std::ostringstream s;
		s << "SwimSciAcc::reform16x16 can only be used for Ne=32/Nd=8 (given: " << getNrEne() << "/" << getNrDefl() << ")";
		fprintf(stderr, "%s\n", s.str().c_str());
		throw std::runtime_error(s.str());
	}
	uint8_t* pac=getStream();
	SwimSciAcc16x16* swim16=new SwimSciAcc16x16(pac, getNrMass());
	return swim16;
}

SwimSciAcc16x16::SwimSciAcc16x16(uint8_t *pac, int nMass) : SwimSciAcc(pac, 16, 16, nMass){}
uint8_t SwimSciAcc16x16::getRaw(int aE, int aD, int iM){
	// Index of the original matrix.
	int iE=(aE<<1)|(aD&1);
	int iD=(aD>>1);

	//Refer to the index of the original matrix (SwimSciAcc).
	int nM=getNrMass();
	int index= iE * 8 * nM + iD * nM + iM;
	return getValue(index);
}

}}


