#include<iostream>
#include<cppunit/extensions/HelperMacros.h>

#include"IrfPac/irfpac.h"
#include"IrfPac/SaraPac/sarapac.h"

using namespace std;
using namespace IrfPac;
using namespace IrfPac::SaraPac;

class SwimSciAcc_test : public CPPUNIT_NS::TestFixture {
	
public:
	
	CPPUNIT_TEST_SUITE(SwimSciAcc_test);
	CPPUNIT_TEST(testTest);
	CPPUNIT_TEST(testSwimSciAcc16x16);
	CPPUNIT_TEST(testSwimSciAcc16x16exception);
	CPPUNIT_TEST_SUITE_END();
	
	void testTest(){
		int ne=32;
		int nd=8;
		int nm=32;
		int size=ne*nd*nm+4;
		
		uint8_t stream[size];
		
		stream[0]=stream[2]=0xaa;
		stream[1]=stream[3]=0xdd;
		
		for(int i=4; i<size; i++){
			stream[i] = (uint8_t)(i%256);
		}
		
		SwimMatrix* acc=new SwimMatrix(stream, size);
		
		acc->dumpAll(stdout);
		
		SwimScienceDataPacket* sci=acc->interpret(ne, nd, nm);
		
		sci->dumpAll(stdout);
		
		SwimSciAcc* sciAcc=(SwimSciAcc*) sci;
		sciAcc->dumpAll(stdout);
		
		CPPUNIT_ASSERT_EQUAL(4, (int)sciAcc->getRaw(0,0,0));
		CPPUNIT_ASSERT_EQUAL(5, (int)sciAcc->getRaw(0,0,1));
		CPPUNIT_ASSERT_EQUAL(36, (int)sciAcc->getRaw(0,1,0));
		CPPUNIT_ASSERT_EQUAL((32*8+4)%256, (int)sciAcc->getRaw(1,0,0));
	}

	void testSwimSciAcc16x16exception(){
		int ne=32;
		int nd=4;
		int nm=32;
		int size=ne*nd*nm;
		
		uint8_t stream[size];
		
		for(int i=0; i<size; i++){
			stream[i] = (uint8_t)(i%185);
		}
		
		SwimSciAcc* sci=new SwimSciAcc(stream, ne, nd, nm);
		
		sci->dumpAll(stdout);

		try{
			SwimSciAcc* sci16=sci->reform16x16();
			CPPUNIT_ASSERT(false);
		}
		catch(const std::runtime_error& e){
		}
		catch(...){
			CPPUNIT_ASSERT(false);
		}
		

	}

	void testSwimSciAcc16x16(){
		fprintf(stdout, "---testSwimSciAcc16x16---\n");
		int ne=32;
		int nd=8;
		int nm=32;
		int size=ne*nd*nm;
		
		uint8_t stream[size];
		
		for(int i=0; i<size; i++){
			stream[i] = (uint8_t)(i%185);
		}
		
		SwimSciAcc* sci=new SwimSciAcc(stream, ne, nd, nm);
		sci->dumpAll(stdout);

		
		SwimSciAcc* sci16=sci->reform16x16();

		CPPUNIT_ASSERT_EQUAL(16, sci16->getNrEne());
		CPPUNIT_ASSERT_EQUAL(16, sci16->getNrDefl());
		CPPUNIT_ASSERT_EQUAL(32, sci16->getNrMass());
		
		try{
			SwimSciAcc* sci16_2 = sci16->reform16x16();
			CPPUNIT_ASSERT(false);
		} catch(const std::runtime_error& e){
		} catch(...){
			CPPUNIT_ASSERT(false);
		}

		int val16=sci16->get(0,0,0);
		int val  =sci  ->get(0,0,0);

		CPPUNIT_ASSERT_EQUAL(val, val16);

		// aE=8, aD=12  <-> iE=16, iD=6
		val16=sci16->get(8, 12, 3);
		val  =sci  ->get(16, 6, 3);

		CPPUNIT_ASSERT_EQUAL(val, val16);
		
		if(sci16!=NULL)delete sci16;
		delete sci;

		
	}
	
	
};

CPPUNIT_TEST_SUITE_REGISTRATION(SwimSciAcc_test);
