#ifndef __IRFPAC_SARAPAC_SWIMSCIENCEPACKAGE_H
#define __IRFPAC_SARAPAC_SWIMSCIENCEPACKAGE_H

#include<sys/stat.h>

#include"IrfPac/SaraPac/SwimPacket.h"

namespace IrfPac{
namespace SaraPac{

class SwimSciencePackageDump{

public:
  SwimSciencePackageDump(char* dirname){
    char name[256];
    sprintf(name, "%s/01hk", dirname); fp1=fopen(name, "w");
    sprintf(name, "%s/02sclstr", dirname); fp2=fopen(name, "w");
    sprintf(name, "%s/03sclstp", dirname); fp3=fopen(name, "w");
    sprintf(name, "%s/04scltof", dirname); fp4=fopen(name, "w");
    sprintf(name, "%s/05accmt", dirname); fp5=fopen(name, "w");
    sprintf(name, "%s/06accdt", dirname); fp6=fopen(name, "w");
    sprintf(name, "%s/07accet", dirname); fp7=fopen(name, "w");
    sprintf(name, "%s/08sclstrdef", dirname); fp8=fopen(name, "w");
    sprintf(name, "%s/09sclstrd", dirname); fp9=fopen(name, "w");
  }
        void flush(){fflush(fp1); fflush(fp2); fflush(fp3); fflush(fp4); fflush(fp5); fflush(fp6); fflush(fp7); fflush(fp8); fflush(fp9);}
        void close(){fclose(fp1); fclose(fp2); fclose(fp3); fclose(fp4); fclose(fp5); fclose(fp6); fclose(fp7); fclose(fp8); fclose(fp9);}
  void add(SwimPacket* swim, TimeConversion* tconv){
    if(!swim->isValid())return;
    SwimPacket* swim_decmp = swim->decompress();
    if(swim_decmp==NULL)return;

    // 01hk HK table is the same as swimhk.
    swim->printParamsTbl2(fp1, tconv);

    SwimAccumTotMatx *tot=swim->getAccumTotMatx();
    // 02sclstr is the start matrix.
    SwimAccumTotMatxEspecDump dp2(SwimAccumTotMatx::START_COUNTER);
    fprintf(fp2, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    tot->dump(fp2, &dp2);

    // 08sclstrdef
    SwimAccumTotMatxDeflDump dp8(SwimAccumTotMatx::START_COUNTER);
    fprintf(fp8, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    tot->dump(fp8, &dp8);

    // 09sclstrd
    SwimAccumTotMatxEspecDump8Dir dp9(SwimAccumTotMatx::START_COUNTER);
    fprintf(fp9, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    tot->dump(fp9, &dp9);

    // 03sclstr is the start matrix.
    SwimAccumTotMatxEspecDump dp3(SwimAccumTotMatx::STOP_COUNTER);
    fprintf(fp3, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    tot->dump(fp3, &dp3);

    // 04sclstr is the start matrix.
    SwimAccumTotMatxEspecDump dp4(SwimAccumTotMatx::TOF_COUNTER);
    fprintf(fp4, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    tot->dump(fp4, &dp4);

    delete tot;
   // Science data
    SwimSciAcc *sci=NULL;
    sci=swim_decmp->getSwimSciAcc();
    delete swim_decmp;

    fprintf(fp5, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    SwimSciAccDump* dp5=new SwimSciAccMSpectraDump();
    dp5->dump(fp5, sci);


    fprintf(fp6, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000); 
    SwimSciAccDump* dp6=new SwimSciAccDSpectraDump();
    dp6->dump(fp6, sci);

    fprintf(fp7, "0x%04x %14.2f ", swim->get8sCycleCntr(), swim->getUniversalTime(tconv)*1000);
    SwimSciAccDump* dp7=new SwimSciAccESpectraDump();
    dp7->dump(fp7, sci);

    delete dp5;
    delete dp6;
    delete dp7;

  }
private:
        FILE *fp1,*fp2,*fp3,*fp4,*fp5,*fp6,*fp7,*fp8, *fp9;

};


/**@brief A class for dumping SWIM science package.

   Series of SwimPacket is converted to so-called science package format which is to be used idlscilib and other programs.
   The package is generated also by SaraDemultiplex program.
   The package is a folder structure such as:
   <pre>
   package_name.pkg -+- swim
                     +- swimhk
                     +- swimsci -+- 01hk
		                 +- 02sclstr
				 +- 03sclstp
				 +- ....
   </pre>
   Reading routine is NOT implemented in libsadppac,
   however, you can use SwimBmuReader for the file package_name.pkg/swim in order to reconstruct SwimPacketArray.

   @section TimeConversion Time Conversion
   Time conversion is represented by an extended class of TimeConversion.
   You can set TimeConversion class by setTimeConversion(TimeConversion) method.

   If TimeConversion is set, the program uses the specified time conversion.
   If it is not set (default), the dump() method try to call getUtc() method.
   In case getUtc() is not defined (this happens if you read the data from SARA BMU format), no conversion is made (i.e. time should be very small value close to 1970's).
 */
class SwimSciencePackage{
private:
	SwimPacketArray *pacArr;
	TimeConversion *tconv;
public:
	static int getVersion(){
		return 2;
	}

	SwimSciencePackage(){
		pacArr=new SwimPacketArray();
		tconv=NULL;
	}
	~SwimSciencePackage(){
		delete pacArr;
	}
	/**@brief Set time conversion method.

	   Specify the method of time conversion.  Look at "Time conversion' section in the class description.

	   This method just copy the pointer of tConv.  No new/delete is done.
	   So be careful that tConv should not be deleted.
	 */
	void setTimeConversion(TimeConversion *tConv){
		tconv=tConv;
	}
	void appendSwimPacketArray(SwimPacketArray* arr){
		for(unsigned int i=0; i<arr->size(); i++){
			SwimPacket *sPac=arr->getPacket(i);
			pacArr->append(sPac);
			delete sPac;
		}
	}
	void appendSwimPacket(SwimPacket* pac){
		pacArr->append(pac);
	}
	void dump(char* pkgname){
#ifdef DEBUG
		fprintf(stderr, "VERSION: %d\n", getVersion());
		fprintf(stderr, "NUMBER OF DATA: %d\n", (int)pacArr->size());

		// Make a root folder of the package.
		fprintf(stderr, "Making package at %s\n", pkgname);
#endif
		mode_t dmode=0000755;
		mkdir(pkgname, dmode);

		char swimverfn[1024];
		sprintf(swimverfn, "%s/swimversion", pkgname);
		FILE* swimver=fopen(swimverfn, "w");
		fprintf(swimver, "%d", getVersion());
		fclose(swimver);


		// SWIM bmu format is inside the package.
		PacketBmuFormatter bmuformatter;
		char swimbmufn[1024];
		sprintf(swimbmufn, "%s/swim", pkgname);
		FILE *swimbmu=fopen(swimbmufn, "w");

		// SWIM HK inside the package.
		char swimhkfn[1024];
		sprintf(swimhkfn, "%s/swimhk", pkgname);
		FILE *swimhk=fopen(swimhkfn, "w");
		SwimPacket::printParamsTblHeader(swimhk);

		// Make a swimsci folder of the pacakge + dump class
		char swimsci[1024];
		sprintf(swimsci, "%s/swimsci", pkgname);
#ifdef DEBUG
		fprintf(stderr, "Making swimsci filder at %s\n", swimsci);
#endif
		mkdir(swimsci, dmode);
		SwimSciencePackageDump sspd(swimsci); 

		double toff=0;

		for(unsigned int i=0; i<pacArr->size(); i++){
			SwimPacket *swim=pacArr->getPacket(i);

			// Dump to swimbmu
			double utc=0;
			if(tconv!=NULL){
				utc=swim->getUniversalTime(tconv);
			}
			else{
				try{
					utc=swim->getUtc();
				}
				catch (const std::runtime_error &exc){
					utc=swim->getUniversalTime(NULL);
				}
			}
			if(i==0){
				toff=utc-swim->getUniversalTime(NULL);
			}
			int qf=swim->getQualityFlag();
			time_t utc_i = (time_t)utc;
			struct tm* tm=gmtime(&utc_i);

			fprintf(swimbmu, "******** SWIM-DATA at Time  %04d-%02d-%02dT%02d:%02d:%02dZ %15.3f %d %d\n", tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec,  utc, (int) swim->getSize(), (int) qf);
			bmuformatter.format(swimbmu, swim);

			// Dump to swimhk
			swim->printParamsTbl2(swimhk, tconv);


			// Dump to swimsci
			sspd.add(swim, tconv);
			sspd.flush();

			// Deleting swim packet.
			delete swim;
		}

		char swimtconvfn[1024];
		sprintf(swimtconvfn, "%s/swimtconv", pkgname);
		FILE* swimtconv=fopen(swimtconvfn, "w");
		fprintf(swimtconv, "%f", toff);
		fclose(swimtconv);

		fclose(swimbmu);
		fclose(swimhk);
		sspd.close();

	}
};
}}


#endif
