#ifndef GENERALPACKET_H
#define GENERALPACKET_H


#include <inttypes.h>
#include <stdio.h>
#include<iostream>
#include<string>
#include<stdexcept>

namespace IrfPac {

    /**	@brief Pure virtual class of packets.

            Base class for packets of general purpose.
	    @author Futaana
    */
    class GeneralPacket{
    public:
	virtual ~GeneralPacket(){};
	/**
		Returns pointer to the byte stream.
		The byte stream should be freeed explicitly after use.
	*/
        virtual uint8_t* getStream(void)=0;
	/**
		Dump the byte stream into the specified file.
		Returned value is the size of the dumped stream in bytes, which is
		normally the size of the packet.
	*/
        virtual size_t dumpByteStream( FILE* fp)=0;
	/**
		Dump the ascii-hex expression of the stream into the specified file.
		Returned value is the size of the dumped stream in bytes, which is
		normally the size of the packet.
	*/
 	virtual size_t dumpHexStream( FILE* fp)=0;
	/**
		Returns the size of the packet.  The unit of the size is BYTEs.
	*/
	virtual size_t getSize()=0;
	/**
		Returns a string expression of hash of the packet.
 	*/
	virtual std::string hash(){
		return std::string("---NOT CALCULATED HASH VALUE----");
	}
	virtual bool equals(GeneralPacket* pac){
		std::string md5=hash();
		if(md5=="---NOT CALCULATED HASH VALUE---"){
			throw std::runtime_error("Hash value is not calculated");
		}
		std::string md5pac=pac->hash();
		return (md5==md5pac);
	}
    };

}

#endif
