#include "IrfPac/packetint32arr.h"

namespace IrfPac {

PacketInt32Arr::PacketInt32Arr(uint32_t *stream, int nIndex)
 : GeneralPacket()
{
	contents=new uint32_t[nIndex];
	size=nIndex*bytes;
	nind=nIndex;
	for(size_t i=0; i<nind; i++){
		contents[i] = stream[i];
	}

}

PacketInt32Arr::~PacketInt32Arr()
{
	delete[] contents;
}

/** @brief value is split into byte array.

	Explicit delete[] is needed to avoid memory leak.
*/
uint8_t* PacketInt32Arr::split2byte(uint32_t val){
	size_t retsize=bytes;
	uint8_t* retval=new uint8_t[retsize];
	uint8_t mask=0xff;

	// BIG ENDIAN ASSUMED HERE
	for(size_t i=0; i<retsize; i++){
		retval[retsize-i-1] = val & mask;
		val = val >> 8;
	}
	return retval;
}

uint8_t* PacketInt32Arr::getStream(void){
	uint8_t* stream=new uint8_t[size];
	int ist=0;

	for(size_t i=0; i<nind; i++){
		uint8_t* segm=split2byte(getValue(i));
		for(size_t j=0; j<bytes; j++){
			stream[ist]=segm[j];
			ist++;
		}
		delete[] segm;
	}

	return stream;
}
size_t PacketInt32Arr::dumpByteStream( FILE* fp){
	uint8_t* stream=getStream();
	return fwrite(stream, sizeof(uint8_t), getSize(), fp);
}
size_t PacketInt32Arr::dumpHexStream( FILE* fp){
	uint8_t* stream=getStream();
	for(size_t i=0; i<size; i++){
		if(i%16==0) fprintf(fp, "0x%08x: ", (int)i);
		fprintf(fp, "%02x", stream[i]);
		if(i%2==1)fprintf(fp, " ");
		if(i%16==15)fprintf(fp, "\n");
	}
	fprintf(fp, "\n");

	delete[] stream;
	return size;
}
size_t PacketInt32Arr::getSize(){
	return size;
}
size_t PacketInt32Arr::getIndexSize(){
	return nind;
}
uint32_t* PacketInt32Arr::getValues(void){
	uint32_t* retCon=new uint32_t[nind];
	for(size_t i=0; i<nind; i++)retCon[i]=contents[i];
	return retCon;
}
uint32_t PacketInt32Arr::getValue(size_t index){
	return contents[index];
}


}
