/*
 *  AlphaEncode.cpp
 *
 *  Created by Futaana Yoshifumi on 07/10/29.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include <vector>
#include "AlphaEncode.h"

using namespace std;


static uint8_t processBuffer(uint32_t& buffer, size_t& pos){
	#ifdef DEBUG
		fprintf(stderr, "	+++ BUFFER STATUS BEFORE PROCESS +++\n");
		fprintf(stderr, "	%08x @ %d\n", buffer, (int)pos);
		fprintf(stderr, "	++++++++++++++++++++++++++++++++++++\n");
	#endif
	

	static uint32_t mask=(((uint32_t)0xff) << 24);
	uint32_t val;
	if(pos>=8){
		val = (buffer & mask) >> 24;
		buffer= (buffer << 8);
		pos-=8;
	}

	#ifdef DEBUG
		fprintf(stderr, "	+++ BUFFER STATUS AFTER PROCESS +++\n");
		fprintf(stderr, "	%08x @ %d\n", buffer, pos);
		fprintf(stderr, "	+++++++++++++++++++++++++++++++++++\n");
	#endif
	return (uint8_t)val;
}


/** @brief Compression by Alpha encoding for 8bit stream.

	Compress the input stream (stream) whose size is packetSize.
	The compressed stream size is substitute to compressedSize argument.
	The output stream will be padded by 0 until the end of the byte.
	
	Alpha encoding is that the value n is expressed by n zero + 1 one.
	For example, 0x0123456789abcdef will be encoded like
	\code
		Original:    01 23 45 67 89 ab cd ef 
		Alpha:
	\endcode
*/
uint8_t* encodeAlpha(uint8_t *stream, int packetSize, int& compressedSize){

	vector<uint8_t> compressedBuffer;

	uint32_t buffer=0;
	size_t   bufferFill=0;

	//
	compressedSize=0;

	// Real algorithm
	for(int i=0; i<packetSize; i++){
		// 
		uint8_t val=stream[i];   // Copy the value to be encoded.
		
		#ifdef DEBUG
			fprintf(stderr, "Value %02x comes\n", val);
		#endif
		
		while(val>=8){   // If val is more or equal to 8, append '0' to buffer
			#ifdef DEBUG
				fprintf(stderr, "	Large Value (%02x).  Store to the vector (original size=%d).\n", val, compressedBuffer.size());
			#endif
			bufferFill +=8;   // Actual processing is just shift the index.
			val-=8;     // the value is decremented.
			compressedBuffer.push_back(processBuffer(buffer, bufferFill));  // Process buffer.  Keeping BufferFill less than 8.
			#ifdef DEBUG
				fprintf(stderr, "	Store finished (current size=%d).\n", compressedBuffer.size());
			#endif

		}

		// Now val should be smaller than 8.
		while(val>0){
			#ifdef DEBUG
				fprintf(stderr, "	One bit shift (%02x).  %d to ", val, bufferFill);
			#endif
			bufferFill ++;
			val--;
			#ifdef DEBUG
				fprintf(stderr, "%d\n", bufferFill);
			#endif
		}
		
		// Now val is 0.  Set 1 to the (64-bufferFill)-th bit.

		#ifdef DEBUG
			fprintf(stderr, "	value 0.  Then, 1 bit flag at %d.\n", (int)bufferFill);
		#endif
		buffer |= (((uint32_t)1) << (31-bufferFill));  // Part of 1 one.
		bufferFill++;

		#ifdef DEBUG
			fprintf(stderr, "	+++ BUFFER STATUS +++\n");
			fprintf(stderr, "	%08x @ %d\n", (long)buffer, (int)bufferFill);
			fprintf(stderr, "	+++++++++++++++++++++\n");
		#endif

		while(bufferFill>=8){
			#ifdef DEBUG
				fprintf(stderr, "	Buffer filled (%08x).  Store to the vector (original size=%d).\n", (long)buffer, compressedBuffer.size());
			#endif
			compressedBuffer.push_back(processBuffer(buffer, bufferFill));  // Process buffer.  Keeping BufferFill less than 8.
			#ifdef DEBUG
				fprintf(stderr, "	Store finished (current size=%d).\n", compressedBuffer.size());
			#endif
		}
	}

	// Post processing.
	if(bufferFill>0){   // If data is remained.
		bufferFill=8;   // Padding by 0 until the byte boundary.
		compressedBuffer.push_back(processBuffer(buffer, bufferFill));  // Process buffer.  Keeping BufferFill less than 8.
	}

	compressedSize=compressedBuffer.size();
	uint8_t* compStream=new uint8_t[compressedSize];
	
	for(int i=0; i<compressedSize; i++){
		compStream[i]=compressedBuffer[i];
	}
	
	return compStream;

}

