/*
 *  RiceEnc.cpp
 *  Rice
 *
 *  Created by Futaana Yoshifumi on 07/10/30.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "RiceEnc.h"

using namespace std;
using namespace IrfPac::Encode;

uint8_t* encodeRice(uint8_t* stream, int prm, int packetSize, int& compressedSize){

	if(prm >=8 || prm < 0){
		cerr << "Exception issued at " << __LINE__ << " in " << __FILE__ << endl;
		throw "Parameter out of range";
	}

	BitBuffer bBuf;

	uint8_t lsb = (uint8_t)(((0xff << (8-prm)) & 0xff) >> (8-prm));  // Take prm bits of LSB 
	uint8_t msb = ((uint8_t)0xff) - lsb;

	#ifdef DEBUG
		fprintf(stderr, "LSB = %x\n", lsb);
		fprintf(stderr, "MSB = %x\n", msb);
	#endif

	for(int i=0; i<packetSize; i++){
		uint8_t lsbValue=stream[i] & lsb;
		uint8_t msbValue=(stream[i] & msb) >> prm;
		
		#ifdef DEBUG
			fprintf(stderr, "Value=%02x   MSB=%02x(%3d)   LSB=%02x\n", stream[i], msbValue, msbValue, lsbValue);
		#endif
		
		// msbValue is alpha encoding:  0 for '1', 1 for '01', 2 for '001', ... 0xf for '00000000000000001'
		// This is simply implemented by 
		while(msbValue >= 8){
			bBuf.appendLBits(0, 8);
			msbValue-=8;
		}
		bBuf.appendLBits(1, msbValue+1);
		
		// lsbValue is simply binary coded
		bBuf.appendLBits(lsbValue, prm);      // NOTE:  bBuf.appendLBits(stream[i], prm) should also work.

		#ifdef DEBUG
			bBuf.showAllBuffer(stderr);
		#endif
	}

	bBuf.paddingToBoundary(0);  // Append padding.

	compressedSize=bBuf.getSize();
	return bBuf.getBuffer();

}