/*
 *  RunLength8bit.cpp
 *  RunLength
 *
 *  Created by Futaana Yoshifumi on 07/10/29.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "RunLength8bit.h"

/** @brief Compression by Run-Length encoding with 8bit.

	Compress the input stream (stream) whose size is packetSize.
	The compressed stream size is substitute to compressedSize argument.
	The size of the stream should be between 0 and 255 (not 256!).
	The 256th byte [index=255] is substitute by packetSize (in many case, 0xfe=255 is here)
	in order to know the original packet size.
	If the packetSize is less than 255, 0x00 is filled for padding.
	If the packetSize is <0 or >=256, "InputStreamSizeException" is fired.

*/
uint8_t* compressRLE8(uint8_t *stream, int packetSize, int& compressedSize){

	if(packetSize <=0 || packetSize >= 256) throw "InputStreamSizeException";

	// Initinalization
	uint8_t* org=new uint8_t[256];
	for(int i=0; i<packetSize; i++){
		org[i] = stream[i];
	}
	for(int i=packetSize; i<255; i++){
		org[i] = 0;
	}
	org[255] = packetSize;
	
	// Real algorithm
	uint8_t* comp=new uint8_t[512];   // RLE encoding expands twice in the worst case.
	size_t i_comp=0;

	uint8_t cntr=1;
	uint8_t savedVal=org[0];

	for(int i_org=1; i_org<256; i_org++){
		#ifdef DEBUG
			fprintf(stderr, "---\n");
			fprintf(stderr, "%3d-th value 0x%02x arrived.  [Buffer=0x%02x %d times]\n", i_org, org[i_org], savedVal, cntr);
		#endif

		if(org[i_org]==savedVal){  // If the same as buffered value
			cntr++;
			#ifdef DEBUG
				fprintf(stderr, "\tAppend to buffer\n");
				fprintf(stderr, "\t###BUFFER###  0x%02x %d times\n", (int)savedVal, (int)cntr);
			#endif
		}
		else{   // If the value is different from buffered value

			// Save the previous value and counts.
			comp[i_comp] = cntr;
			comp[i_comp+1] = savedVal;
			i_comp+=2;
			#ifdef DEBUG
				fprintf(stderr, "\tFlush the buffer -- Value 0x%02x	for %d times\n", (int) cntr, (int)savedVal);
			#endif
			
			// Buffer the current value and counts.
			cntr=1;
			savedVal=org[i_org];
			#ifdef DEBUG
				fprintf(stderr, "\t###BUFFER###  0x%02x %d times\n", (int) savedVal, (int) cntr);
			#endif
		}
	}
	// Flush the last buffer.
	comp[i_comp] = cntr;
	comp[i_comp+1] = savedVal;
	i_comp+=2;
	#ifdef DEBUG
		fprintf(stderr, "\tFlush the buffer -- Value 0x%02x	for %d times\n", (int) cntr, (int)savedVal);
	#endif

	compressedSize=i_comp;
	
	#ifdef DEBUG
		fprintf(stderr, "*** Compression chunk finished.\n");
		fprintf(stderr, "\tOriginal %d -> Compress %d\n", packetSize, compressedSize);
		for(int i=0; i<compressedSize; i++){
			fprintf(stderr, "%02x", comp[i]);
			if(i%2==1)fprintf(stderr, " ");
			if(i%16==15)fprintf(stderr, "\n");
		}
		fprintf(stderr, "\n");
	#endif
	
	
	delete[] org;
	return comp;
}

