#include <iostream>
#include <IrfPac/Encode/encode.h>  // Include encoding stuff.

#include "RiceDiffEnc.h"

using namespace IrfPac::Encode;

void help(int argc, char * const argv[]){
		std::cerr << "HELP:  The program compresses/decompress stdin stream writing to stdout." << std::endl;
		std::cerr << "COMPRESSION:  Rice encoding (8-bits length/256 byte chunk/difference) is used." << std::endl;
		std::cerr << "USAGE:  " << argv[0] << "{-c|-d}   {prm:0-7}   " << std::endl;
}


uint8_t* getStream(FILE *fp, int maxByte, int& packetSizeByte){

	uint8_t* pac=new uint8_t[maxByte];
	
//	int nread=fread(pac, 1, maxByte, fp);
//	packetSizeByte=nread;

	packetSizeByte=maxByte;

	for(int i=0; i<maxByte; i++){ 
		fread(pac+i, 1, 1, fp);
		if(feof(fp)){
			packetSizeByte=i;
			break;
		}
	}
	
	
	return pac;

}


int main (int argc, char * const argv[]) {

	int flg=0;   // 1 is compress, -1 is decompress

	FILE* fin=stdin;
	FILE* fout=stdout;

	if(argc < 3){
		help(argc, argv);
		exit(-1);
	}
	else if(argc == 4){
		fin=fopen(argv[3], "rb");
	}
	else if(argc >= 5){
		fin=fopen(argv[3], "rb");
		fout=fopen(argv[4], "wb");
	}
	
	if(fin == NULL){
		std::cerr << "Failed to read " << argv[3] << std::endl;
		exit(-3);
	}
	if(fout == NULL){
		std::cerr << "Failed to write " << argv[4] << std::endl;
		exit(-4);
	}

	int prm = atoi(argv[2]);
	std::cerr << "Using parameter M=2^" << prm << std::endl;

	if(strcmp(argv[1], "-c")==0){
		std::cerr << "Compress mode with Rice+Diff" << std::endl;

		while(! feof(fin)){
			int packetSize;
			int compressedSize;
			uint8_t* stream=getStream(fin, 256, packetSize);    // Read 256 byte from the file
			uint8_t* compStream=encodeRiceDiff(stream, prm, packetSize, compressedSize);  // 8bit Rice compression
			std::cerr << packetSize << " B -> " << compressedSize << " B" << std::endl;
			fwrite(compStream, 1, compressedSize, fout);

			delete[] stream;
			delete[] compStream;
		}

		flg=1;
	}
	else if(strcmp(argv[1], "-d")==0){
		std::cerr << "Decompress mode with RLE" << std::endl;
		std::cerr << "--- NOT YET IMPLEMENTED ---" << std::endl;
		flg=-1;
	}
	else{
		help(argc, argv);
		exit(-2);
	}

    return 0;
}
