/*
 Rice decompression software 
 Reads the 16-bit word from the file c.txt.
 Featches for the header AA55AA55 till the end of the file and decompresses the data.
 Each rice packet with the header AA55AA55 contains the compressed data for the 512 samples
 Once the header is found, starts decompressing the data for 512 samples.
 In case of sync lost, that is if a header is found before decompressing the 512 samples,
 decompression starts for the new packet. Old packet which is incomplete,filled with FFFF for 
 the remaining samples.Rice decompressed output is 8 bits only. If anywhere in the output file
 the 16 bit data FFFF is found, it means that the particular packet has some errors.
 The output file is D.txt
 */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

unsigned long lut2[16]={0x1,0x2,0x4,0x8,0x10,0x20,0x40,0x80,
	0x100,0x200,0x400,0x800,0x1000,0x2000,0x4000,0x8000};

FILE *fp1,*fp2;

int log_indent_level=0;

char* indent(){
	static char* ind=new char[1000];
	char* pind=ind;
	int i;
	for(i=0; i<log_indent_level; i++){
		sprintf(pind, "  \0");
		pind+=2;
	}
	return ind;
}

void show_val(int pos, unsigned long val){
	
	log_indent_level++;
	// show value in 32 bit length;
	printf("%s%02x %02x %02x %02x\n", indent(), (val&0xff000000) >> 24, (val&0xff0000) >> 16, (val&0xff00)>>8, val&0xff );
	
	printf("%s",indent());
	for(int i=31; i>=0; i--){
		printf("%01d", (val& (1<<i))>>i);
		if(i%8==0)printf(" ");
	}
	printf("\n");
	
	printf("%s",indent());
	
	int ispace=31-pos;
	if(ispace > 23) ispace+=3;
	else if(ispace > 15) ispace+=2;
	else if(ispace > 7) ispace++;
	for(int i=0; i<ispace; i++)printf(" ");
	printf("^ pos=%2d", pos);
	
	printf("\n");
	
	log_indent_level--;
	
}

void read_val(int pos, unsigned long& val, int& y_value, int& sync_lost, int& y_sync_lost, int* sample)
{
	static int flag=0;  // I do not knot flag should be static or not, but probably safer.
	
	log_indent_level++;
	printf("%sread_val() started\n", indent());
	log_indent_level++;
	
	unsigned long val1;
	
	if(!sync_lost)
    { 
		if(flag==0)
		{
			if((fscanf(fp1,"%lx",&val))!=EOF)
			{
				//printf("%lx",val);
				// check if it is the header
				if(val==0xAA55)
				{
					// read the second word store it in val1
					fscanf(fp1,"%lx",&val1);
					if (val1==0xAA55 && y_value<512)
					{
						// header found 
						sync_lost=1;
						y_sync_lost=y_value;
						val=0xFFFF;   
					}
					else
						flag=1; 
					
				}
			}
			else // if end of file
			{
				
				if(y_value<512)
				{
					for(int x_value=0;x_value<y_value;x_value++)
						fprintf(fp2,"0x%x\n",sample[x_value]);
					for(int x_value=y_value;x_value<512;x_value++)
						fprintf(fp2,"0xFFFF\n");
				}
				fclose(fp2);
				//printf("reached End of file");
				exit(0);
			}   
			
		}
		else // if flag==1
		{
			val=val1;
			flag=0;
		}
    }
	else // if sync lost
    {
		val=0xFFFF;
    }
	
	printf("%sNew value is:\n", indent());
	show_val(pos, val);
	
	log_indent_level--;
	log_indent_level--;
}


void find_option(int& pos, unsigned long& val, int& kID, 
				/*for read_val()*/ int& y_value, int& sync_lost, int& y_sync_lost, int* sample )   // Just obtain 3 bit to substitute kID.  If kID is 0, proceed one bit (WHY?)
{
	static unsigned long lut1[18]={0x0,0x1,0x7,0xe,0x1c,0x38,
	0x70,0xE0,0x1C0,0x380,0x700,
	0xE00,0x1C00,0x3800,0x7000,
	0xE000,0x1C000,0x38000};

	
	
	log_indent_level++;
	printf("%sfind_option() started\n", indent());
	log_indent_level++;
	
	printf("%sObtains next 3 bit from:\n", indent());
	show_val(pos, val);
	
	if(pos>2)
    {
		kID=((val & lut1[pos])>>(pos-2));
		pos=pos-3;
    }
	else if(pos==2)
    {
		kID=((val & lut1[pos])>>(pos-2));
		read_val(pos, val, y_value, sync_lost, y_sync_lost, sample);
		pos=15;
    }
	else
    {
    	unsigned long temp_val;
    	
		temp_val=val<<16;
		read_val(pos, val, y_value, sync_lost, y_sync_lost, sample);
		val=val | temp_val;
		pos=pos+16;
		kID=((val & lut1[pos])>>(pos-2));
		pos=pos-3;
    }
	
	printf("%skID determined.  Parameters are modified.\n", indent());
    printf("%skID=%d\n", indent(), kID);
	show_val(pos, val);
	
	if(kID==0){
		pos=pos-1;
		printf("%skID==0 forcely proceed pos, any case...???\n", indent());
	}
	
	log_indent_level--;
	log_indent_level--;
	
}

void decompK(int& pos, unsigned long& val, int& idx, int* temp, int& ref_block, int& one_count, int& zero_count, 
						/*for read_val()*/ int& y_value, int& sync_lost, int& y_sync_lost, int* sample)
{
	// It is originally stated before the call of decompK().
	// But it can be in the function.
	one_count=0;
	
	log_indent_level++;
	printf("%sdecompK() started\n", indent());
	log_indent_level++;
	
	while(one_count<(16-ref_block))       //  ref_block is fixed
	{
		if(pos>=0)
		{
			if((val & lut2[pos])==0)    // pos bit is 1 -> false
			{
				zero_count=zero_count+1;
				pos=pos-1;
			}
			else
			{
				temp[idx++]=zero_count;
				zero_count=0;
				one_count=one_count+1;
				pos=pos-1;
			}
		}
		
		else
		{
			pos=15;
			read_val(pos, val, y_value, sync_lost, y_sync_lost, sample);
		}
		
    }// end of while
	
	log_indent_level--;
	log_indent_level--;
}

void appendLSB(int& kID, int& ref_block, int& pos, unsigned long& val, int& idx, int* temp, 
						/*for read_val()*/ int& y_value, int& sync_lost, int& y_sync_lost, int* sample)
{
	static unsigned long lut3[6]={0x0,0x1,0x3,0x7,0xF,0x1F};
// NOTE-1
	int append_count=0;
	unsigned long append;
	
	
	log_indent_level++;
	printf("%sappendLSB() started with kID=%d\n", indent(), kID);
	log_indent_level++;
	
	printf("%sref_block=%d / append_count=%d\n", indent(), ref_block, append_count);
	printf("%sThen, repeat %d (16-ref_block-append_count) times\n", indent(), 16-ref_block-append_count);

	while(append_count < (16-ref_block))
    {
		printf("%sLoop started:  append_count (%d) < 16-ref_block (%d)\n", indent(), append_count, 16-ref_block);
		log_indent_level++;

			printf("%sCurrent val:\n", indent());
			show_val(pos, val);
		
		if((pos+1)>=(kID-1))   // Just checking whether the val is enough for processing.  Otherwise, read a new value.  This means that the appendLSB() uses kID-1 bits for decoding.
		{
			
			// append=(val & (lut3[kID-1]<<(pos-kID+2))>>(pos-(kID-2)));
			append=lut3[kID-1]<<(pos-kID+2);
			append=append & val;
			append=(append >> (pos-(kID-2))) & lut3[kID-1];

			printf("%sappend value is 0x%x\n", indent(), append);
			printf("%stemp value (before appending) is 0x%x\n", indent(), temp[idx]);

			temp[idx]=((temp[idx]<<(kID-1)) | append) & 0xFF; //???????
			printf("%stemp value (after appending) is 0x%x\n", indent(), temp[idx]);

			idx++;
			append_count=append_count+1;
			pos=pos-(kID-1);
		}
		else
		{
			unsigned long temp_val;
			
			printf("%sDue to lack of buffered val, read&append a new value\n", indent());
			temp_val=val <<16;
			read_val(pos, val, y_value, sync_lost, y_sync_lost, sample);
			val=val | temp_val;
			pos=pos+16;
		}
		
		log_indent_level--;
		
    }// end of while
	
	
	log_indent_level--;
	log_indent_level--;
	
}

// Just "alpha-decoding" and save it to zero_count
void decomp_zeroblk(int& pos, unsigned long& val, int& zero_count,
						/*for read_val()*/ int& y_value, int& sync_lost, int& y_sync_lost, int* sample)
{
	int zero_block_flg=0;
	
	// Originally zero_count is set to zero in main() just calling before decomp_zeroblk.
	zero_count=0;
				
	
	log_indent_level++;
	printf("%sdecomp_zeroblk() started\n", indent());
	log_indent_level++;

	printf("%sCurrent zero_count=%d (should be zero?)\n", indent(), zero_count);
	printf("%sCurrent val is:\n", indent());
	show_val(pos, val);
	
	while(zero_block_flg==0)
    {
		if(pos>=0)
		{
			if((val & lut2[pos])==0)
			{
				zero_count++;
				pos=pos-1;
			}
			else
			{
				zero_block_flg=1;
				pos=pos-1;
				
			}
			
		}
		else
		{
			read_val(pos, val, y_value, sync_lost, y_sync_lost, sample);
			pos=15;
		}
    }
	
	printf("%sUpdated zero_count=%d\n", indent(), zero_count);
	printf("%sUpdated val is:\n", indent());
	show_val(pos, val);
		
	log_indent_level--;
	log_indent_level--;
}

// Obtain 8 bits from the stream without processing (16-ref_block-one_count) times.
// The read values are substituted into temp[] array from idx.  idx is incremented (16-ref_block-one_count) times as well.
// one_count is updated to (16-ref_block) finally.
void decomp_nocomp(int& ref_block, int& pos, unsigned long& val, int& idx, int* temp, int& one_count,
						/*for read_val()*/ int& y_value, int& sync_lost, int& y_sync_lost, int* sample)
{
	log_indent_level++;
	printf("%sdecomp_nocomp() started\n", indent());
	log_indent_level++;
	
	while(one_count< (16-ref_block))
    {
		if(pos>7)
		{
			temp[idx++]=((val & ((0xFF << (pos-7))))>>(pos-7)) & 0xFF;
			//printf("%x\n",((val & ((0xFF << (pos-7))))>>(pos-7)) & 0xFF);
			pos=pos-8;
			one_count++;
		}
		else if(pos==7)
		{
			temp[idx++]=((val & ((0xFF << (pos-7))))>>(pos-7)) & 0xFF;
			//printf("%x\n",((val & ((0xFF << (pos-7))))>>(pos-7)) & 0xFF);
			pos=15;
			read_val(pos, val, y_value, sync_lost, y_sync_lost, sample);
			one_count++;
			
		}  
		else
		{
			unsigned long temp_val;
			
			temp_val=val << 16;
			read_val(pos, val, y_value, sync_lost, y_sync_lost, sample);
			val=val | temp_val;
			pos=pos+16;
		}
		
    }
	
	log_indent_level--;
	log_indent_level--;
	
}

int min(int x)
{
	if(x > (255-x))
		return(255-x);
	else
		return(x);
}

void post_process(int& ref_block, int& idx, int* temp, int& y_value, int* sample)
{
	log_indent_level++;
	printf("%spost_process() started\n", indent());
	log_indent_level++;
	
	int k=0;
	int theta=0;
	int delta[48];

	if(ref_block==1)
    {
		
		delta[0]=temp[0];
		sample[y_value]=delta[0];
		y_value=y_value+1;
		for (k=1;k<=15;k++)
		{
			theta=min(sample[y_value-1]);   // theta is defined by the previous uncompressed value
			if(temp[k] <= 2*theta)
			{
				if(temp[k]%2==0)
					delta[k]=temp[k]/2;   // If temp[k] is even, delta[k] is the half
				else
					delta[k]=-1*(temp[k]+1)/2;   // if temp[i] is odd, delta[k] is minus ceil half (1 is -1, 3 is -2, 5 is -3, ....)
			}
			else
			{
				if (theta==sample[y_value-1])
					delta[k]=temp[k]-theta;
				else
					delta[k]=theta-temp[k];
			}
			
			sample[y_value]=sample[y_value-1]+delta[k];
			y_value++;
			
		}
		
    }
	else if(ref_block==0)
    {
		theta=min(sample[y_value-1]);
		if(temp[0] <= 2*theta)
		{
			if(temp[0]%2==0)
				delta[0]=temp[0]/2;
			else
				delta[0]=-1*(temp[0]+1)/2;
		}
		else
		{
			if (theta==(sample[y_value-1]))
				delta[0]=temp[0]-theta;
			else
				delta[0]=theta-temp[0];
		}
		
		sample[y_value++]=sample[y_value-1]+delta[0];
		
		for (k=1;k<=15;k++)
		{
			theta=min(sample[y_value-1]);
			if(temp[k] <= 2*theta)
			{
				if(temp[k]%2==0)
					delta[k]=temp[k]/2;
				else
					delta[k]=-1*(temp[k]+1)/2;
			}
			else
			{
				if (theta==sample[y_value-1])
					delta[k]=temp[k]-theta;
				else
					delta[k]=theta-temp[k];
			}
			
			sample[y_value++]=sample[y_value-1]+delta[k];
			
		}
    }
//	dummy=0;
	
	log_indent_level--;
	log_indent_level--;
	
}

void look_header(int& pos, unsigned long& val, int& one_more)
{
	log_indent_level++;
	printf("%slook_header() started\n", indent());
	log_indent_level++;
	
	
	// read the value
	//read_val();
	fscanf(fp1,"%lx",&val);
	if(val==0xAA55)
    {
		fscanf(fp1,"%lx",&val);
		// read the second one
		if(val==0xAA55)
		{
			one_more=1;
		}
		else
		{
			one_more=0;
		}
    }
	else
    {
		one_more=0;
		fscanf(fp1,"%lx",&val);
		if(val==0xAA55)
		{
			fscanf(fp1,"%lx",&val);
			// read the second one
			if(val==0xAA55)
			{
				one_more=1;
			}
			else
			{
				one_more=0;
			}
		}
    }
	
	log_indent_level--;
	log_indent_level--;
	
} 


int main()
{
	int ref_block=0;      // Probably specifies exist (1) or non-exist of 'reference block'.
	unsigned long val=0;  // This is a buffer of compressed packet.
	int pos=0;            // Pointer of val (bit)
	int temp[16];
	for(int i=0; i<16; i++)temp[i]=0;
	int idx=0;            // Current index of tmp[].
	int kID=0;            // Compression type  3 bit value.
	int sample[512];    // This is a decompressed packet.
	for(int i=0; i<512; i++)sample[i]=0;
	int y_value=0;        // Current index of decompressed packet, sample[y_value]

	int one_count=0,zero_count=0;

	int one_more=1;   // Used for checking header, aa55aa55. not so important.

	int sync_lost=0, y_sync_lost=0;

	printf("%smain() started\n", indent());
	log_indent_level++;
	
	one_more=0;
	fp2=fopen("d.txt","w");
	fp1=fopen("c.txt","r");
	// firts read for the header
	while((!one_more) && (!feof(fp1)))
    {
		fscanf(fp1,"%lx",&val);//read_val();
		if(val==0xAA55)
		{
			fscanf(fp1,"%lx",&val);
			if(val==0xAA55)
				one_more=1;
			else
				one_more=0;
		}
		else
		{
			one_more=0;
		}
		
    }
	
	if(!one_more)
    {
		printf(" No header Found");
		exit(0);
    }
	while(one_more)
    {
		
		//printf("found header");
		read_val(pos, val, y_value, sync_lost, y_sync_lost, sample);
		pos=15;
		ref_block=1;
		printf("%sref_block is forcely set to 1\n", indent());
		
		while(y_value<512)
		{
			
			printf("%smain() calls find_option()\n", indent());
			find_option(pos, val, kID, /*for read_val()*/ y_value, sync_lost, y_sync_lost, sample );   // Just obtain 3 bit to substitute kID.  If kID is 0, proceed one bit (WHY?)
			
			if(ref_block==1)
			{
				idx=0;
				one_count=14;
				decomp_nocomp(ref_block, pos, val, idx, temp, one_count,
						/*for read_val()*/ y_value, sync_lost, y_sync_lost, sample);
						 // to read the refference sample
			}
			else
			{
				idx=0;
				one_count=0;
			}
			
			if(kID==0)
			{
				decomp_zeroblk(pos, val, zero_count,
						/*for read_val()*/ y_value, sync_lost, y_sync_lost, sample);
				if(ref_block==1)
				{
					//fill first block with refference
					for(int x_value=1;x_value<=15;x_value++)
						temp[idx++]=0;
					post_process(ref_block, idx, temp, y_value, sample);
					zero_count--;
					idx=0;
					ref_block=0;
					for(int x_value=zero_count;x_value>0;x_value--)
					{
						for(int p_value=0;p_value<=15;p_value++)
							temp[idx++]=0;
						post_process(ref_block, idx, temp, y_value, sample);
						idx=0;
					}
				}
				else
				{
					for(int x_value=zero_count;x_value>0;x_value--)
					{
						for(int p_value=0;p_value<=15;p_value++)
							temp[idx++]=0;
						post_process(ref_block, idx, temp, y_value, sample);
						idx=0;
						
					}
					
				}
			}
			else if(kID==1)
			{
				decompK(pos, val, idx, temp, ref_block, one_count, zero_count, 
						/*for read_val()*/ y_value, sync_lost, y_sync_lost, sample);
					post_process(ref_block, idx, temp, y_value, sample);
			}
			else if(kID==7)
			{
				one_count=0;
				decomp_nocomp(ref_block, pos, val, idx, temp, one_count,
						/*for read_val()*/ y_value, sync_lost, y_sync_lost, sample);
				post_process(ref_block, idx, temp, y_value, sample);
			}
			else
			{
				
				decompK(pos, val, idx, temp, ref_block, one_count, zero_count, 
						/*for read_val()*/ y_value, sync_lost, y_sync_lost, sample);
				if(ref_block==1)
					idx=1;
				else
					idx=0;
				appendLSB(kID, ref_block, pos, val, idx, temp, 
						/*for read_val()*/ y_value, sync_lost, y_sync_lost, sample);
				post_process(ref_block, idx, temp, y_value, sample);
				
			}
			ref_block=0;
			zero_count=0;
			one_count=0;
			idx=0;
			
			
		}
		
		if(!sync_lost)
		{
			for(int x_value=0;x_value<512;x_value++)
			{
				fprintf(fp2,"0x%lx\n",sample[x_value]);
			}
			look_header(pos, val, one_more);
		}
		
		else
		{
			// already header found
			for(int x_value=0;x_value<y_sync_lost;x_value++)
				fprintf(fp2,"0x%lx\n",sample[x_value]); // write the recovered values
			for(int x_value=y_sync_lost;x_value<512;x_value++)
				fprintf(fp2,"0xFFFF\n"); // fill filler
			one_more=1;
			sync_lost=0; 
			y_sync_lost=0;
		}
		
		ref_block=1;
		y_value=0;
		one_count=0;
		zero_count=0;
		idx=0;
    }
	//printf("completed");
	//getch();
	return 0;
};

