/*
 *  RiceDiffEnc.cpp
 *  RiceDiff
 *
 *  Created by Futaana Yoshifumi on 07/10/30.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "RiceDiffEnc.h"

using namespace std;
using namespace IrfPac::Encode;

uint8_t* encodeRiceDiff(uint8_t* stream, int prm, int packetSize, int& compressedSize){

	if(prm >=8 || prm < 0){
		cerr << "Exception issued at " << __LINE__ << " in " << __FILE__ << endl;
		throw "Parameter out of range";
	}

	BitBuffer bBuf;

	
	uint8_t lsb = (uint8_t)(((0xff << (8-prm)) & 0xff) >> (8-prm));  // Take prm bits of LSB 
	uint8_t msb = ((uint8_t)0xff) - lsb;

	#ifdef DEBUG
		fprintf(stderr, "MSB = %x\n", msb);
		fprintf(stderr, "LSB = %x\n", lsb);
	#endif


	// First byte is encoded directly.
	bBuf.appendMBits(stream[0], 8);

	#ifdef DEBUG
		fprintf(stderr, "Encode %x\n", stream[0]);
		bBuf.showAllBuffer(stderr);
	#endif

	for(int i=1; i<packetSize; i++){
		int diff = ((int)stream[i]) - ((int)stream[i-1]);
		// Difference is pre-encoded to unsigned value.
		// diff -> enc     diff -> enc
		//   0      0
		//   1      2       -1      1
		//   2      4       -2      3
		int encdiff=  (  diff >=0 ? diff*2 : -diff*2 -1   ) ;

		#ifdef DEBUG
			fprintf(stderr, "V[-1]=%d V[0]=%d DIFF=%d ENC=%d\n", (int)stream[i-1], (int)stream[i], diff, (int)encdiff);
		#endif

		uint8_t lsbValue=((uint8_t)encdiff) & lsb;
		uint8_t msbValue=(((uint8_t)encdiff) & msb) >> prm;
		
		#ifdef DEBUG
			fprintf(stderr, "EncDiff=%02x   MSB=%02x(%3d)   LSB=%02x\n", (int)encdiff, msbValue, msbValue, lsbValue);
		#endif
		
		// msbValue is alpha encoding:  0 for '1', 1 for '01', 2 for '001', ... 0xf for '00000000000000001'
		// This is simply implemented by 
		while(msbValue >= 8){
			bBuf.appendLBits(0, 8);
			msbValue-=8;
		}
		bBuf.appendLBits(1, msbValue+1);
		
		// lsbValue is simply binary coded
		bBuf.appendLBits(lsbValue, prm);      // NOTE:  bBuf.appendLBits(stream[i], prm) should also work.

		#ifdef DEBUG
			bBuf.showAllBuffer(stderr);
		#endif
	}

	bBuf.paddingToBoundary(0);  // Append padding.

	compressedSize=bBuf.getSize();
	return bBuf.getBuffer();

}