/**@file saradecomp.cpp
   @brief Sample file of the rice-decompression of SARA.

   The input file should be the name of C.TXT, which should be a continuous
   CENA or SWIM packet starting "AA55 AA55".
   The format of the input file should be "xxxx xxxx xxxx xxxx xxxx "
   in one line (no \\n should be placed... The followings are ignored).

 */

/*
  Rice decompression software 
  Reads the 16-bit word from the file c.txt.
  Featches for the header AA55AA55 till the end of the file and decompresses the data.
  Each rice packet with the header AA55AA55 contains the compressed data for the 512 samples
  Once the header is found, starts decompressing the data for 512 samples.
  In case of sync lost, that is if a header is found before decompressing the 512 samples,
  decompression starts for the new packet. Old packet which is incomplete,filled with FFFF for 
  the remaining samples.Rice decompressed output is 8 bits only. If anywhere in the output file
  the 16 bit data FFFF is found, it means that the particular packet has some errors.
  The output file is D.txt
*/
#define DEBUG_DCMP

#include"saradecomp.h"
using namespace IrfPac;
using namespace IrfPac::SaraPac;

int main(void){
	// 80000 should be modified more efficient way.  80000 comes from just the test input file.
    char input_string[80000];

	FILE* fp1=fopen("c.txt", "r");
	if(fp1==NULL){
		fprintf(stderr, "Failed to open file c.txt\n");
		exit(-10);
	}
	fgets(input_string, 80000, fp1);
	fclose(fp1);
	
	int size=strlen(input_string);

	fprintf(stdout, "INPUT SIZE=%d (%d byte)\n", size, size/5*2);

	Decompression* decomp=new Decompression();

	size_t output_length;

 	uint8_t* buff=decomp->decompressionAll(input_string, size, &output_length);	

        fprintf(stdout, "OUTPUT SIZE=%d byte\n", output_length);

	FILE* fp2=fopen("d.txt","w");
	for(int i=0; i<output_length; i++){
		fprintf(fp2, "0x%x\n", buff[i]);
	}
    fclose(fp2);  
	
	delete[] buff;
	delete decomp;

	return 0;

}

