#ifndef IRFPAC_SARAPAC_SARADECOMPRESSION_H
#define IRFPAC_SARAPAC_SARADECOMPRESSION_H

/** @mainpage

    @par
    SARA rice decompression routine.
    The implementation is done in the header file, 'saradecomp.h'.
    You can use just including the header file to use it.

    @par
    Usage is very simple.  First you need to instance the
    IrfPac::SaraPac::Decompression class, and use the
    decompressionAll() method.

    @par
    Sample code is in saradecomp.cpp file

    @author
    Y. Futaana (note that I refactored VSSC decompression sample to be
    C++ class.)

 */

#include<iostream>
#include<stdlib.h>
#include<string.h>
#include<inttypes.h>
#include<math.h>
#include<vector>

#include<stdexcept>

namespace IrfPac{
namespace SaraPac{

  /** @brief Decompression class
   */
class Decompression {
	
	public:
  /** @brief Only the constructor.  Just initialize the member variables.
   */

	Decompression(){
	
		unsigned long lut11[18]={0x0,0x1,0x7,0xe,0x1c,0x38,
    	                    0x70,0xE0,0x1C0,0x380,0x700,
							0xE00,0x1C00,0x3800,0x7000,
							0xE000,0x1C000,0x38000};

		unsigned long lut21[16]={0x1,0x2,0x4,0x8,0x10,0x20,0x40,0x80,
                        0x100,0x200,0x400,0x800,0x1000,0x2000,0x4000,0x8000};
		unsigned long lut31[6]={0x0,0x1,0x3,0x7,0xF,0x1F};

		for(int i=0; i<18; i++)lut1[i]=lut11[i];
		for(int i=0; i<16; i++)lut2[i]=lut21[i];
		for(int i=0; i<6; i++)lut3[i]=lut31[i];


		val=temp_val=val1=val2=0;
		for(int i=0; i<16; i++)temp[i]=0;
		idx=one_count=zero_count=append_count=0;
		pos=kID=0;
		append=0;
		one_more=1;
		for(int i=0; i<512; i++) sample[i]=0;
		zero_block_flg=0;
		y_value=dummy=x_value=p_value=0;
		q_value=0;
		flag=0;
		sync_lost=y_sync_lost=0;
	
	}

private:
int ref_block;
unsigned long lut1[18];

unsigned long lut2[16];
unsigned long lut3[6];

unsigned long val,temp_val,val1,val2;
int temp[16];
int idx,one_count,zero_count,append_count;
int pos,kID;
unsigned long append;
int one_more;
int sample[512];
int zero_block_flg;
int y_value,dummy,x_value,p_value;
unsigned long q_value;
int flag;
int sync_lost, y_sync_lost;

// Below is defined by Futaana, and initialized in decompressionAll()
char *sp1;    // They are used as input and output.
size_t len, end_len;
std::vector<uint8_t> obuf;

void read_val()
{
  if(!sync_lost)
    { 
      if(flag==0)
	{
	  next();
	  if(!seof())
	    {
	      //printf("%lx",val);
	      // check if it is the header
	      if(val==0xAA55)
		{
		  // read the second word store it in val1
		  next();
		  if (val1==0xAA55 && y_value<512)
		    {
		      // header found 
		      sync_lost=1;
		      y_sync_lost=y_value;
		      val=0xFFFF;   
		    }
		  else
		    flag=1; 

		}
	    }
	  else // if end of file
	    {

	      if(y_value<512)
		{
		  for(x_value=0;x_value<y_value;x_value++){
			save(sample[x_value]);
		  }
		  for(x_value=y_value;x_value<512;x_value++){
			save(0xffff);
		  }

		}
	      //printf("reached End of file");
	      throw std::runtime_error("reached end of file");
	    }   

	}
      else // if flag==1
	{
	  val=val1;
	  flag=0;
	}
    }
  else // if sync lost
    {
      val=0xFFFF;
    }



}


void find_option(void)
{
  if(pos>2)
    {
      kID=((val & lut1[pos])>>(pos-2));
      pos=pos-3;
    }
  else if(pos==2)
    {
      kID=((val & lut1[pos])>>(pos-2));
      read_val();
      pos=15;
    }
  else
    {
      temp_val=val<<16;
      read_val(); // read the new value
      val=val | temp_val;
      pos=pos+16;
      kID=((val & lut1[pos])>>(pos-2));
      pos=pos-3;
    }
  if(kID==0)
    pos=pos-1;


}

void decompK(void)
{
  while(one_count<(16-ref_block))
    {
      if(pos>=0)
	{
	  if((val & lut2[pos])==0)
	    {
	      zero_count=zero_count+1;
	      pos=pos-1;
	    }
	  else
	    {
	      temp[idx++]=zero_count;
	      zero_count=0;
	      one_count=one_count+1;
	      pos=pos-1;
	    }
	}

      else
	{
	  pos=15;
	  read_val();
	}

    }// end of while
}

void appendLSB(void)
{
  while(append_count < (16-ref_block))
    {
      if((pos+1)>=(kID-1))
	{
	  // append=(val & (lut3[kID-1]<<(pos-kID+2))>>(pos-(kID-2)));
	  append=lut3[kID-1]<<(pos-kID+2);
	  append=append & val;
	  append=(append >> (pos-(kID-2))) & lut3[kID-1];
	  temp[idx++]=((temp[idx]<<(kID-1)) | append) & 0xFF; //???????
	  append_count=append_count+1;
	  pos=pos-(kID-1);
	}
      else
	{
	  temp_val=val <<16;
	  read_val();
	  val=val | temp_val;
	  pos=pos+16;
	}

    }// end of while


}

void decomp_zeroblk(void)
{
  while(zero_block_flg==0)
    {
      if(pos>=0)
	{
	  if((val & lut2[pos])==0)
	    {
	      zero_count++;
	      pos=pos-1;
	    }
	  else
	    {
	      zero_block_flg=1;
	      pos=pos-1;

	    }

	}
      else
	{
	  read_val();
	  pos=15;
	}
    }


}

void decomp_nocomp(void)
{
  while(one_count< (16-ref_block))
    {
      if(pos>7)
	{
	  temp[idx++]=((val & ((0xFF << (pos-7))))>>(pos-7)) & 0xFF;
	  //printf("%x\n",((val & ((0xFF << (pos-7))))>>(pos-7)) & 0xFF);
	  pos=pos-8;
	  one_count++;
	}
      else if(pos==7)
	{
	  temp[idx++]=((val & ((0xFF << (pos-7))))>>(pos-7)) & 0xFF;
	  //printf("%x\n",((val & ((0xFF << (pos-7))))>>(pos-7)) & 0xFF);
	  pos=15;
	  read_val();
	  one_count++;

	}  
      else
	{
	  temp_val=val << 16;
	  read_val();
	  val=val | temp_val;
	  pos=pos+16;
	}

    }

}

int min(int x)
{
  if(x > (255-x))
    return(255-x);
  else
    return(x);
}

void post_process(void)
{

  int k=0;
  int theta,delta[48];

  if(ref_block==1)
    {

      delta[0]=temp[0];
      sample[y_value]=delta[0];
      y_value=y_value+1;
      for (k=1;k<=15;k++)
	{
	  theta=min(sample[y_value-1]);
	  if(temp[k] <= 2*theta)
	    {
	      if(temp[k]%2==0)
		delta[k]=temp[k]/2;
	      else
		delta[k]=-1*(temp[k]+1)/2;
	    }
	  else
	    {
	      if (theta==sample[y_value-1])
		delta[k]=temp[k]-theta;
	      else
		delta[k]=theta-temp[k];
	    }

	  sample[y_value]=sample[y_value-1]+delta[k];
	  y_value++;

	}

    }
  else if(ref_block==0)
    {
      theta=min(sample[y_value-1]);
      if(temp[0] <= 2*theta)
	{
	  if(temp[0]%2==0)
	    delta[0]=temp[0]/2;
	  else
	    delta[0]=-1*(temp[0]+1)/2;
	}
      else
	{
	  if (theta==(sample[y_value-1]))
	    delta[0]=temp[0]-theta;
	  else
	    delta[0]=theta-temp[0];
	}
	if(y_value < 512){
      sample[y_value++]=sample[y_value-1]+delta[0];
	}

      for (k=1;k<=15;k++)
	{
	  theta=min(sample[y_value-1]);
	  if(temp[k] <= 2*theta)
	    {
	      if(temp[k]%2==0)
		delta[k]=temp[k]/2;
	      else
		delta[k]=-1*(temp[k]+1)/2;
	    }
	  else
	    {
	      if (theta==sample[y_value-1])
		delta[k]=temp[k]-theta;
	      else
		delta[k]=theta-temp[k];
	    }

	if(y_value < 512){
	  sample[y_value++]=sample[y_value-1]+delta[k];
	}

	}
    }
  dummy=0;

}

void look_header(void)
{

  // read the value
  //read_val();
  next();
  if(val==0xAA55)
    {
     next();
      // read the second one
      if(val==0xAA55)
	{
	  one_more=1;
	}
      else
	{
	  one_more=0;
	}
    }
  else
    {
      one_more=0;
      next();
      if(val==0xAA55)
	{
      next();
	  // read the second one
	  if(val==0xAA55)
	    {
	      one_more=1;
	    }
	  else
	    {
	      one_more=0;
	    }
	}
    }

} 

/** @brief Check the string is end or not.  Use instead of feof(fp1)
*/
bool seof(){
	return len >= end_len;
}

/** @brief read next word.  Set to a read value to val.
*/
void next(){
      sscanf(sp1, "%lx", &val);
      sp1+=5;
      len+=5;
#ifdef DEBUG
	  printf("Next: %x at %d\n", (int)val, (int)len);
#endif
}

void save(int value){
	obuf.push_back(value);
}

public:
/** @brief Decompress the specified input_stream.

	Decompression is conducted for the given input_stream, which is char*.
	The length of the characters should be specified.
	The input stream should be a character array (as in C case) not a bynary array.
	The format should strictly follow 4 characters and one space.
	
	The output is an array of the uint8_t.  The number of element is variable.
	Pointer to output_length can fetch the number.
	The obtained array should be released manually after the usage by delete[] explicitly.

	If decompression is failed, std::runtime_error is returned.
*/
uint8_t* decompressionAll(char* input_stream, size_t input_length, size_t* output_length)  // input_length is the length of the characters!!
{
	static char header[]="aa55 aa55";

	#ifdef DEBUG_DCMP
	fprintf(stderr, "###  decompressionAll called\n");
	#endif

	// Algorithm for decompression changed.
	
	// Try to sprit the input stream by header of AA55AA55.

	char* pStartHeader=input_stream;  // The pointer to specify the header position of input stream

	*output_length=0;   // Initialize output_length

	// List of pointers to header of each block.
	std::vector<size_t> headerPos;

	// Look for the header.
	while( ( pStartHeader = strcasestr(pStartHeader, header)) != NULL ){
		#ifdef DEBUG
			fprintf(stdout, "INPUT STREAM: %p ", input_stream);
			for(int i=0; i<20; i++)fprintf(stdout, "%c", input_stream[i]);
			fprintf(stdout, "\n");
			fprintf(stdout, "HEADER STREAM: %p ", pStartHeader);
			for(int i=0; i<20; i++)fprintf(stdout, "%c", pStartHeader[i]);
			fprintf(stdout, "\nBLOCK INFO: 0x%x (=%d) offset\n", pStartHeader-input_stream, pStartHeader-input_stream);
			fprintf(stdout, "---\n");
		#endif

		size_t offset=  (size_t) (pStartHeader-input_stream);
		if(offset >= input_length) break;  // strcasestr depends on '\0' while this function should consider input_length, too.
		headerPos.push_back( offset);

		pStartHeader += 10;    // Skip  10  characters 'aa55 aa55'
	}

	int nrBlock=headerPos.size();


	#ifdef DEBUG_DCMP
		fprintf(stderr, "Nr of block = %d\n", nrBlock);
	#endif
	
	headerPos.push_back(input_length);	// Append the value of 'end pointer'.

	// The main routine.  Each block will be decompressed.
	
	std::vector<uint8_t> oBuf;
	
	for(int i=0; i<nrBlock; i++){
		size_t offset=headerPos.at(i);
		size_t length=headerPos.at(i+1)-offset;
		pStartHeader = input_stream+offset;

		// I decided to make a copy of the stream.
		char* inputstr=new char[length+1];
		for(unsigned int j=0; j<length; j++)inputstr[j] = pStartHeader[j];
		inputstr[length]='\0';
		
		length++;

		#ifdef DEBUG_DCMP
		fprintf(stderr, "%4d | ", (int)length);
		for(size_t j=0; j<length; j++){
		  fprintf(stderr, "%c", inputstr[j]);
		}
		#endif

		#ifdef DEBUG
			fprintf(stdout, "#%d-th block\n", i);
			fprintf(stdout, "\tOffset:%d\tSize:%d\t", (int)offset, (int)length);
			for(int j=0; j<20; j++)fprintf(stdout, "%c",pStartHeader[j]);
			fprintf(stdout, "\n");
		#endif

		size_t outLen=0;
//		uint8_t* decmpBlock=decompressionMain(pStartHeader, length, &outLen);
		try{

			uint8_t* decmpBlock=decompressionMain(inputstr, length, &outLen);

////			fprintf(stdout, "MARKER3 size=%d\n", outLen);

			*output_length += outLen;


			#ifdef DEBUG_DCMP
			{
			int zeros=0;
			fprintf(stderr, " =>  %5d | ", outLen);
			for(unsigned int j=0; j<outLen; j++){
			  if(decmpBlock[j]==0){
			    zeros++;
			  }
			  else{
			    if(zeros!=0)fprintf(stderr, "0*[%d] ", zeros);
			    fprintf(stderr, "%x ", decmpBlock[j]);
			    zeros=0;
			  }
			}
			if(zeros!=0){
			  fprintf(stderr, "0*[%d] ", zeros);
			}
			fprintf(stderr, "\n");
			}
			#endif

#ifdef DEBUG
			fprintf(stdout, "Decompression done: Size %d (total %d)\n", (int)outLen, (int)(*output_length));
#endif
			
			for(unsigned int j=0; j<outLen; j++){
#ifdef DEBUG
				fprintf(stdout, "RES: %d %x\n", j, decmpBlock[j]);
#endif
				
				oBuf.push_back(decmpBlock[j]);
			}
			
			delete[] inputstr;
			delete[] decmpBlock;
			cleanup();


		}
		catch(const std::runtime_error& exc){
			std::cerr << "Compression failed : " << exc.what() << std::endl;
			// TODO a step for memory leak is needed.

			throw exc;
		}

	}

	// Post process.  Copy the buffered output stream.
	uint8_t* output_stream=new uint8_t[oBuf.size()];

#ifdef DEBUG
	fprintf(stdout, "%d\n", (int)oBuf.size());
#endif

	for(unsigned int i=0; i<oBuf.size(); i++){
		output_stream[i] = oBuf.at(i);
	}

	#ifdef DEBUG
		fprintf(stdout, "Returned address = %p\n", output_stream);
	#endif
	
	return output_stream;

}

private:
uint8_t* decompressionMain(char* input_stream, size_t input_length, size_t* output_length)  // input_length is the length of the characters!!
{

  one_more=0;
  sp1=input_stream;
  len=0;
  end_len=input_length;
  
#ifdef DEBUG
  printf("%d\n", seof());
#endif
	// first read for the header
	while((!one_more) && (!seof()))
	{
		next();
		if(val==0xAA55)
		{
			next();
			if(val==0xAA55)
				one_more=1;
			else
				one_more=0;
		}
		else
		{
			one_more=0;
		}
	}
	if(!one_more)
	{
		printf(" No header Found");
		throw std::runtime_error("No header Found");
	}
	while(one_more)
	{

		//printf("found header");
		read_val();
		pos=15;
		ref_block=1;
		while(y_value<512)
		{
			
			find_option();
			
			if(ref_block==1)
			{
				idx=0;
				one_count=14;
				decomp_nocomp(); // to read the refference sample
			}
			else
			{
				idx=0;
				one_count=0;
			}
			if(kID==0)
			{
				//one_count=0; 
				zero_count=0;
				zero_block_flg=0;
				decomp_zeroblk();
				zero_block_flg=0;
				if(ref_block==1)
				{
					//fill first block with refference
					for(x_value=1;x_value<=15;x_value++)
						temp[idx++]=0;
					post_process();
					zero_count--;
					idx=0;
					ref_block=0;
					for(x_value=zero_count;x_value>0;x_value--)
					{
						for(p_value=0;p_value<=15;p_value++)
							temp[idx++]=0;
						post_process();
						idx=0;
					}
				}
				else
				{
					for(x_value=zero_count;x_value>0;x_value--)
					{
						for(p_value=0;p_value<=15;p_value++)
							temp[idx++]=0;
						post_process();
						idx=0;
						
					}
					
				}
			}
			else if(kID==1)
			{
				one_count=0;
				decompK();
				post_process();
			}
			else if(kID==7)
			{
				one_count=0;
				decomp_nocomp();
				post_process();
			}
			else
			{
				
				one_count=0;
				decompK();
				if(ref_block==1)
					idx=1;
				else
					idx=0;
				append_count=0;
				appendLSB();
				post_process();
				
			}
			ref_block=0;
			zero_count=0;
			one_count=0;
			idx=0;

			
		}
		if(!sync_lost)
		{
			for(x_value=0;x_value<512;x_value++)
			{
				save(sample[x_value]);
			}
			look_header();
		}

		else
		{
			// already header found
			for(x_value=0;x_value<y_sync_lost;x_value++){
				save(sample[x_value]);
			}
			for(x_value=y_sync_lost;x_value<512;x_value++){
				save(0xffff);
			}
			one_more=1;
			sync_lost=0; 
			y_sync_lost=0;
		}



		ref_block=1;
		y_value=0;
		one_count=0;
		zero_count=0;
		idx=0;
	}
	//printf("completed");
	//getch();
	
	int olength=obuf.size();
	*output_length=obuf.size();
	
	uint8_t* intbuf=new uint8_t[olength];
	for(size_t i=0; i<*output_length; i++){
		intbuf[i] =(uint8_t)(obuf.at(i));
	}
	
	return intbuf;
};

void cleanup(){
	y_value=0;
	obuf.clear();
}


public:
	double getVersion(){
		return 1.0;
	}

};

}} //end namespace

#endif
