/** @brief A sample class of decompression library using in Java
 */
public class Main{

	// You always have to use static block to call JNI library.
	//
	static{
		System.loadLibrary("saradecompjava");
	}

	public static void main(String[] argv){
		System.out.println("Sample of Decompression");

		// Decompression class is instanced.
		Decompression dec=new Decompression();

		// Calling getVersion() method to obtain version info.
		System.out.println("Version="+dec.getVersion());

		// This is an compressed stream as a input of hex dump
		// string format.
		//
		String input = "aa55 aa55 2017 ac46 469a ae2a b88e ffff 0042 0000 c901 4e00 0422 2fea 4822 3eff ff29 c011 9420 00c6 0194 2024 6970 1005 f252 fb48 b082 6522 daaa 2254 abd4 a105 2080 af08 0dac 97a2 288a 474d 7923 8896 6584 1960 a2e5 6594 b4b4 bd16 a820 3c96 5808 002d bcf2 91bd 4291 fee0 8091 4d22 080d 7f95 ebf6 9375 ffcd 7f11 11e7 0401 7917 1109 04b2 37c0 8039 6b11 d6b9 fcbb 74ed 7a97 9410 148b f610 12ea 1048 41cd 9f55 6d64 8840 6c47 fa5f f56a 2000 0000 aa55 aa55 2031 05d7 5083 3d44 82d5 4a7c a588 fcae a597 a9eb a101 4415 4e6c aea8 89af fc84 14ab 2f82 8d77 fff9 9201 8c41 0046 1180 2101 54bd b622 76ea 3192 27a1 07d7 ceb6 d579 11ff fc88 f112 d2d1 2250 82ba af3a e845 91e5 97a6 62a9 294b 288e 54b2 295a b34c d5dd e120 c44b 5f11 4e10 6111 df9e a040 19b5 cff5 9551 23e4 55ad 4fe5 db29 2a88 f115 ceff ff18 0220 80a9 1948 5000 0237 f288 8405 2420 e48c 4548 c965 95bd 4800 0000 aa55 aa55 203a ec81 0057 25f1 1082 8455 2841 b2f4 4732 ae47 7cd5 72f2 977f ff80 2109 d610 0400 0046 509f e23c 41a6 a97d 11c9 8a23 594b cc46 b96a a96a 1055 2324 1922 5e82 0ac2 292f ac41 659f a080 f559 9154 9422 9725 511e 4100 5e6a 97fa cf91 645c 2064 0075 fe08 0d37 d357 ce23 5b7f bfff c010 8021 0004 121c 2132 77ff f802 1084 6311 9211 0c20 2105 a104 88c7 912c bd44 1f4f d2a8 ad9d 8416 6454 923e b210 1669 97d7 ae6d a22d 0401 72bd 57f7 7fff 90e1 2142 5480 8500 4200 0000 ";

		// SWIGTYPE_p_size_t is a class defined by SWIG as a bridge
		// of the pointer to size_t.  You have to prepare the
		// pointer to the size_t in order to call decompressionAll().
		//
		SWIGTYPE_p_size_t newSize=DecompressionSwig.new_sizeTptr();

		// You can call decompressionAll() method in order to obtain
		// decompressed packet.  The returned packet, in C++ version
		// which is uint8_t*, is a wrapper class
		// SWIGTYPE_p_unsigned_char.
		// The method decompressionAll() also sets the decompressed
		// stream size to the third argument.
		//
		SWIGTYPE_p_unsigned_char decomp = dec.decompressionAll(input, input.length(), newSize);

		// Obtain the contents of newSize.
		// newSize is an instance of SWIGTYPE_p_size_t, so that
		// DecompressionSwig.sizeTptr_value() method is needed to
		// get a contents.
		//
		int size=(int)DecompressionSwig.sizeTptr_value(newSize);
		System.out.println("New size = "+size);

		for(int i=0; i<size; i++){

		    // Now you can obtain the decompressed packet stream.
		    // The type of the returned packet is SWIGTYPE_p_unsigned_int,
		    // so you need to use DecompressionSwig.uint8arr_getitem()
		    // method to obtain the contents.
		    //
		    System.out.println("Value["+i+"] = "+DecompressionSwig.uint8arr_getitem(decomp, i));
		}

	}

}
